/*
 * Created on 10.05.2005
 */
package org.sfb441.tokenizer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;

/**
 * GUI for different tokenizer modules created with JFlex.
 * 
 * @author wolfgang
 */
public class GUI extends JFrame {

	private static final String VERSION = "Tokenizer SFB 441 B14 - Version 0.0.9";

	@SuppressWarnings("unused")
	private static final String AUTHOR = "wmaier@sfs.uni-tuebingen.de & malisi@informatik.uni-tuebingen.de";

	// command constants
	private static final String TOKENIZE_GENERAL = "tokenize general";
	
	private static final String TOKENIZE_OS = "tokenize old spanish";

	private static final String TOKENIZE_SV = "tokenize surselvisch";

	private static final String TOKENIZE_OS_S = "tokenize old spanish syllables (use dictionary)";
	
	private static final String TOKENIZE_OS_S_ND = "tokenize old spanish syllables (don't use dictionary)";
	
	private static final String ELIMINATE = "eliminate angled brackets";

	private static final String LINEBREAKS = "eliminate linebreaks";

	private static final String XMLIFY = "create xml output";

	private static final String SEPARATOR = "---";

	// GUI stuff
	private JTextArea msgPane;

	private JTextField inputFileTextField;

	private JTextField outputFileTextField;

	private JComboBox commandBox;

	private JCheckBox outputToPaneCheckbox;

	private File currentDir = null;

	/*
	 * GUI Objekt erzeugen, Frame (self) anzeigen
	 */
	private GUI() {
		this.setTitle("Tokenizer");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setContentPane(createContentPane());
		this.setSize(400, 600);
		this.pack();
		this.setLocationRelativeTo(null);
		this.setVisible(true);
	}

	private void tokenize(File inputFile, File outputFile, String dictionary) {
		try {
			// message
			msgPane.append(inputFile.getName() + " --> " + outputFile.getName()
					+ ": Tokenize\n");
			int tokenCount = 0;
			// tokenisieren
			BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
			FileReader in = new FileReader(inputFile);
			Class tokenizerClass = Class.forName("org.sfb441.tokenizer.Tokenizer" + dictionary);
			Class[] paramClasses = {Reader.class};
			Constructor tokenizerConstructor = tokenizerClass.getDeclaredConstructor(paramClasses);
			Object[] params = {in};
			Object scanner = tokenizerConstructor.newInstance(params);
			Class[] noParamClasses = {};
			Object[] noParamObjects = {};
			//scanner = new TokenizerGeneral(in);
			Method yylex = scanner.getClass().getDeclaredMethod("yylex", noParamClasses);
			String buf = "";//(String)yylex.invoke(scanner, noParamObjects);
			while ((buf = (String)yylex.invoke(scanner, noParamObjects)) != null) {
				out.write(buf);
				if (outputToPaneCheckbox.isSelected()) {
					msgPane.append(buf);
				}
				if(buf != ""){
					++tokenCount;  // count tokens
				}
			}
			in.close();
			out.close();
			msgPane.append("finished! Found " + String.valueOf(tokenCount)
					+ " tokens.\n");
		} catch (ClassNotFoundException e1) {			
			e1.printStackTrace();
		} catch (SecurityException e) {			
			e.printStackTrace();
		} catch (NoSuchMethodException e) {			
			e.printStackTrace();		
		} catch (FileNotFoundException e) {
			msgPane.append("[error] File not found : \"" + inputFile.getName()
					+ "\"\n");
			return;
		} catch (IOException e) {
			msgPane.append("[error] IO error scanning file \""
					+ inputFile.getName() + "\"\n");
			msgPane.append(e + "\n");
			return;
		} catch (Exception e) {
			msgPane.append("[error] Unexpected exception.\n");
			// e.printStackTrace();
			return;
		} catch (Error e) {
			msgPane.append("[error] Scan error: " + e.getMessage() + "\n");
			// e.printStackTrace();
			return;
		}
	}

	
	/*
	 * alle '<' und '>' in inputFile loeschen, Ergebnis in outputFile
	 * schreiben. (JFlex).
	 */
	private void deleteCharacters(File inputFile, File outputFile) {
		msgPane.append(inputFile.getName() + " --> " + outputFile.getName()
				+ ": Delete angled brackets\n");
		Eliminator scanner = null;
		String buf = null;
		int deleteCount = 0;
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
			FileReader in = new FileReader(inputFile);
			scanner = new Eliminator(in);
			while ((buf = (String) scanner.yylex()) != null) {
				out.write(buf);
				if (outputToPaneCheckbox.isSelected()) {
					msgPane.append(buf);
				}
				if (buf.equals(""))
					++deleteCount; // geloeschte Klammern zaehlen
			}
			in.close();
			out.close();
		} catch (FileNotFoundException e) {
			msgPane.append("[error] File not found : \"" + inputFile.getName()
					+ "\"\n");
			return;
		} catch (IOException e) {
			msgPane.append("[error] IO error scanning file \""
					+ inputFile.getName() + "\"\n");
			msgPane.append(e + "\n");
			return;
		} catch (Exception e) {
			msgPane.append("[error] Unexpected exception.\n");
			// e.printStackTrace();
			return;
		} catch (Error e) {
			msgPane.append("[error] Scan error: " + e.getMessage() + "\n");
			// e.printStackTrace();
			return;
		}
		msgPane.append("finished! " + String.valueOf(deleteCount)
				+ " brackets deleted.\n");
	}

	/*
	 * alle zeilenumbrueche loeschen bzw. mit " " ersetzen. (JFlex)
	 */
	private void killLineBreaks(File inputFile, File outputFile) {
		msgPane.append(inputFile.getName() + " --> " + outputFile.getName()
				+ ": kill linebreaks\n");
		if (outputToPaneCheckbox.isSelected()) {
			msgPane.append("(output won't be visible)\n");
		}
		Linebreaks scanner = null;
		String buf = null;
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
			FileReader in = new FileReader(inputFile);
			scanner = new Linebreaks(in);
			while ((buf = (String) scanner.yylex()) != null) {
				out.write(buf);
			}
			in.close();
			out.close();
		} catch (FileNotFoundException e) {
			msgPane.append("[error] File not found : \"" + inputFile.getName()
					+ "\"\n");
			return;
		} catch (IOException e) {
			msgPane.append("[error] I/O error scanning file \""
					+ inputFile.getName() + "\"\n");
			msgPane.append(e + "\n");
			return;
		} catch (Exception e) {
			msgPane.append("[error] Unexpected exception.\n");
			// e.printStackTrace();
			return;
		} catch (Error e) {
			msgPane.append("[error] Scan error: " + e.getMessage() + "\n");
			// e.printStackTrace();
			return;
		}
		msgPane.append("finished! linebreaks deleted.\n");
	}

	/*
	 * XML erzeugen aus tokenisiertem korpus
	 */
	private void xmlify(File inputFile, File outputFile) {
		msgPane.append(inputFile.getName() + " --> " + outputFile.getName()
				+ ": create XML corpus\n");
		if (!outputFile.getName().toLowerCase().endsWith(".xml")) {
			msgPane.append("output filename must end with '.xml'!\n");
			return;
		}
		if (outputToPaneCheckbox.isSelected()) {
			msgPane.append("(output won't be visible)\n");
		}

		int lineCounter = 0;
		// XMLifier macht XML
		Xmlifier xmlifier = new Xmlifier(inputFile, outputFile);
		try {
			// lass ihn arbeiten
			lineCounter = xmlifier.xmlify();
		} catch (IOException e) {
			msgPane.append("[error] I/O Exception\n");
			return;
		} catch (TokenizerException e) {
			msgPane.append("[error] Could not xmlify corpus: " + e.getMessage()
					+ "\n");
			// e.printStackTrace();
			return;
		}
		msgPane.append("finished! " + lineCounter + " lines xmlified.\n");
	}

	/*
	 * aktionen starten, parameter vorher checken
	 */
	private void runAction(String command) {
		if (msgPane.getText().length() > 0) {
			msgPane.append("\n");
		}
		// System.err.println("x\\f\\t\\r\\nx: x\f\t\r\nx");
		msgPane.setCaretPosition(msgPane.getText().length());
		Date d = new Date(System.currentTimeMillis());
		msgPane.append("Starting session at " + d.toString() + "...\n");
		// input file
		File inputFile = new File(inputFileTextField.getText());
		if (inputFileTextField.getText().length() == 0) {
			msgPane.append("[error] please specify an input file!\n");
			return;
		}
		// output file
		if (outputFileTextField.getText().length() == 0) {
			msgPane.append("[error] please specify an output file.\n");
			return;
		}
		File outputFile = new File(outputFileTextField.getText());
		if (outputFile.exists()) {
			int n = JOptionPane.showConfirmDialog(null,
					"Overwrite output file?", "Please confirm",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.NO_OPTION) {
				msgPane.append("Canceled!\n");
				return;
			}
		}
		if (outputFile.equals(inputFile)) {
			msgPane
					.append("[error] Output file and input file may not be identical.\n");
			return;
		}

		// aktion bestimmen
		if (TOKENIZE_GENERAL.equals(command)){
			tokenize(inputFile, outputFile, "General");
		} else if (TOKENIZE_OS.equals(command)) {
			tokenize(inputFile, outputFile, "OldSpanish");
		} else if(TOKENIZE_OS_S.equals(command)){
			tokenize(inputFile, outputFile, "OldSpanishSyllablesDict");
		}else if(TOKENIZE_OS_S_ND.equals(command)){
			tokenize(inputFile, outputFile, "OldSpanishSyllablesNoDict");
		}else if (TOKENIZE_SV.equals(command)) {				
			tokenize(inputFile, outputFile, "Surselvisch");
		} else if (ELIMINATE.equals(command)) {
			deleteCharacters(inputFile, outputFile);
		} else if (LINEBREAKS.equals(command)) {
			killLineBreaks(inputFile, outputFile);
		} else if (XMLIFY.equals(command)) {
			xmlify(inputFile, outputFile);
		}
	}

	/*
	 * content pane bevoelkern
	 */
	private JPanel createContentPane() {
		// vorbereitung
		// buttons
		String[] commandList = { TOKENIZE_GENERAL, TOKENIZE_OS, TOKENIZE_OS_S, TOKENIZE_OS_S_ND,  TOKENIZE_SV, SEPARATOR, XMLIFY,
				SEPARATOR, ELIMINATE, LINEBREAKS };
		commandBox = new JComboBox(commandList);
		commandBox.setSelectedIndex(0);
		JButton runButton = new JButton("run action");
		runButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Object o = commandBox.getSelectedItem();
				if (o != null && !SEPARATOR.equals((String) o)) {
					runAction((String) o);
				}
			}
		});
		JButton clearButton = new JButton("clear output");
		clearButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				msgPane.setText("");
			}
		});

		// text fields
		inputFileTextField = new JTextField(10);
		outputFileTextField = new JTextField(10);

		// content pane zusammensetzen
		JPanel ctrlPane = new JPanel();
		ctrlPane.setLayout(new BoxLayout(ctrlPane, BoxLayout.PAGE_AXIS));

		// toolbar
		JToolBar toolbar = new JToolBar();
		toolbar.setLayout(new BoxLayout(toolbar, BoxLayout.LINE_AXIS));
		toolbar.add(commandBox);
		toolbar.addSeparator(new Dimension(1, 1));
		toolbar.add(runButton);
		toolbar.addSeparator();
		toolbar.add(clearButton);
		toolbar.setBorder(BorderFactory.createRaisedBevelBorder());
		ctrlPane.add(toolbar);

		// input file selection
		JPanel filePane = new JPanel();
		filePane.setLayout(new BoxLayout(filePane, BoxLayout.LINE_AXIS));
		filePane.setBorder(BorderFactory.createEmptyBorder(6, 1, 1, 1));
		filePane.add(new JLabel(" Input File: "));
		filePane.add(inputFileTextField);
		JButton loadFileButton = new JButton("browse");
		loadFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();
				if (currentDir != null)
					fc.setCurrentDirectory(currentDir);
				if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
					inputFileTextField.setText(fc.getSelectedFile()
							.getAbsolutePath());
					currentDir = fc.getCurrentDirectory();
				}
			}
		});
		filePane.add(loadFileButton);
		ctrlPane.add(filePane);

		// output file selection
		filePane = new JPanel();
		filePane.setLayout(new BoxLayout(filePane, BoxLayout.LINE_AXIS));
		filePane.setBorder(BorderFactory.createEmptyBorder(1, 1, 6, 1));
		filePane.add(new JLabel(" Output File: "));
		filePane.add(outputFileTextField);
		loadFileButton = new JButton("browse");
		loadFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();
				if (currentDir != null)
					fc.setCurrentDirectory(currentDir);
				if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
					outputFileTextField.setText(fc.getSelectedFile()
							.getAbsolutePath());
					currentDir = fc.getCurrentDirectory();
				}
			}
		});
		filePane.add(loadFileButton);
		ctrlPane.add(filePane);

		// output to text pane -> checkbox
		filePane = new JPanel(new BorderLayout());
		filePane.setBorder(BorderFactory.createEmptyBorder(1, 1, 6, 1));
		outputToPaneCheckbox = new JCheckBox("Write output to text pane", false);
		filePane.add(outputToPaneCheckbox);
		ctrlPane.add(filePane);

		// control pane fertig, jetzt JTextArea fuer die meldungen.
		//JPanel msgPanel = new JPanel();
		//msgPanel.setOpaque(true);
		msgPane = new JTextArea();
		msgPane.setEditable(false);				
		JScrollPane msgPaneScroll = new JScrollPane(msgPane);
		msgPaneScroll.setPreferredSize(new Dimension(400, 400));		
		// info bar mit Versionsinfo
		JLabel infoLabel = new JLabel(" " + VERSION);
		infoLabel.setToolTipText("Click for version history");
		infoLabel.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {
				msgPane.setText("");
				BufferedReader br = new BufferedReader(new InputStreamReader(
						getClass().getResourceAsStream("history")));
				String tmp = null;
				try {
					while ((tmp = br.readLine()) != null) {
						msgPane.append(tmp + "\n");
					}
				} catch (Exception e) {
					msgPane.append("Could not load version history!");
				}
				msgPane.setCaretPosition(0);
			}

			public void mousePressed(MouseEvent arg0) {
			}

			public void mouseReleased(MouseEvent arg0) {
			}

			public void mouseEntered(MouseEvent arg0) {
			}

			public void mouseExited(MouseEvent arg0) {
			}
		});

		// alles zusammenbauen
		JPanel ret = new JPanel(new BorderLayout());
		ret.add(ctrlPane, BorderLayout.NORTH);
		ret.add(msgPaneScroll, BorderLayout.CENTER);
		ret.add(infoLabel, BorderLayout.SOUTH);
		return ret;
	}

	private static void createAndShowGUI() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		new GUI();
	}

	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}

}