package org.sfb441.tokenizer;

public class TokenizeUtilities {

	private static final String[] special = {"brir", "cier", "chos",
		"chas", "cuer", "guar", "guer", "mien", 
		"pues", "quel", "ssen", "tien", "tras", "vier", "brr", "cer", 
		"cir", "chs", "chs", "cur", "cer", "gur", "gar", "gur", 
		"ger", "men", "min", "pus", "pes", "qul", "qel", "ssn", 
		"ten", "tin", "trs", "ver", "vir", "bor", "bol", "bra", 
		"bri", "cer", "cie", "con", "cho", "cha", "dan", "der", 
		"des", "dri", "fas", "has", "las", "lon", "los", "lue", 
		"man", "nas", "ner", "nor", "nos", "par", "pie", "pin", 
		"plo", "pre", "por", "pro", "pue", "que", "ran", "ras", 
		"res", "ron", "sie", "sse", "sin", "sir", "son", "tar", 
		"tes", "tio", "tri", "tro", "van", "ver", "ves", "vir", 
		"vie", "vio", "vol", "vos", "xol", "zen", "zer", "zes", 
		"zos", "zas", "br", "bl", "br", "br", "cr", "ce", 
		"ci", "cn", "ch", "ch", "dn", "dr", "ds", "dr", 
		"fs", "hs", "ls", "ln", "ls", "lu", "le", "mn", 
		"ns", "nr", "nr", "ns", "pr", "pe", "pi", "pn", 
		"pl", "pr", "pr", "pr", "pu", "pe", "qu", "qe", 
		"rn", "rs", "rs", "rn", "se", "si", "ss", "sn", 
		"sr", "sn", "tr", "ts", "to", "ti", "tr", "tr", 
		"vn", "vr", "vs", "vr", "ve", "vi", "vo", "vi", 
		"vl", "vs", "xl", "zn", "zr", "zs", "zs", "zs", 
		"al", "an", "ar", "as", "o", "el", "en", "es", 
		"om", "op", "or", "se", "xo", "l", "n", "r", 
		"s", "", "l", "n", "s", "m", "p", "r", 
		"s", "x"};
	
	private static final String[] nosplits = {"pr", "br", "tr", "dr", "kr", "gr", "cr", "fr", "fl", "pl", "cl", "bl", "ch", "ll", "ss", "qu"};
	private static final String[] diphtong = {"ai", "ie", "ei", "ey", "oi", "ue", "ua", "io"};
	
	public static String findSyllables(String word, boolean useDict){
		if (useDict){
			for(int i = 0; i<special.length; ++i){
				if(word.toLowerCase().contains(special[i])){
					int st = word.toLowerCase().indexOf(special[i]);				
					String start = "";
					String end = "";				
					if(word.toLowerCase().equals(special[i])){
						return word;
					}else if (st == 0){
						end = word.substring(special[i].length(), word.length());
						return word.substring(0, special[i].length()) + "\n\n" + findSyllables(end, true);
					}else if(st == word.length() - special[i].length()){
						start = word.substring(0, st);
						return findSyllables(start, true) + "\n\n" + special[i];
					}else{
						start = word.substring(0, st);
						end = word.substring(st+special[i].length(), word.length());
						return findSyllables(start, true) + "\n\n" + special[i] + "\n\n" + findSyllables(end, true);
					}
				}
			}
		}
/*
		d)      Diphthonge (Verbindung aus zwei Vokalen) wie ie (es gibt ai, ie, ei [auch
		als ey wie in ley], teilweise oi, ue, ua), sind als zwei Vokale (VV)
		wiederzugeben, z.B.
		ardiente = ar.dien.te = CV.CVVC.CV; cuando = cuan.do = CVVC.CV; fue = CVV,
		fuente = CVVC.CV. Kein Diphthong: ae wie in caer = ca.er = CV.VC oder
		a.ca.es.ci; ambig ist oi, einmal o.r = V.VC, einmal oi.ga
		Es gibt auch einige wenige Triphthonge (aus drei Vokalen) wie buey = CVVV
		(eine Silbe).
		e)      Schreibung, Sonderzeichen: <, , , , , , > normalerweise wie <c, n,
		a, e, i, o, u> werten, aber in Fllen wie or = o.r [es mte noch weitere
		Flle geben] zeigt  eine Silbengrenze an; <h> [kommt in der Schreibung nur
		selten vor, wurde und wird nicht auszusprechen] ist generell nicht zu werten,
		<ch> zhlt wie schon gesagt als ein C.
		
		
Bei den Vokalverbindungen kann man generell alle Flle wie o, a etc.
als sog. Hiat (d.h. zwei Vokale nebeneinander, die in zu zwei
verschiedenen Silben gehren) betrachten (Bsp. o.r, ra.z), auch z.B.
a in atad = a.ta.d z.B. im Katalanischen gibt es noch mehr Flle auch
mit anderen Vokalen (ra = ra.), d.h. man kann generell am Vokal mit
Akzent erkennen, da hier - wenn ein weiterer Vokal folgt - eine
Silbentrennung (im Unterschied zu Diphthongen wie oi, ai etc.) erfolgen mu.
	*/	
		String ret = "";			
		String syl = word.toLowerCase();
		for(int i = 0; i<nosplits.length; ++i)
			syl = syl.replaceAll(nosplits[i], "DD");	
		for(int i = 0; i<diphtong.length; ++i)
			syl = syl.replaceAll(diphtong[i], "WW");			

		syl = syl.replaceAll("[aeiou]", "V");
		syl = syl.replaceAll("[qwrtzpsdfghjklyxcvbnm]", "C");
		//wenn das Wort nur aus einer Silbe besteht
		if(syl.indexOf("V") == syl.lastIndexOf("V") && !syl.contains("WW"))
			return word;
		if(syl.indexOf("WW") == syl.lastIndexOf("WW") && !syl.contains("V"))
			return word;
		int j = 0;
		int lastVowel = Math.max(syl.lastIndexOf('V'),syl.lastIndexOf('W'));
		for(int i = 0; i<syl.length(); i++){
			if( (syl.charAt(i) == 'V' ) || 
					(i > 0 && syl.charAt(i) == 'W' && syl.charAt(i-1) == 'W')){
				if (i<lastVowel){
					if(syl.charAt(i+1) == 'C' && syl.charAt(i+2) == 'C'){
						ret = ret + word.substring(j, i+2) + "\n\n";						
						j = i+2;						
					} else if(syl.charAt(i+1) == 'D' && syl.charAt(i+3) == 'C'){
						ret = ret + word.substring(j, i+3) + "\n\n";						
						j = i+3;												
					} else if(syl.charAt(i+1) == 'C' && syl.charAt(i+2) == 'D'){
						ret = ret + word.substring(j, i+2) + "\n\n";						
						j = i+2;												
					} else if(syl.charAt(i+1) == 'D' 
						      && (syl.charAt(i+3) == 'V' || syl.charAt(i+3) == 'W')){
						ret = ret + word.substring(j, i+1) + "\n\n";						
						j = i+1;												
					} else {
						ret = ret + word.substring(j, i+1) + "\n\n";
						j = i+1;
					}
				} else {
					break;
				}
			}
		}	
		ret = ret + word.substring(j, word.length());
/*		for(int i = 0; i<syl.length(); i++){
			if(syl.charAt(i) == 'V'){
				if (syl.lastIndexOf("V") != i  &&  syl.charAt(i+1) != 'V'){
					if(syl.charAt(i+1) == 'C' && syl.charAt(i+2) == 'C'){
						ret = ret + word.substring(j, i+2) + "\n\n";						
						j = i+2;						
					} else if(syl.charAt(i+1) == 'D' && syl.charAt(i+3) == 'C'){
						ret = ret + word.substring(j, i+3) + "\n\n";						
						j = i+3;												
					} else if(syl.charAt(i+1) == 'C' && syl.charAt(i+2) == 'D'){
						ret = ret + word.substring(j, i+2) + "\n\n";						
						j = i+2;												
					} else if(syl.charAt(i+1) == 'D' && syl.charAt(i+3) == 'V'){
						ret = ret + word.substring(j, i+1) + "\n\n";						
						j = i+1;												
					} else {
						ret = ret + word.substring(j, i+1) + "\n\n";
						j = i+1;
					}
				}
			}
		}
		ret = ret + word.substring(j, word.length());
		*/
								 
		return ret;
	} 	
}
