/*
 * Created on 03.06.2005
 */
package org.sfb441.tokenizer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.regex.Pattern;

/**
 * @author wolfgang
 */
public class Xmlifier {

	private File inputFile;

	private File outputFile;

	// Konstanten
	private static final int MAXLINESIZ = 25; // laengere Zeilen werden nicht
												// akzeptiert

	private static final String ENCODING = "ISO-8859-1"; // xml output
															// encoding

	private static final String DOCUMENTELEMENT = "corpus"; // xml output
															// document element

	private static final Pattern XMLCHARS = Pattern.compile("[<>&\"]");

	// weitere patterns
	private static final String COMMENTS_STR = "(\\[[^\\]]+\\])";

	private static final String PUNCT_STR = "([\\.\\[\\]\\,;\\(\\)?!:\\+-])";

	private static final String DIGIT = "([0123456789])";

	private static final Pattern NO_TOKEN = Pattern.compile(COMMENTS_STR + "|"
			+ PUNCT_STR + "|" + DIGIT);

	// das Zeichen nach dem \\s ist ein Leerzeichen aus dem CID, das ich von
	// dort
	// kopiert habe: Es wird komischerweise von Java nicht als Whitespace
	// behandelt.
	// Keine Ahnung, was das ist.
	private static final Pattern EMPTY = Pattern.compile("^[\\s]*$");

	public Xmlifier(File inputFile, File outputFile) {
		this.inputFile = inputFile;
		this.outputFile = outputFile;
	}

	public int xmlify() throws IOException, TokenizerException {
		String buf = "";
		
		int lineCounter = 0;
		// dateien oeffnen
		BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));
		BufferedReader in = new BufferedReader(new FileReader(inputFile));
		// xml header schreiben
		out.write("<?xml version=\"1.0\" encoding=\"" + ENCODING + "\"?>\n");
		out.write("<" + DOCUMENTELEMENT + ">\n");
		// input file lesen
		while ((buf = in.readLine()) != null) {
			++lineCounter; // fuer die Fehlermeldungen
			// zeile laenger als MAXLINE zeichen? (das ist vielleicht ein
			// untokenisiertes corpus)
			if (buf.length() > MAXLINESIZ)
				throw new TokenizerException("line " + lineCounter + " is "
						+ buf.length() + " chars long, max allowed "
						+ MAXLINESIZ);
			// sind vielleicht noch reservierte zeichen drin? ('<', '>', '"', '&')
			// mit entsprechender entity ersetzen.
			if (XMLCHARS.matcher(buf).find()){
				buf = buf.replaceAll("&", "&amp;");
				buf = buf.replaceAll("\"","&quot;" );
				buf = buf.replaceAll("<", "&lt;");
				buf = buf.replaceAll(">", "&gt;");
			}

			// was schreiben wir??
			// kein token
			System.err.println("+" + buf.trim() + "+");
			if(buf.equals("&amp;")){
				out.write("<token f=\"" + buf + "\"/>\n");
			}else if (NO_TOKEN.matcher(buf).find()){
				out.write("<other f=\"" + buf + "\"/>\n");
			// richtiges token
			} else {
				out.write("<token f=\"" + buf + "\"/>\n");
			}
		}
		// document element schliessen
		out.write("</" + DOCUMENTELEMENT + ">\n");
		// dateien schliessen
		in.close();
		out.close();
		return lineCounter;
	}
}
