package org.sfb441.tradisc;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.*;
import javax.swing.text.DefaultCaret;
import org.sfb441.tradisc.annotation.AnnotationException;
import org.sfb441.tradisc.annotation.Annotator;
import org.sfb441.tradisc.eval.EvaluationTable;
import org.sfb441.tradisc.schema.SchemaEditor;
import org.sfb441.tradisc.schema.SchemaException;
import org.sfb441.tradisc.schema.SchemaOverview;
import org.sfb441.tradisc.eval.*;
import org.sfb441.tradisc.inputformat.CorpusFormatEditor;

/**
 * TraDisc-Einstieg 12.07.2005
 */

public class GUI implements WindowListener {

	// System
	protected final static String LOOKANDFEEL = "Metal";
	protected final static String PRGNAME = "TraDisc";
	protected final static String RELEASEDATE = "December 2006";


	// command constants
	protected final static String ABOUT = "about";
	protected final static String ANN_HELP = "ann_help";
	protected final static String EXIT = "exit";
	protected final static String ANN_NEW = "ann_new";
	protected final static String ANN_OPEN = "ann_open";
	protected final static String ANN_MERGE = "ann_merge";
	protected final static String ANN_SPLIT = "ann_split";
	protected final static String ANN_CLOSE = "ann_close";
	protected final static String ANN_SAVE = "ann_save";
	protected final static String ANN_SAVEAS = "ann_saveas";
	protected final static String ANN_BW_VIEW = "toggle b/w print view";
	protected final static String ANN_PRINT = "print annotation";
	protected final static String SCH_EDIT = "sch_edit";
	protected final static String SCH_OVERVIEW = "schema overview";
	protected final static String SCH_NEW = "sch_new";
	protected final static String SCH_OPEN = "sch_open";
	protected final static String SCH_MERGE = "sch_merge";
	protected final static String SCH_SAVE = "sch_save";
	protected final static String SCH_SAVEAS = "sch_saveas";
	protected final static String EVAL_SHOW = "eval_show";
	protected final static String COMPLEX = "compute_global_complexity_score";
	protected final static String COUNT_ANN_MARKABLES = "count annotated markables";
	protected final static String ANN_MARKABLES_DIST = "compute annotated markables distribution";
	protected final static String C_DISTR = "complexity_distribution";
	protected final static String COMP_TABLE = "complexity_score_table";
	protected final static String SCH_USAGE = "show schema usage";
	protected final static String JUNCTOGRAM = "create and export junctogram";
	protected final static String SET_FORMAT = "set token input format";
	protected final static String LOAD_FORMAT = "load input format specifications";
	protected final static String SAVE_FORMAT = "save input format specifications";
	protected final static String SET_STANDARD_FORMAT = "Set TraDisc standard input format specifications";

	// GUI
	private JFrame guiFrame;
	private JTextPane guiTextPane;

	// Annotation
	private JMenuItem closeAnnotationMenuItem;
	private JMenuItem saveAnnotationMenuItem;
	private JMenuItem saveasAnnotationMenuItem;
	private JMenuItem printAnnotationMenuItem;
	private JMenuItem mergeWithAnnotationMenuItem;
	private JMenuItem splitAnnotationMenuItem;

	private Annotator annotator;
	private File annotationFile;
	private File annotationDataFile;

	// Schema
	private SchemaEditor schemaEditor;
	private File schemaFile;

	// Token Format
	private CorpusFormatEditor corpusFormatEditor;

	// Evaluation
	private EvaluationTable evalMan;
	private GlobalComplexityScorer complexScorer;
	private MarkableCount markableCounter;
	private MarkableCountDistribution markableCountDistribution;
	private ComplexityDistribution complexityDistribution;
	private ComplexityTable complexityTable;
	private Junctogram junctogram;
	
	// zurckgestellt:
	private JMenuItem evalShowMenuItem;
	private JMenuItem countMarkablesMenuItem;
	private JMenuItem markableCountDistributionMenuItem;
	private JMenuItem globalComplexityMenuItem;
	private JMenuItem complexityDistributionMenuItem;
	private JMenuItem complexityScoreTableMenuItem;
	private JMenuItem schemaUsageMenuItem;
	private JMenuItem junctogramMenuItem;
	
	private JMenuItem editCorpusFormatMenuItem;
	private JMenuItem setTraDiscStandardMenuItem;
	private JMenuItem loadFormatSpecsMenuItem;
	private JMenuItem saveFormatSpecsMenuItem;
	
	//private AnnotationPrinter printer;
	/**
	 * Konstruktor, protected, wegen Thread-Sicherheit. Siehe Main.
	 */
	
	protected GUI() {
		// Hauptprogramm: Frame erstellen
		guiFrame = new JFrame();
		guiFrame.addWindowListener(this);
		
		guiFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		// guiFrame.set
		guiFrame.setJMenuBar(createMenuBar());
		guiFrame.setContentPane(createContentPane());
		guiFrame.setSize(800, 600);
		guiFrame.setLocationRelativeTo(null); // zentrieren
		guiFrame.setVisible(true);
		// schema editor
		schemaFile = null;
		schemaEditor = new SchemaEditor(this);
		// annotation
		annotationFile = null;
		annotationDataFile = null;
		annotator = new Annotator(this);
		// eval
		evalMan = new EvaluationTable(this);
		complexScorer = new GlobalComplexityScorer(this);
		markableCounter = new MarkableCount(this);
		complexityDistribution = new ComplexityDistribution(this);
		markableCountDistribution = new MarkableCountDistribution(this);
		complexityTable = new ComplexityTable(this);
		corpusFormatEditor = new CorpusFormatEditor(this);
		junctogram = new Junctogram(this);
		this.setHeader();
		java.net.URL imgURL = GUI.class.getResource("images/td.GIF");
		guiFrame.setIconImage(new ImageIcon(imgURL).getImage());	
		Action focusAnnotatorAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {				
				if(annotator.isAnnotationOpen()){
					annotator.getFocus();
				}
			}
		};
		guiFrame.getRootPane().getActionMap().put("FOCUS_TO_ANNOTATOR", focusAnnotatorAction);
		KeyStroke left = KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0, false);
		guiFrame.getRootPane().getInputMap(
				JComponent.WHEN_FOCUSED).put(left, "FOCUS_TO_ANNOTATOR");		
		KeyStroke right = KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0, false);
		guiFrame.getRootPane().getInputMap(
				JComponent.WHEN_FOCUSED).put(right, "FOCUS_TO_ANNOTATOR");
		KeyStroke space = KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, false);
		guiFrame.getRootPane().getInputMap(
				JComponent.WHEN_FOCUSED).put(space, "FOCUS_TO_ANNOTATOR");
	}

	public File getSchemaFile() {
		return schemaFile;
	}

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = SchemaEditor.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		}
		System.err.println("Couldn't find image resource: " + path);
		return null;
	}
	
	/*
	 * File -> about
	 */
	private void showAboutDialog() {
		Date d = new Date();
		d.setTime(System.currentTimeMillis());
		JOptionPane.showMessageDialog(guiFrame, "<html><body><center>" 
				+ "<h2>" + PRGNAME + "</h2>"
				+ "Released " + RELEASEDATE
				+ "<h3>Authors: </h3>"
				+ "<p>Wolfgang Maier<br>" + "wmaier@sfs.uni-tuebingen.de<br>" 
				+ "Chris Malisi<br>" + "malisi@informatik.uni-tuebingen.de<br></p>" 				
				+ "</center></body></html>");
	}

	private void showAnn_helpDialog() {
		JOptionPane.showMessageDialog(guiFrame,"<html>"
				+ "<body><p>"
				+ "For documentation and help on TraDisc refer to <br>"
				+ "<i>http://www.sfb441.uni-tuebingen.de/b14/tools.html</i>"
				+ "</p></body></html>");
	}

	/*
	 * File -> exit Hier muessen noch entsprechende Fragen gestellt werden
	 * (wirklich schlieen?)
	 */
	private void exitProgram() {
		if (annotator.isAnnotationChanged()) {
			System.err.println("Annotation is open and has changed");
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Annotation has unsaved changes.\nExit program anyway?",
					"Really exit program?", JOptionPane.YES_NO_OPTION,
					JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.YES_OPTION)
				System.exit(0);
			if (n == JOptionPane.NO_OPTION)
				return;
		}
		if (schemaEditor.isSchemaChanged()) {
			System.err.println("Schema has unsaved changes");
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Schema has unsaved changes.\nExit program anyway?",
					"Really exit program?", JOptionPane.YES_NO_OPTION,
					JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.YES_OPTION)
				System.exit(0);
			if (n == JOptionPane.NO_OPTION)
				return;
		}
		System.exit(0);
	}

	/*
	 * neue Annotation erstellen
	 */
	private void newAnnotation() {
		// ist noch eine annotation offen?
		if (annotator.isAnnotationChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Annotation has unsaved changes.\nProceed anyway?",
					"Discard unsaved changes?", JOptionPane.YES_NO_OPTION,
					JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.NO_OPTION) {
				return;
			}
		}
		// schema im editor vorhanden?
		if (schemaEditor.getSchema().isEmpty()) {
			JOptionPane.showMessageDialog(guiFrame,
					"The schema is empty. Please edit/load schema!",
					"Empty schema", JOptionPane.WARNING_MESSAGE);
			return;
		}
		// alles ok: ungeandertes schema vorhanden, keine annotation offen
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("open corpus file");
		File corpusFile;
		int n = fc.showOpenDialog(guiFrame);
		if (n == JFileChooser.APPROVE_OPTION) {
			corpusFile = fc.getSelectedFile();
			try {
				annotator.newAnnotation(corpusFile, schemaEditor.getSchema());
				menuSetAnnotationOpen(true); // menue aktivieren
				evalMan.resetDisplayOptions();
			} catch (AnnotationException e) { // oeffnen hat nicht geklappt
				JOptionPane.showMessageDialog(guiFrame, "Error Message: "
						+ e.getMessage(), "Could not create annotation.",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		} else if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		setHeader();
	}

	/*
	 * eine schon begonnene Annotation oeffnen. Sie besteht aus zwei Dateien:
	 * Korpus + Annotationdatei
	 */
	private void openAnnotation() {
		// ist noch eine andere Annotation offen?
		if (annotator.isAnnotationChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Discard unsaved changes?",
					"Annotation has unsaved changes.\nProceed anyway?",
					JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.NO_OPTION) {
				return;
			}
		}
		// wurde das Schema geaendert? -> in extra-datei abspeichern
		if (schemaEditor.isSchemaChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Discard unsaved changes?",
					"Current Schema has unsaved changes.\nProceed anyway?",
					JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.NO_OPTION) {
				return;
			}
		}
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("open annotated corpus file");
		int n = fc.showOpenDialog(guiFrame);
		if (n == JFileChooser.APPROVE_OPTION) {
			annotationFile = fc.getSelectedFile();
		} else {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		File dir = fc.getCurrentDirectory();
		fc = new JFileChooser();
		fc.setCurrentDirectory(dir);
		fc.setDialogTitle("open annotation data file");
		n = fc.showOpenDialog(guiFrame);
		if (n == JFileChooser.APPROVE_OPTION) {
			File annoDataFile = fc.getSelectedFile();
			boolean same = annoDataFile.equals(annotationFile);
			while (same) {
				JOptionPane.showMessageDialog(guiFrame,
						"Annotation data and corpus files must be different!");
				n = fc.showOpenDialog(guiFrame);
				if (n == JFileChooser.APPROVE_OPTION) {
					annoDataFile = fc.getSelectedFile();
					same = annoDataFile.equals(annotationFile);
				} else {
					JOptionPane.showMessageDialog(guiFrame,
							"Operation canceled!");
					return;
				}
			}
			annotationDataFile = annoDataFile;
		} else {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		try {
			annotator.openAnnotation(annotationFile, annotationDataFile);
			menuSetAnnotationOpen(true);
			complexityTable.ensureCompatibility();
			evalMan.resetDisplayOptions();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(guiFrame, "Error Message: "
					+ e.getMessage(), "Could not open annotations.",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		setHeader();
		schemaFile = null;
	}

	private void mergeWithAnnotation() {
		if (!annotator.isAnnotationOpen()) {
			JOptionPane.showMessageDialog(guiFrame,
					"Only possible if an annotation is open");
			return;
		} else if (annotator.isAnnotationChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Proceed and loose changes?",
					"Annotation has unsaved changes.",
					JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.NO_OPTION) {
				return;
			}
		}
		File mergeCorpusFile = null;
		File mergeDataFile = null;
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("open annotated corpus file to merge");
		int n = fc.showOpenDialog(guiFrame);
		if (n == JFileChooser.APPROVE_OPTION) {
			mergeCorpusFile = fc.getSelectedFile();
		} else {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		File dir = fc.getCurrentDirectory();
		fc = new JFileChooser();
		fc.setCurrentDirectory(dir);
		fc.setDialogTitle("open annotation data file to merge");
		n = fc.showOpenDialog(guiFrame);
		if (n == JFileChooser.APPROVE_OPTION) {
			mergeDataFile = fc.getSelectedFile();
			boolean same = mergeDataFile.equals(mergeCorpusFile);
			while (same) {
				JOptionPane.showMessageDialog(guiFrame,
						"Annotation data and corpus files must be different!");
				n = fc.showOpenDialog(guiFrame);
				if (n == JFileChooser.APPROVE_OPTION) {
					mergeDataFile = fc.getSelectedFile();
					same = mergeDataFile.equals(mergeCorpusFile);
				} else {
					JOptionPane.showMessageDialog(guiFrame,
							"Operation canceled!");
					return;
				}
			}
		} else {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		try {
			annotator.mergeWithAnnotation(mergeCorpusFile, mergeDataFile);
			menuSetAnnotationOpen(true);
			evalMan.resetDisplayOptions();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(guiFrame, "Error Message: "
					+ e.getMessage(), "Could not merge annotations.",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		schemaFile = null;
		annotationFile = null;
		annotationDataFile = null;
		setHeader();
	}

	private void splitAnnotation() {
		if (!annotator.isAnnotationOpen()) {
			JOptionPane.showMessageDialog(guiFrame,
					"Only possible if an annotation is open");
			return;
		}
		/*
		 * else if (annotator.isAnnotationChanged()) { int n =
		 * JOptionPane.showConfirmDialog(guiFrame, "Proceed and loose changes?",
		 * "Annotation has unsaved changes.", JOptionPane.YES_NO_OPTION,
		 * JOptionPane.WARNING_MESSAGE); if(n == JOptionPane.NO_OPTION){ return; } }
		 */// get the filenames for the two annotations
		File corpusFile1 = null;
		File dataFile1 = null;
		File corpusFile2 = null;
		File dataFile2 = null;

		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save first annotation corpus file");
		int n = fc.showSaveDialog(guiFrame);
		if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation Canceled!");
			return;
		}
		if (n == JFileChooser.APPROVE_OPTION) {
			// erfrage namen des corpus
			corpusFile1 = fc.getSelectedFile();
			boolean exists = corpusFile1.exists();
			while (exists) {
				n = JOptionPane.showConfirmDialog(guiFrame, corpusFile1
						.getName()
						+ " already exists. Overwrite?", "Please confirm",
						JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
				if (n == JOptionPane.YES_OPTION)
					exists = false;
				if (n == JOptionPane.NO_OPTION) {
					n = fc.showSaveDialog(guiFrame);
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(guiFrame,
								"Operation Canceled!");
						return;
					}
					if (n == JFileChooser.APPROVE_OPTION) {
						corpusFile1 = fc.getSelectedFile();
						exists = corpusFile1.exists();
					}
				}
			}
		}
		File dir = fc.getCurrentDirectory();
		fc = new JFileChooser();
		fc.setCurrentDirectory(dir);

		fc.setDialogTitle("save first annotation data file:");
		n = fc.showSaveDialog(guiFrame);
		if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation Canceled!");
			return;
		}
		if (n == JFileChooser.APPROVE_OPTION) {
			// erfrage namen des corpus
			dataFile1 = fc.getSelectedFile();
			boolean exists = dataFile1.exists();
			boolean same = dataFile1.equals(corpusFile1);
			while (exists || same) {
				if (exists) {
					n = JOptionPane.showConfirmDialog(guiFrame, dataFile1
							.getName()
							+ " already exists. Overwrite?", "Please confirm",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n == JOptionPane.YES_OPTION)
						exists = false;
					if (n == JOptionPane.NO_OPTION) {
						n = fc.showSaveDialog(guiFrame);
						if (n == JFileChooser.CANCEL_OPTION) {
							JOptionPane.showMessageDialog(guiFrame,
									"Operation Canceled!");
							return;
						}
						if (n == JFileChooser.APPROVE_OPTION) {
							dataFile1 = fc.getSelectedFile();
							exists = dataFile1.exists();
							same = dataFile1.equals(corpusFile1);
						}
					}
				}
				if (same) {
					JOptionPane.showMessageDialog(guiFrame,
							"Please choose a different filename",
							"Filename already chosen",
							JOptionPane.WARNING_MESSAGE);
					n = fc.showSaveDialog(guiFrame);
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(guiFrame,
								"Operation Canceled!");
						return;
					}
					if (n == JFileChooser.APPROVE_OPTION) {
						dataFile1 = fc.getSelectedFile();
						exists = dataFile1.exists();
						same = dataFile1.equals(corpusFile1);
					}
				}
			}
		}

		dir = fc.getCurrentDirectory();
		fc = new JFileChooser();
		fc.setCurrentDirectory(dir);
		fc.setDialogTitle("save second annotation corpus file");
		n = fc.showSaveDialog(guiFrame);
		if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation Canceled!");
			return;
		}
		if (n == JFileChooser.APPROVE_OPTION) {
			// erfrage namen des corpus
			corpusFile2 = fc.getSelectedFile();
			boolean exists = corpusFile2.exists();
			boolean same = (corpusFile2.equals(corpusFile1) || corpusFile2
					.equals(dataFile1));
			while (exists || same) {
				if (exists) {
					n = JOptionPane.showConfirmDialog(guiFrame, corpusFile2
							.getName()
							+ " already exists. Overwrite?", "Please confirm",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n == JOptionPane.YES_OPTION)
						exists = false;
					if (n == JOptionPane.NO_OPTION) {
						n = fc.showSaveDialog(guiFrame);
						if (n == JFileChooser.CANCEL_OPTION) {
							JOptionPane.showMessageDialog(guiFrame,
									"Operation Canceled!");
							return;
						}
						if (n == JFileChooser.APPROVE_OPTION) {
							corpusFile2 = fc.getSelectedFile();
							exists = corpusFile2.exists();
							same = (corpusFile2.equals(corpusFile1) || corpusFile2
									.equals(dataFile1));
						}
					}
				}
				if (same) {
					JOptionPane.showMessageDialog(guiFrame,
							"Please choose a different filename",
							"Filename already chosen",
							JOptionPane.WARNING_MESSAGE);
					n = fc.showSaveDialog(guiFrame);
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(guiFrame,
								"Operation Canceled!");
						return;
					}
					if (n == JFileChooser.APPROVE_OPTION) {
						corpusFile2 = fc.getSelectedFile();
						exists = corpusFile2.exists();
						same = (corpusFile2.equals(corpusFile1) || corpusFile2
								.equals(dataFile1));
					}
				}
			}
		}
		dir = fc.getCurrentDirectory();
		fc = new JFileChooser();
		fc.setCurrentDirectory(dir);
		fc.setDialogTitle("save second annotation data file:");
		n = fc.showSaveDialog(guiFrame);
		if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation Canceled!");
			return;
		}
		if (n == JFileChooser.APPROVE_OPTION) {
			// erfrage namen des corpus
			dataFile2 = fc.getSelectedFile();
			boolean exists = dataFile2.exists();
			boolean same = (dataFile2.equals(corpusFile1)
					|| dataFile2.equals(corpusFile2) || dataFile2
					.equals(dataFile1));
			while (exists || same) {
				if (exists) {
					n = JOptionPane.showConfirmDialog(guiFrame, dataFile2
							.getName()
							+ " already exists. Overwrite?", "Please confirm",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n == JOptionPane.YES_OPTION)
						exists = false;
					if (n == JOptionPane.NO_OPTION) {
						n = fc.showSaveDialog(guiFrame);
						if (n == JFileChooser.CANCEL_OPTION) {
							JOptionPane.showMessageDialog(guiFrame,
									"Operation Canceled!");
							return;
						}
						if (n == JFileChooser.APPROVE_OPTION) {
							dataFile2 = fc.getSelectedFile();
							exists = dataFile2.exists();
							same = (dataFile2.equals(corpusFile1)
									|| dataFile2.equals(corpusFile2) || dataFile2
									.equals(dataFile1));
						}
					}
				}
				if (same) {
					JOptionPane.showMessageDialog(guiFrame,
							"Please choose a different filename",
							"Filename already chosen",
							JOptionPane.WARNING_MESSAGE);
					n = fc.showSaveDialog(guiFrame);
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(guiFrame,
								"Operation Canceled!");
						return;
					}
					if (n == JFileChooser.APPROVE_OPTION) {
						dataFile2 = fc.getSelectedFile();
						exists = dataFile2.exists();
						same = (dataFile2.equals(corpusFile1)
								|| dataFile2.equals(corpusFile2) || dataFile2
								.equals(dataFile1));
					}
				}
			}
		}
		try {
			annotator.splitAnnotation(corpusFile1, dataFile1, corpusFile2,
					dataFile2);
			// menuSetAnnotationOpen(true);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(guiFrame, "Error Message: "
					+ e.getMessage(), "Could not split Annotation.",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		schemaFile = null;

	}

	/*
	 * annotation schliessen
	 */
	private void closeAnnotation() {
		if (annotator.isAnnotationChanged()) {
			int n = JOptionPane
					.showConfirmDialog(
							guiFrame,
							"Annotation has unsaved changes. Proceed and loose changes?",
							"Please confirm", JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.NO_OPTION) {
				return;
			}
		}
		annotator.hide();
		menuSetAnnotationOpen(false);
		annotationFile = null;
		annotationDataFile = null;
		setHeader();
		evalMan.hide();
		evalMan.resetDisplayOptions();
	}

	/*
	 * eine begonnene Annotation abspeicheren
	 */
	private void saveAnnotation() {
		if (annotationFile == null || annotationDataFile == null) {
			// wenn dateiname unbekannt, muss er erst erfragt werden
			saveasAnnotation();
		} else {
			// annotation abspeichern
			try {
				annotator.saveAnnotation(annotationFile, annotationDataFile);
				JOptionPane.showMessageDialog(guiFrame, "Annotation saved!");
			} catch (Exception e) {
				JOptionPane.showMessageDialog(guiFrame, "Error Message: "
						+ e.getMessage(), "Could not save annotation.",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		}
		setHeader();
	}

	/*
	 * annotation abspeichern: hier werden nur die namen erfragt, dann wird
	 * saveAnnotation() aufgerufen
	 */
	private void saveasAnnotation() {
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save corpus");
		File cd = new File(System.getProperty("user.home"));
		cd = new File(cd, "Desktop");
		System.err.println(cd.getAbsolutePath());
		// hier weiter mit desktop auswhlen probiern
		int n = fc.showSaveDialog(guiFrame);
		if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		System.err.println(System.getProperties());
		File annoFile = null;
		if (n == JFileChooser.APPROVE_OPTION) {
			// erfrage namen des corpus
			annoFile = fc.getSelectedFile();
			boolean exists = annoFile.exists();
			while (exists) {
				n = JOptionPane.showConfirmDialog(guiFrame, annoFile.getName()
						+ " already exists. Overwrite?", "Please confirm",
						JOptionPane.YES_NO_OPTION);
				if (n == JOptionPane.YES_OPTION)
					exists = false;
				else if (n == JOptionPane.NO_OPTION) {
					n = fc.showSaveDialog(guiFrame);
					if (n == JFileChooser.APPROVE_OPTION)
						annoFile = fc.getSelectedFile();
					exists = annoFile.exists();
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(guiFrame,
								"Operation canceled!");
						return;
					}
				}
			}
		}
		File dir = fc.getCurrentDirectory();
		fc = new JFileChooser();
		fc.setCurrentDirectory(dir);
		fc.setDialogTitle("save annotation data");
		n = fc.showSaveDialog(guiFrame);
		if (n == JFileChooser.CANCEL_OPTION) {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		File annoDataFile = null;
		if (n == JFileChooser.APPROVE_OPTION) {
			// erfrage namen der datendatei
			// annotationDataFile = fc.getSelectedFile();
			annoDataFile = fc.getSelectedFile();
			boolean same = annoDataFile.equals(annoFile);
			boolean exists = annoDataFile.exists();
			while (exists || same) {
				if (exists) {
					n = JOptionPane.showConfirmDialog(guiFrame, annoDataFile
							.getName()
							+ " already exists. Overwrite?", "Please confirm",
							JOptionPane.YES_NO_OPTION);
					if (n == JOptionPane.YES_OPTION)
						exists = false;
					else {
						n = fc.showSaveDialog(guiFrame);
						if (n == JFileChooser.APPROVE_OPTION) {
							annoDataFile = fc.getSelectedFile();
							exists = annoDataFile.exists();
							same = annoDataFile.equals(annoFile);
						}
						if (n == JFileChooser.CANCEL_OPTION) {
							JOptionPane.showMessageDialog(guiFrame,
									"Operation canceled!");
							return;
						}
					}
				}
				if (same) {
					JOptionPane
							.showMessageDialog(
									guiFrame,
									"Annotation corpus and data files must have different names!",
									"Please enter other name",
									JOptionPane.INFORMATION_MESSAGE);
					n = fc.showSaveDialog(guiFrame);
					if (n == JFileChooser.APPROVE_OPTION) {
						annoDataFile = fc.getSelectedFile();
						exists = annoDataFile.exists();
						same = annoDataFile.equals(annoFile);
					}
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(guiFrame,
								"Operation canceled!");
						return;
					}

				}
			}
		}
		annotationFile = annoFile;
		annotationDataFile = annoDataFile;
		// beide namen da -> annotation speichern
		saveAnnotation();
	}

	// annotation offen? Menue aktivieren/deaktivieren
	private void menuSetAnnotationOpen(boolean b) {
		closeAnnotationMenuItem.setEnabled(b);
		saveAnnotationMenuItem.setEnabled(b);
		saveasAnnotationMenuItem.setEnabled(b);
		printAnnotationMenuItem.setEnabled(b);
		mergeWithAnnotationMenuItem.setEnabled(b);
		splitAnnotationMenuItem.setEnabled(b);
		evalShowMenuItem.setEnabled(b);
		countMarkablesMenuItem.setEnabled(b);
		markableCountDistributionMenuItem.setEnabled(b);
		globalComplexityMenuItem.setEnabled(b);
		complexityDistributionMenuItem.setEnabled(b);
		junctogramMenuItem.setEnabled(b);
		schemaUsageMenuItem.setEnabled(b);
		editCorpusFormatMenuItem.setEnabled(!b);
		loadFormatSpecsMenuItem.setEnabled(!b);
		saveFormatSpecsMenuItem.setEnabled(!b);
		setTraDiscStandardMenuItem.setEnabled(!b);
	}

	// annotation nach einer Schemanderung neu laden und die genderten
	// Markables bercksichtigen
	public void reloadAnnotation() {
		annotator.reloadAnnotation();
		evalMan.update();
		evalMan.loadActiveMarkables();		
	}

	/*
	 * public void refreshAnnotationWindow() { this.guiTextPane = new
	 * JTextPane(); annotator. }
	 */
	// schema *********************************************************
	/*
	 * schema-editor anzeigen
	 */
	private void editSchema() {
		if (annotator.isAnnotationOpen()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Annotation is open. Really edit schema?", "Please decide",
					JOptionPane.CANCEL_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				schemaEditor.setFixDimensions(true);
			} else
				return;
		} else {
			schemaEditor.setFixDimensions(false);
		}
		schemaEditor.show();
	}

	private void showSchemaOverview() {
		new SchemaOverview(this);
	}
	
	private void showSchemaUsage(){
		new SchemaUsage(this);
	}
	
	/*
	 * neues Schema erstellen, d.h. Schema-Editor leeren
	 */
	private void newSchema() {
		if (annotator.isAnnotationOpen()) {
			JOptionPane
					.showMessageDialog(
							guiFrame,
							"You cannot change to a new schema if an annotation is open",
							"Opening new schema: Error",
							JOptionPane.ERROR_MESSAGE);
			return;
		}

		if (schemaEditor.isSchemaChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"The schema has unsaved changes. Save it now?",
					"Create new empty schema", JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				saveSchema();
			}
		}
		// neues (leeres) Schema erzeugen
		schemaEditor.newSchema(true);
		// Dateiname auf null (="unbekannt") setzen
		schemaFile = null;
		complexityTable.ensureCompatibility();
		JOptionPane.showMessageDialog(guiFrame, "Schema editor is now empty!");
	}

	/*
	 * Schema zum Bearbeiten im Schema-Editor oeffnen
	 */
	private void openSchema() {
		// ist noch ein Schema offen?
		if (annotator.isAnnotationOpen()) {
			JOptionPane
					.showMessageDialog(
							guiFrame,
							"You cannot change to another schema if an annotation is open",
							"Open existing schema: Error",
							JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (schemaEditor.isSchemaChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"The schema has unsaved changes. Save it now?",
					"Open existing schema", JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				saveSchema();
			}
		}
		// schema whlen
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("open schema file");
		int n = fc.showOpenDialog(guiFrame);
		// wenn nicht "cancel"
		if (n == JFileChooser.APPROVE_OPTION) {
			schemaFile = fc.getSelectedFile();
			try {
				// versuche, das schema zu laden, behandle ausnahmen
				schemaEditor.openSchema(schemaFile);
				complexityTable.ensureCompatibility();
			} catch (FileNotFoundException e) {
				JOptionPane.showMessageDialog(guiFrame,
						"Could not open schema file " + schemaFile.getName()
								+ "!", "Open existing schema: Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			} catch (SchemaException e) {
				JOptionPane.showMessageDialog(guiFrame,
						"Format error in schema file " + schemaFile.getName()
								+ "!", "Open existing schema: Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			if (schemaEditor.getSchema() == null) { // should not happen. All
													// errors are treated
													// before.
				JOptionPane.showMessageDialog(guiFrame,
						"Could not open schema file " + schemaFile.getName()
								+ "!", "Open existing schema: Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
	}

	
	private void mergeSchema() {
		if (annotator.isAnnotationOpen() && annotator.isAnnotationChanged()) {
			int n = JOptionPane.showConfirmDialog(guiFrame,
					"Proceed without saving?",
					"Annotation has unsaved changes.",
					JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
			if (n == JOptionPane.NO_OPTION) {
				return;
			}
		}
		File mergeSchemaFile = null;
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("open schema file to merge");
		int n = fc.showOpenDialog(guiFrame);
		if (n == JFileChooser.APPROVE_OPTION) {
			mergeSchemaFile = fc.getSelectedFile();
		} else {
			JOptionPane.showMessageDialog(guiFrame, "Operation canceled!");
			return;
		}
		try {
			schemaEditor.mergeWithSchema(mergeSchemaFile);
			
			reloadAnnotation();
			//annotator.loadActiveMarkables();


			//annotator.annotate(getCurrentMarkableLocation());
			evalMan.resetDisplayOptions();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(guiFrame, "Error Message: "
					+ e.getMessage(), "Could not merge schema.",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
		schemaFile = null;
		setHeader();
	}
	
	/*
	 * schema speichern
	 */
	private void saveSchema() {
		// wenn Dateiname noch nicht bekannt
		if (schemaFile == null) {
			saveasSchema(); // erst erfragen
			return;
		}
		try {
			schemaEditor.saveSchema(schemaFile);
			JOptionPane.showMessageDialog(guiFrame, "Schema saved!");
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(guiFrame,
					"Could not save schema to file " + schemaFile.getName()
							+ ": Write error", "Save schema: Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (SchemaException ex) {
			JOptionPane.showMessageDialog(guiFrame,
					"Could not save schema to file " + schemaFile.getName()
							+ ": It seems to be empty", "Save schema: Error",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/*
	 * Dateiname fr Schema erfragen
	 */
	private void saveasSchema() {
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save schema");
		int n = fc.showSaveDialog(guiFrame);
		if (n != JFileChooser.APPROVE_OPTION)
			return;

		// dateiname erfragen
		File file = fc.getSelectedFile();
		boolean exists = file.exists();
		while (exists) {
			n = JOptionPane.showConfirmDialog(guiFrame, file.getName()
					+ " already exists. Overwrite?", "Save schema",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.NO_OPTION) {
				n = fc.showSaveDialog(guiFrame);
				if (n == JFileChooser.APPROVE_OPTION) {
					file = fc.getSelectedFile();
					exists = file.exists();
				} else {
					JOptionPane.showMessageDialog(guiFrame,
							"Operation canceled!", "Save schema",
							JOptionPane.INFORMATION_MESSAGE);
					return;
				}
			} else
				exists = false;
		}
		schemaFile = file;
		// Dateiname bekannt -> Schema speichern
		saveSchema();
	}

	//CorpusFormat********************************************************
	
	private void loadCorpusFormat() {
		File file;
		// schema whlen
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("load input format specifications");
		int n = fc.showOpenDialog(guiFrame);
		// wenn nicht "cancel"
		if (n == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
			try {
				this.corpusFormatEditor.openCorpusFormat(file);
				JOptionPane.showMessageDialog(guiFrame, "Corpus format loaded!");
				// versuche, das format zu laden, behandle ausnahmen
								
			} catch (FileNotFoundException e) {
				JOptionPane.showMessageDialog(guiFrame,
						"Could not open corpus format file " + file.getName()
								+ "!", "Open corpus format: Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			} catch (Exception e) {
				JOptionPane.showMessageDialog(guiFrame,
						"Format error in corpus format file " + file.getName()
								+ "!", "Open corpus format: Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}			
		}
	}

	private void saveCorpusFormat() {
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save input format specifications");
		int n = fc.showSaveDialog(guiFrame);
		if (n != JFileChooser.APPROVE_OPTION)
			return;
		// dateiname erfragen
		File file = fc.getSelectedFile();
		boolean exists = file.exists();
		while (exists) {
			n = JOptionPane.showConfirmDialog(guiFrame, file.getName()
					+ " already exists. Overwrite?", "Save corpus format",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.NO_OPTION) {
				n = fc.showSaveDialog(guiFrame);
				if (n == JFileChooser.APPROVE_OPTION) {
					file = fc.getSelectedFile();
					exists = file.exists();
				} else {
					JOptionPane.showMessageDialog(guiFrame,
							"Operation canceled!", "Save corpus format",
							JOptionPane.INFORMATION_MESSAGE);
					return;
				}
			} else
				exists = false;
		}
		try {
			this.corpusFormatEditor.saveCorpusFormat(file);
			JOptionPane.showMessageDialog(guiFrame, "Corpus format saved!");
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(guiFrame,
					"Could not save to file " + file.getName()
							+ "!", "Save corpus format: Error",
					JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
	
	// Evaluation ********************************************************

	private void evalShow() {
		evalMan.show();
	}

	private void computeComplexityScore() {
		complexScorer.computeGlobalComplexityScore();
	}

	private void countAnnotatedMarkables() {
		markableCounter.countAnnotatedMarkables();
	}
	
	private void showMarkableCountDistribution(){
		markableCountDistribution.show();
	}
	
	private void showComplexityDistribution() {
		complexityDistribution.show();
	}

	private void showComplexityScoreTable() {
		complexityTable.show();
	}

	private void printAnnotation(){
		try{			
			PrintUtilities.printTextPane(getAnnotator().getPrintLayout());				
		} catch (Exception e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(guiFrame, "Error while creating print layout", "Printing Error", JOptionPane.ERROR_MESSAGE);
			return;
		}	
	}
	
	/**
	 * Gebe das Annotationsmodul zurck
	 * 
	 * @return das Annotationsmodul
	 */
	public Annotator getAnnotator() {
		return annotator;
	}

	public CorpusFormatEditor getCorpusFormatEditor() {
		return corpusFormatEditor;
	}

	/**
	 * Gebe Komplexittswert-Tabelle zurck
	 * 
	 * @return ComplexityTable
	 */
	public ComplexityTable getComplexityTable() {
		return complexityTable;
	}

	/**
	 * Gebe den Schema-Editor zurck
	 * 
	 * @return der Schema-Editor
	 */
	public SchemaEditor getSchemaEditor() {
		return schemaEditor;
	}

	/**
	 * Gibt den EvaluationManager zurck
	 * 
	 * @return EvaluationManager
	 */
	public EvaluationTable getEvaluationManager() {
		return evalMan;
	}

	/**
	 * Gebe die JTextPane des Hauptfensters zurck
	 * 
	 * @return JTextPane
	 */
	public JTextPane getGuiTextPane() {
		return guiTextPane;
	}

	/**
	 * Gebe den JFrame des Hauptfensters zurck
	 * 
	 * @return JFrame (Hauptfenster)
	 */
	public JFrame getGuiFrame() {
		return guiFrame;
	}

	/**
	 * Gibt die berschrift der Hauptfensters zurck
	 * 
	 * @return String
	 */
	public void setHeader() {
		String header = "TraDisc - ";
		if (annotator.isAnnotationOpen())
			if (this.annotationFile == null)
				header = header + "unsaved annotation";
			else {
				header = header + annotationFile.getName();
				if (getAnnotator().isAnnotationChanged())
					header = header + " [changed]";
			}
		else
			header = header + "no annotation active";
		guiFrame.setTitle(header);
	}

	/*
	 * look+feel initialisieren
	 */
	private static void initLookAndFeel() {
		String lookAndFeel = null;
		if (LOOKANDFEEL != null) {
			if (LOOKANDFEEL.equals("Metal")) {
				lookAndFeel = UIManager.getCrossPlatformLookAndFeelClassName();
			} else if (LOOKANDFEEL.equals("System")) {
				lookAndFeel = UIManager.getSystemLookAndFeelClassName();
			} else if (LOOKANDFEEL.equals("Motif")) {
				lookAndFeel = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
			} else if (LOOKANDFEEL.equals("GTK+")) {
				lookAndFeel = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
			} else {
				System.err
						.println("Unexpected value of LOOKANDFEEL specified: "
								+ LOOKANDFEEL);
				lookAndFeel = UIManager.getCrossPlatformLookAndFeelClassName();
			}
			try {
				UIManager.setLookAndFeel(lookAndFeel);
			} catch (ClassNotFoundException e) {
				System.err
						.println("Couldn't find class for specified look and feel:"
								+ lookAndFeel);
				System.err
						.println("Did you include the L&F library in the class path?");
				System.err.println("Using the default look and feel.");
			} catch (UnsupportedLookAndFeelException e) {
				System.err.println("Can't use the specified look and feel ("
						+ lookAndFeel + ") on this platform.");
				System.err.println("Using the default look and feel.");
			} catch (Exception e) {
				System.err.println("Couldn't get specified look and feel ("
						+ lookAndFeel + "), for some reason.");
				System.err.println("Using the default look and feel.");
				e.printStackTrace();
			}
		}
	}

	/*
	 * Behandlet ActionEvents, die vom Hauptprogramm (GUI) ausgehen
	 */
	private class ActionHandler implements ActionListener {
		GUI caller;

		public ActionHandler(GUI caller) {
			this.caller = caller;
		}

		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();

			// debug
			String s = "[GUI] Command: " + command;
			System.err.println(s);

			if (ABOUT.equals(command)) {
				caller.showAboutDialog();
			} else if (EXIT.equals(command)) {
				caller.exitProgram();
			} else if (ANN_HELP.equals(command)) {
				caller.showAnn_helpDialog();
			}

			else if (ANN_NEW.equals(command)) {
				newAnnotation();
			} else if (ANN_OPEN.equals(command)) {
				openAnnotation();
			} else if (ANN_MERGE.equals(command)) {
				mergeWithAnnotation();
			} else if (ANN_SPLIT.equals(command)) {
				splitAnnotation();
			} else if (ANN_CLOSE.equals(command)) {
				closeAnnotation();
			} else if (ANN_SAVE.equals(command)) {
				saveAnnotation();
			} else if (ANN_SAVEAS.equals(command)) {
				saveasAnnotation();
			} else if(ANN_PRINT.equals(command)){
				printAnnotation();
			}
			
			else if (SCH_EDIT.equals(command)) {
				editSchema();
			}else if(SCH_OVERVIEW.equals(command)){
				showSchemaOverview();
			}else if(SCH_MERGE.equals(command)){
				mergeSchema();
			}else if (SCH_NEW.equals(command)) {
				newSchema();
			} else if (SCH_OPEN.equals(command)) {
				openSchema();
			} else if (SCH_SAVE.equals(command)) {
				saveSchema();
			} else if (SCH_SAVEAS.equals(command)) {
				saveasSchema();
			}
			else if (EVAL_SHOW.equals(command)) {
				evalShow();
			} else if (COMPLEX.equals(command)) {
				computeComplexityScore();
			} else if(COUNT_ANN_MARKABLES.equals(command)){
				countAnnotatedMarkables();
			}else if(ANN_MARKABLES_DIST.equals(command)){
				showMarkableCountDistribution();
			} else if (C_DISTR.equals(command)) {
				showComplexityDistribution();
			} else if (COMP_TABLE.equals(command)) {
				showComplexityScoreTable();
			} else if(SCH_USAGE.equals(command)){
				showSchemaUsage();
			}else if (JUNCTOGRAM.equals(command)) {
				junctogram.show();
			} else if (SET_FORMAT.equals(command)) {
				corpusFormatEditor.show();
			} else if (SET_STANDARD_FORMAT.equals(command)) {
				corpusFormatEditor.setTraDiscStandard();
			} else if (SAVE_FORMAT.equals(command)) {
				caller.saveCorpusFormat();
			} else if (LOAD_FORMAT.equals(command)) {
				caller.loadCorpusFormat();
			}
		}
	}


	
	/*
	 * erstellt die JMenuBar
	 */
	protected JMenuBar createMenuBar() {
		ActionHandler actionListener = new ActionHandler(this);

		JMenuBar menuBar;
		JMenu menu;
		JMenuItem menuItem;

		menuBar = new JMenuBar();

		// annotation menu.
		menu = new JMenu("Annotation");
		menu.setMnemonic(KeyEvent.VK_A);
		menuBar.add(menu);

		JMenuItem newAnnotationMenuItem = new JMenuItem("New...");
		newAnnotationMenuItem.setActionCommand(ANN_NEW);
		newAnnotationMenuItem.setMnemonic(KeyEvent.VK_N);
		newAnnotationMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		newAnnotationMenuItem.addActionListener(actionListener);
		menu.add(newAnnotationMenuItem);

		JMenuItem openAnnotationMenuItem = new JMenuItem("Open...");
		openAnnotationMenuItem.setActionCommand(ANN_OPEN);
		openAnnotationMenuItem.setMnemonic(KeyEvent.VK_O);
		openAnnotationMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		openAnnotationMenuItem.addActionListener(actionListener);
		menu.add(openAnnotationMenuItem);

		mergeWithAnnotationMenuItem = new JMenuItem("Merge with...");
		mergeWithAnnotationMenuItem.setActionCommand(ANN_MERGE);
		mergeWithAnnotationMenuItem.setMnemonic(KeyEvent.VK_M);
		mergeWithAnnotationMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		mergeWithAnnotationMenuItem.setEnabled(false);
		mergeWithAnnotationMenuItem.addActionListener(actionListener);
		menu.add(mergeWithAnnotationMenuItem);

		splitAnnotationMenuItem = new JMenuItem("Split");
		splitAnnotationMenuItem.setActionCommand(ANN_SPLIT);
		splitAnnotationMenuItem.setMnemonic(KeyEvent.VK_P);
		splitAnnotationMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_P, ActionEvent.CTRL_MASK));
		splitAnnotationMenuItem.setEnabled(false);
		splitAnnotationMenuItem.addActionListener(actionListener);
		menu.add(splitAnnotationMenuItem);

		closeAnnotationMenuItem = new JMenuItem("Close");
		closeAnnotationMenuItem.setActionCommand(ANN_CLOSE);
		closeAnnotationMenuItem.setMnemonic(KeyEvent.VK_C);
		closeAnnotationMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_W, ActionEvent.CTRL_MASK));
		closeAnnotationMenuItem.setEnabled(false);
		closeAnnotationMenuItem.addActionListener(actionListener);
		menu.add(closeAnnotationMenuItem);

		menu.addSeparator();

		saveAnnotationMenuItem = new JMenuItem("Save");
		saveAnnotationMenuItem.setActionCommand(ANN_SAVE);
		saveAnnotationMenuItem.setMnemonic(KeyEvent.VK_S);
		saveAnnotationMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		saveAnnotationMenuItem.setEnabled(false);
		saveAnnotationMenuItem.addActionListener(actionListener);
		menu.add(saveAnnotationMenuItem);

		saveasAnnotationMenuItem = new JMenuItem("Save as...");
		saveasAnnotationMenuItem.setActionCommand(ANN_SAVEAS);
		saveasAnnotationMenuItem.setMnemonic(KeyEvent.VK_A);
		saveasAnnotationMenuItem.setEnabled(false);
		saveasAnnotationMenuItem.addActionListener(actionListener);
		menu.add(saveasAnnotationMenuItem);

		menu.addSeparator();
		
		printAnnotationMenuItem = new JMenuItem("Print");
		printAnnotationMenuItem.setActionCommand(ANN_PRINT);
		printAnnotationMenuItem.setMnemonic(KeyEvent.VK_P);
		printAnnotationMenuItem.setEnabled(false);
		printAnnotationMenuItem.addActionListener(actionListener);
		menu.add(printAnnotationMenuItem);		
		
		menu.addSeparator();

		menuItem = new JMenuItem("Exit");
		menuItem.setActionCommand(EXIT);
		menuItem.setMnemonic(KeyEvent.VK_X);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,
				ActionEvent.ALT_MASK));
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		// schema menu.
		menu = new JMenu("Schema");
		menu.setMnemonic(KeyEvent.VK_S);
		menuBar.add(menu);

		menuItem = new JMenuItem("Edit...");
		menuItem.setActionCommand(SCH_EDIT);
		menuItem.setMnemonic(KeyEvent.VK_E);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
				ActionEvent.CTRL_MASK));
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		menuItem = new JMenuItem("Merge with schema...");
		menuItem.setActionCommand(SCH_MERGE);		
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);		
				
		menuItem = new JMenuItem("Overview");
		menuItem.setActionCommand(SCH_OVERVIEW);
		menuItem.setMnemonic(KeyEvent.VK_O);
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);				
		
		menuItem = new JMenuItem("New");
		menuItem.setActionCommand(SCH_NEW);
		menuItem.setMnemonic(KeyEvent.VK_N);
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		menuItem = new JMenuItem("Open...");
		menuItem.setActionCommand(SCH_OPEN);
		menuItem.setMnemonic(KeyEvent.VK_P);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
				ActionEvent.CTRL_MASK));
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		menu.addSeparator();

		menuItem = new JMenuItem("Save");
		menuItem.setActionCommand(SCH_SAVE);
		menuItem.setMnemonic(KeyEvent.VK_S);
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		menuItem = new JMenuItem("Save as...");
		menuItem.setActionCommand(SCH_SAVEAS);
		menuItem.setMnemonic(KeyEvent.VK_A);
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		// eval menu.
		menu = new JMenu("Evaluation");
		menu.setMnemonic(KeyEvent.VK_V);
		menuBar.add(menu);

		evalShowMenuItem = new JMenuItem("Show evaluation table");
		evalShowMenuItem.setActionCommand(EVAL_SHOW);
		evalShowMenuItem.setMnemonic(KeyEvent.VK_S);
		evalShowMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
				ActionEvent.CTRL_MASK));
		evalShowMenuItem.setEnabled(false);
		evalShowMenuItem.addActionListener(actionListener);
		menu.add(evalShowMenuItem);
		
		schemaUsageMenuItem = new JMenuItem("Schema usage");
		schemaUsageMenuItem.setActionCommand(SCH_USAGE);
		schemaUsageMenuItem.setMnemonic(KeyEvent.VK_U);
		schemaUsageMenuItem.setEnabled(false);
		schemaUsageMenuItem.addActionListener(actionListener);
		menu.add(schemaUsageMenuItem);

		menu.addSeparator();
		
		countMarkablesMenuItem = new JMenuItem("Count annotated markables");
		countMarkablesMenuItem.setActionCommand(COUNT_ANN_MARKABLES);
		countMarkablesMenuItem.setMnemonic(KeyEvent.VK_A);
		countMarkablesMenuItem.setEnabled(false);
		countMarkablesMenuItem.addActionListener(actionListener);
		menu.add(countMarkablesMenuItem);

		markableCountDistributionMenuItem = new JMenuItem("Distribution of annotated markables");
		markableCountDistributionMenuItem.setActionCommand(ANN_MARKABLES_DIST);
		markableCountDistributionMenuItem.setMnemonic(KeyEvent.VK_M);
		markableCountDistributionMenuItem.setEnabled(false);
		markableCountDistributionMenuItem.addActionListener(actionListener);
		menu.add(markableCountDistributionMenuItem);

		menu.addSeparator();

		globalComplexityMenuItem = new JMenuItem("Complexity score");
		globalComplexityMenuItem.setActionCommand(COMPLEX);
		globalComplexityMenuItem.setMnemonic(KeyEvent.VK_C);
		// evalShowMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
		// ActionEvent.CTRL_MASK));
		globalComplexityMenuItem.setEnabled(false);
		globalComplexityMenuItem.addActionListener(actionListener);
		menu.add(globalComplexityMenuItem);

		complexityDistributionMenuItem = new JMenuItem(
				"Complexity distribution");
		complexityDistributionMenuItem.setActionCommand(C_DISTR);
		complexityDistributionMenuItem.setMnemonic(KeyEvent.VK_D);
		// evalShowMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
		// ActionEvent.CTRL_MASK));
		complexityDistributionMenuItem.setEnabled(false);
		complexityDistributionMenuItem.addActionListener(actionListener);
		menu.add(complexityDistributionMenuItem);

		complexityScoreTableMenuItem = new JMenuItem("Complexity score table");
		complexityScoreTableMenuItem.setActionCommand(COMP_TABLE);
		complexityScoreTableMenuItem.setMnemonic(KeyEvent.VK_T);
		complexityScoreTableMenuItem.setEnabled(true);
		complexityScoreTableMenuItem.addActionListener(actionListener);
		menu.add(complexityScoreTableMenuItem);

		menu.addSeparator();

		junctogramMenuItem = new JMenuItem("Export junctogram");
		junctogramMenuItem.setActionCommand(JUNCTOGRAM);
		junctogramMenuItem.setMnemonic(KeyEvent.VK_J);
		junctogramMenuItem.setEnabled(false);
		junctogramMenuItem.addActionListener(actionListener);
		menu.add(junctogramMenuItem);
		
		// InputFormat Men
		menu = new JMenu("Input format");
		menu.setMnemonic(KeyEvent.VK_I);
		menuBar.add(menu);

		editCorpusFormatMenuItem = new JMenuItem("Edit corpus input format");
		editCorpusFormatMenuItem.setActionCommand(SET_FORMAT);
		editCorpusFormatMenuItem.setMnemonic(KeyEvent.VK_C);
		editCorpusFormatMenuItem.addActionListener(actionListener);
		menu.add(editCorpusFormatMenuItem);

		setTraDiscStandardMenuItem = new JMenuItem(
				"Set TraDisc standard format");
		setTraDiscStandardMenuItem.setActionCommand(SET_STANDARD_FORMAT);
		setTraDiscStandardMenuItem.setMnemonic(KeyEvent.VK_T);
		setTraDiscStandardMenuItem.addActionListener(actionListener);
		menu.add(setTraDiscStandardMenuItem);

		saveFormatSpecsMenuItem = new JMenuItem(
				"Save input format specifications...");
		saveFormatSpecsMenuItem.setActionCommand(SAVE_FORMAT);
		saveFormatSpecsMenuItem.setMnemonic(KeyEvent.VK_S);
		saveFormatSpecsMenuItem.addActionListener(actionListener);
		menu.add(saveFormatSpecsMenuItem);

		loadFormatSpecsMenuItem = new JMenuItem(
				"Load input format specifications...");
		loadFormatSpecsMenuItem.setActionCommand(LOAD_FORMAT);
		loadFormatSpecsMenuItem.setMnemonic(KeyEvent.VK_L);
		loadFormatSpecsMenuItem.addActionListener(actionListener);
		menu.add(loadFormatSpecsMenuItem);

		// help men
		menu = new JMenu("?");
		menu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(menu);

		menuItem = new JMenuItem("About...");
		menuItem.setActionCommand(ABOUT);
		menuItem.setMnemonic(KeyEvent.VK_A);
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		menuItem = new JMenuItem("Annotator help");
		menuItem.setActionCommand(ANN_HELP);
		menuItem.setMnemonic(KeyEvent.VK_H);
		menuItem.addActionListener(actionListener);
		menu.add(menuItem);

		return menuBar;
	}
	
	/*
	 * contentPane fr GUI
	 */
	protected Container createContentPane() {
		// JTextPane fr GUI, scrollbar
		guiTextPane = new JTextPane();
		guiTextPane.setEditable(false);
		guiTextPane.setCaret(new TraDiscCaret());
		JScrollPane guiScrollPane = new JScrollPane(guiTextPane);
		// JTextPane kommt in die GUI-ContentPane
		JPanel contentPane = new JPanel(new BorderLayout());
		
		contentPane.add(guiScrollPane, BorderLayout.CENTER);
		return contentPane;
	}

	
	private class TraDiscCaret extends DefaultCaret {

		public void mouseClicked(MouseEvent e) {
			if (getAnnotator().isAnnotationOpen() && e.getClickCount() == 2) {			
				int m = getMark();
				Vector elements = getAnnotator().getAllElementLocations();
				Iterator it = elements.iterator();
				while (it.hasNext()) {
					Location l = (Location) it.next();
					if (l.start.getOffset() > m) {
						int i = elements.indexOf(l);
						if (i > 0) {
							l = (Location) elements.elementAt(--i);							
						}
						getAnnotator().setPositionToElement(i);
						break;
					}
				}
				if(((Location)elements.elementAt(elements.size()-1)).start.getOffset() <= m){
					getAnnotator().setPositionToElement(elements.size() - 1);
				}
			}
		}
	}
	
	
	
	
	private static void createAndShowGUI() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		initLookAndFeel();
		new GUI();
	}

	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}




	
	
	public void windowOpened(WindowEvent arg0) {
	}

	public void windowClosing(WindowEvent arg0) {
		exitProgram();
	}

	public void windowClosed(WindowEvent arg0) {
	}

	public void windowIconified(WindowEvent arg0) {
	}

	public void windowDeiconified(WindowEvent arg0) {
	}

	public void windowActivated(WindowEvent arg0) {
	}

	public void windowDeactivated(WindowEvent arg0) {
	}

}