package org.sfb441.tradisc;

import java.util.Enumeration;
import java.util.Hashtable;

import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/*
 * Markable: Beschreibt ein (als Junktor) "markierbares" XML-Element.
 * Hauptschlich im Schema-Modul verwendet.
 * 
 * Eigentlich ist es nicht gut, diese Klasse zu haben. Vielleicht 
 * wrde eine Klasse statt zwei (Markable/Location) reichen.  
 */
public class Markable {

	private String xmlElementName;

	private Hashtable<String, String> xmlElementAttributes;

	/**
	 * Konstruktor mit Initialisierung
	 * 
	 * @param tagName
	 * @param tagValue
	 * @param tagAttributes
	 */
	
	public String getAttributeValue(String attributeName){
		if(xmlElementAttributes.containsKey(attributeName)){
			return xmlElementAttributes.get(attributeName);
		}else return null;
	}
	
	public Markable(String tagName, Hashtable<String, String> tagAttributes) {
		this.xmlElementName = tagName;
		if (tagAttributes != null) {
			this.xmlElementAttributes = tagAttributes;
		} else {
			this.xmlElementAttributes = new Hashtable<String, String>();
		}
	}

	public Markable(Markable m) {
		this.xmlElementName = new String(m.xmlElementName);
		if (m.xmlElementAttributes != null) {
			this.xmlElementAttributes = new Hashtable<String, String>(m.xmlElementAttributes);
		} else {
			this.xmlElementAttributes = new Hashtable<String, String>();
		}
	}

	
	public static Markable element2markable(Node node) {
		Markable m = new Markable();
		m.xmlElementName = node.getNodeName();
		NamedNodeMap attrs = node.getAttributes();
		if (attrs != null) {
			for (int i = 0; i < attrs.getLength(); i++) {
				Attr attr = (Attr) attrs.item(i);
				String attrName = attr.getNodeName();
				String attrValue = attr.getNodeValue();
				m.xmlElementAttributes.put(attrName, attrValue);
			}
		}
		return m;
	}

	/**
	 * leerer Konstruktor. Vielleicht kann man es auch ohne ihn hinkriegen
	 */
	public Markable() {
		this.xmlElementAttributes = new Hashtable<String, String>();
	}

	/**
	 * Schlechter Stil. checkt, ob zwei Markables miteinander "kompatibel" sind
	 * (besonders in bezug auf wildcards, also *) wenn beide Attribute "f"
	 * gleich sind, return true wenn "f" von m wildcard enthlt, und "f" von
	 * this dazupasst, return true andersherum false(this wildcard, m passt
	 * dazu) !! andere Attribute spielen keine Rolle (z.B. "annref") --> this
	 * ist corpus-markable, m ist schema-markable
	 * 
	 * @param m
	 * @return
	 */

	public boolean isCompatibleWith(Markable m) {		
		if (!this.xmlElementName.equals(m.xmlElementName)) {
			return false;
		}
		//System.err.println("  this: " + this + "  m: " + m);
		Enumeration e = m.xmlElementAttributes.keys();
		while (e.hasMoreElements()) {
			String mname = (String) e.nextElement();
			if (!this.xmlElementAttributes.containsKey(mname)){				
				return false;
			}
			String mval = (m.xmlElementAttributes.get(mname)).toLowerCase();
			String thisval = (this.xmlElementAttributes.get(mname))
					.toLowerCase();
			if (!mval.equals(thisval)) {
				if (mval.contains("*")) {										
					String mFstripped = mval.replace("*", "");					
					if (mval.startsWith("*") && mval.endsWith("*")){
						if(!thisval.contains(mFstripped))
							return false;
					}
					else if (mval.startsWith("*")){
						if(!thisval.endsWith(mFstripped))
							return false;
					}
					else if (!thisval.startsWith(mFstripped)){
						return false;
					}
				} else
					return false;
			}
		}
		return true;
	}


	/**
	 * genauer Vergleich
	 */
	public boolean equals(Object o) {
		if (!(o instanceof Markable))
			return false;
		Markable m = (Markable) o;
		return (this.xmlElementName.equals(m.xmlElementName)
				 && this.xmlElementAttributes.equals(m.xmlElementAttributes));
	}

	/*
	 * drucke eine XML-Reprsentation des Markables
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String ret = "";
		if (xmlElementName == null || xmlElementName.equals("")) {
			return " ";
		}
		ret += "<" + xmlElementName;
		if (xmlElementAttributes != null && xmlElementAttributes.size() > 0) {
			Enumeration enumeration = xmlElementAttributes.keys();
			while (enumeration.hasMoreElements()) {
				String attrName = (String) enumeration.nextElement();
				String attrValue = xmlElementAttributes.get(attrName);
				ret += " " + attrName + "=\"" + attrValue + "\"";
			}
		}
		ret += "/>";
		return ret;
	}

	public String getXmlElementName() {
		return xmlElementName;
	}

	public void setXmlElementName(String xmlElementName) {
		this.xmlElementName = xmlElementName;
	}

	public Hashtable<String, String> getXmlElementAttributes() {
		return xmlElementAttributes;
	}

	public void setXmlElementAttributes(
			Hashtable<String, String> xmlElementAttributes) {
		this.xmlElementAttributes = xmlElementAttributes;
	}

	

}
