package org.sfb441.tradisc;

import java.io.File;
import java.io.IOException;
import java.io.Writer;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

/**
 * xml utilities 12.07.2005
 * 
 * @author wolfgang
 */
public class XMLUtilities {

	/**
	 * parse eine XML-Datei
	 * 
	 * @param f
	 *            Die XML-Datei
	 * @param validating
	 *            validieren oder nicht
	 * @return das geparste Dokument, wenn erfolgreich
	 */
	public static Document parseXMLFile(File f, boolean validating) {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setValidating(validating);
			Document ldoc = factory.newDocumentBuilder().parse(f);
			ldoc.normalize();
			return ldoc;
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Schreibe ein DOM-Document auf einen Writer
	 * 
	 * @param doc
	 *            DOM-Document
	 * @param w
	 *            Writer
	 */
	public static void writeDOMDocument(Document doc, Writer w) {
		try {
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();
			transformer.setOutputProperty("encoding", "ISO-8859-1");
			// transformer.setOutputProperty(OutputKeyINDENT, "yes");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			DOMSource source = new DOMSource(doc);

			StreamResult result = new StreamResult(w);
			transformer.transform(source, result);
		} catch (TransformerConfigurationException tce) {
			// parser error
			System.out.println("* Transformer Factory error");
			System.out.println("  " + tce.getMessage());

			Throwable x = tce;
			if (tce.getException() != null)
				x = tce.getException();
			x.printStackTrace();
		} catch (TransformerException te) {
			// parser generated error
			System.out.println("* Transformation error");
			System.out.println("  " + te.getMessage());

			Throwable x = te;
			if (te.getException() != null)
				x = te.getException();
			x.printStackTrace();
		}
	}

}
