package org.sfb441.tradisc.annotation;

import java.util.*;

import org.sfb441.tradisc.Location;
import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.inputformat.CorpusFormat;
import org.sfb441.tradisc.schema.SchemaModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class AnnotationDocument {

	private Element corpusDocEl;

	private CorpusFormat tokenFormat;

	protected Vector<Location> tokens;

	protected Vector<Location> markables;

	protected Vector<Location> allElements;

	private SchemaModel schema;

	public AnnotationDocument() {
		tokens = new Vector<Location>();
		markables = new Vector<Location>();
		allElements = new Vector<Location>();
	}

	public CorpusFormat getTokenFormat() {
		return tokenFormat;
	}

	public Element getCorpusDocumentElement() {
		return corpusDocEl;
	}

	public void setCorpusDocumentElement(Element corpusDocEl) {
		this.corpusDocEl = corpusDocEl;
	}

	public SchemaModel getSchema() {
		return schema;
	}

	public void setSchema(SchemaModel schema) {
		this.schema = schema;
	}

	private boolean isToken(Node node) {
		if (!node.getNodeName().equals(tokenFormat.getTokenTag()))
			return false;
		Element el = (Element) node;
		if (!el.hasAttribute(tokenFormat.getTokenDisplayAttr()))
			return false;
		Enumeration e = tokenFormat.getTokenAttributeNames();
		while (e.hasMoreElements()) {
			String att = (String) e.nextElement();
			if (!el.hasAttribute(att))
				return false;
			if (!tokenFormat.getTokenAttribute(att)
					.equals(el.getAttribute(att)))
				return false;
		}
		if (tokenFormat.getTokenTag().equals(tokenFormat.getOtherTag())) {
			e = tokenFormat.getOtherAttributeNames();
			while (e.hasMoreElements()) {
				String name = (String) e.nextElement();
				if (tokenFormat.getTokenAttribute(name) == null) {
					if (el.hasAttribute(name)
							&& el.getAttribute(name).equals(
									tokenFormat.getOtherAttribute(name)))
						return false;
				}
			}
		}
		return true;
	}

	private boolean isOtherDisplayElement(Node node) {
		if (!node.getNodeName().equals(tokenFormat.getOtherTag()))
			return false;
		Element el = (Element) node;
		if (!el.hasAttribute(tokenFormat.getOtherDisplayAttr()))
			return false;
		Enumeration e = tokenFormat.getOtherAttributeNames();
		while (e.hasMoreElements()) {
			String att = (String) e.nextElement();
			if (!el.hasAttribute(att))
				return false;
			if (!tokenFormat.getOtherAttribute(att)
					.equals(el.getAttribute(att)))
				return false;
		}
		return true;
	}

	private boolean isMarkable(Node node) {
		Markable m = Markable.element2markable(node);		
		return schema.contains(m);
	}

	
	private Location buildNode(Node node, boolean isMarked,
			Hashtable annotationInformation) {
		Location nodeLocation = new Location();
		nodeLocation.node = node;

		// is the node a markable?
		if (isMarkable(node)) {
			isMarked = true;
			// does it have a tag already?
			if (node.getAttributes().getNamedItem("annref") != null) {
				String reference = node.getAttributes().getNamedItem("annref")
						.getNodeValue();
				// wenn schon ein tag da, richtig setzen
				
				if (annotationInformation.containsKey(reference)){
					nodeLocation.tag = (Tag) annotationInformation
							.get(reference);					
				}
				
			}
			markables.addElement(nodeLocation);
		}

		// marked-Eigenschaft vererben
		nodeLocation.marked = isMarked;

		// ist der knoten ein token? wenn es ein token ist, dann schauen wir die
		// kinder nicht an
		if (isToken(node)) {
			nodeLocation.isToken = true;
			Element el = (Element) node;
			String text = el.getAttribute(tokenFormat.getTokenDisplayAttr());
			// nodeLocation.len = Markable.getTokenText(node,
			// tokenInfo).length();
			nodeLocation.len = text.length();
			tokens.addElement(nodeLocation);
			allElements.addElement(nodeLocation);
			// das am weitesten links liegende token unterhalb eines tokens
			// ist das token selbst
			nodeLocation.lmt = nodeLocation;
		} else {
			// kein token -> kinder anschauen
			NodeList children = node.getChildNodes();
			// hat kinder -> ist <corpus> - Element
			if (children.getLength() > 0) {
				for (int i = 0; i < children.getLength(); ++i) {
					Node child = children.item(i);
					Location childLocation = buildNode(child, isMarked,
							annotationInformation);
					// wenn wir noch kein "lmt" gesetzt haben und das kind eines
					// hat,
					// d.h. wenn das kind ein richtiger knoten ist (kein
					// textknoten),
					// dann setzte das "lmt".
					// so muesste eigentlich zu annotierende knoten ein solches
					// lmt kriegen.
					if (nodeLocation.lmt == null && childLocation.lmt != null)
						nodeLocation.lmt = childLocation.lmt;
				}
				// hat keine kinder -> other
			} else {
				if (isOtherDisplayElement(nodeLocation.node)) {
					nodeLocation.isToken = false;
					allElements.addElement(nodeLocation);
					Element el = (Element) node;
					String text = el.getAttribute(tokenFormat
							.getTokenDisplayAttr());
					nodeLocation.len = text.length();
					// nodeLocation.len = Markable.getTokenText(node,
					// tokenInfo).length();
				}
			}
		}
		isMarked = false;
		return nodeLocation;
	}

	/**
	 * Baut ein AnnotationDocument auf
	 * 
	 * @param corpus
	 * @param schema
	 * @param tokenInfo
	 * @link{Markable}, dass das Tag beschreibt, das die Tokens enthaelt
	 * @param annotationInformation
	 *            Hashtable mit annotation referenz (int) nach tag (Tag)
	 * @throws AnnotationException
	 */
	public void build(Document corpus, SchemaModel schema,
			CorpusFormat tokenFormat, Hashtable annotationInformation)
			throws AnnotationException {
		// xml corpus lesen und in internes format umwandeln
		corpus.normalize();
		this.corpusDocEl = corpus.getDocumentElement();
		this.schema = schema;
		this.tokenFormat = tokenFormat;
		// wenn es ein neues AnnotationDocument ist (d.h. das AnnotationDocument
		// gerade nicht aus einer Datei geladen wird), dann brauchen
		// wir eine leere Hashtable
		if (annotationInformation == null)
			annotationInformation = new Hashtable();
		// rekursiv ueber xml baum laufen
		buildNode(corpusDocEl, false, annotationInformation);
		// keine tokens da
		/*
		 * Iterator it = allElements.iterator(); while(it.hasNext())
		 * System.err.println(it.next());
		 */if (tokens.size() == 0)
			throw new AnnotationException("No tokens found");

		// nichts annotatierbares gefunden
		if (markables.size() == 0)
			throw new AnnotationException("No markables found");
	}

	
	public void rebuild() {
		Vector<Location> newMarkables = new Vector<Location>();
		Vector<Location> newTokens = new Vector<Location>();
		Iterator it = tokens.iterator();
		while (it.hasNext()) {
			Location l = (Location) it.next();
			Node node = l.node;
			Markable m = Markable.element2markable(node);
			// Markable wurde aus dem Schema gelscht
			if (!schema.contains(m)) {
				l.marked = false;
				l.tag = null;
			} else { // Markable ist auch in neuem Schema enthalten
				l.marked = true;
				if (l.tag != null && !schema.containsTagforMarkable(m, l.tag)){
					l.tag = null;
				}
				newMarkables.add(l);
			}
			newTokens.add(l);
		}
		this.markables = newMarkables;
		this.tokens = newTokens;
		// this.schema = newSchema;
	}


	public void mergeWith(AnnotationDocument mergeDoc)
			throws AnnotationException {
		try {
			markables.addAll(mergeDoc.markables);
			tokens.addAll(mergeDoc.tokens);
			allElements.addAll(mergeDoc.allElements);
			schema.mergeWith(mergeDoc.getSchema());
			Document mergeCorp = mergeDoc.getCorpusDocumentElement()
					.getOwnerDocument();
			Document corpus = this.corpusDocEl.getOwnerDocument();
			if (mergeCorp == null || corpus == null) {
				throw new AnnotationException("Couldn't get Document "
						+ mergeCorp.getNodeName());
			}
			System.err.println(mergeCorp.getDocumentElement().getTagName());
			NodeList children = mergeCorp.getDocumentElement()
					.getElementsByTagName("*");
			System.err.println("laenge: " + children.getLength());
			while (children.getLength() > 0) {
				Node n = children.item(0);
				Node replacingNode = (corpus.adoptNode(n));
				corpusDocEl.appendChild(replacingNode);
			}
		} catch (Exception e) {
			throw new AnnotationException(e.getMessage());
		}
	}

	/**
	 * returns how often Tag t was used as an Annotation
	 * 
	 * @param t
	 * @return int
	 */
	public int getTagCount(Tag t) {
		int ret = 0;
		Iterator it = markables.iterator();
		while (it.hasNext()) {
			Location l = (Location) it.next();
			if (l.tag != null && l.tag.equals(t))
				++ret;
		}
		return ret;
	}

	/**
	 * returns how often Tag t was used as an Annotation for Tokens between
	 * positions start to end
	 * 
	 * @param t
	 * @return
	 */
	public int getWindowTagCount(Tag t, int start, int end) {
		int ret = 0;
		int tokenpos = start;
		Location ll = tokens.elementAt(tokenpos);
		// get first markable in window
		while (!ll.marked && tokenpos < tokens.size() -1) {
			ll =  tokens.elementAt(++tokenpos);
		}
		int markablepos = markables.indexOf(ll);
		while (tokenpos >= start && tokenpos < end && markablepos < markables.size()) {
			if (ll.tag != null && ll.tag.equals(t)) {
				++ret;
			}
			if(markablepos == markables.size() - 1){
				break;
			}
			ll = markables.elementAt(++markablepos);
			tokenpos = tokens.indexOf(ll);
		}
		return ret;		
	}

	public double getDistributionFor(Vector<Markable> v, Tag t) {
		double ret = 0.0;
		Iterator it = markables.iterator();
		while (it.hasNext()) {
			Location l = (Location) it.next();
			Markable lm = Markable.element2markable(l.node);
			// System.err.println("lm: " + lm);
			Iterator<Markable> im = v.iterator();
			while (im.hasNext()) {
				Markable m = im.next();
				if ((l.tag != null) && lm.isCompatibleWith(m)
						&& l.tag.equals(t)) {
					++ret;
					break;
				}
			}
		}
		return ret;
	}

	
	public int countAnnotatedMarkables() {
		int ret = 0;
		Iterator<Location> it = markables.iterator();
		while (it.hasNext()) {
			Location l = it.next();
			if (l.marked && l.tag != null && !l.tag.isNonMarkableTag()) {
				++ret;
			}
		}		
		return ret;
	}	
	
	public int countAnnotatedMarkablesInWindow(int start, int end) {
		int ret = 0;
		int tokenpos = start;
		Location ll = tokens.elementAt(tokenpos);
		// get first markable in window
		while (!ll.marked && tokenpos < tokens.size() - 1) {
			ll =  tokens.elementAt(++tokenpos);
		}
		int markablepos = markables.indexOf(ll);
		while (tokenpos >= start && tokenpos < end && markablepos < markables.size()) {
			if (ll.tag != null && !ll.tag.isNonMarkableTag()) {
				++ret;
			} 
			if(markablepos == markables.size() - 1){
				break;
			}
			ll = markables.elementAt(++markablepos);
			tokenpos = tokens.indexOf(ll);
		}
		return ret;
	}	

	
	/**
	 * returns -1 if Location is not included in tokens vector
	 */
	public int getTokenNumberFor(Location l) {
		return tokens.indexOf(l);
	}

	/**
	 * Gibt Anzahl der Tokens in diesem Annotationsdokument zurueck
	 * 
	 * @return Anzahl der Tokens
	 */
	public int getTokenCount() {
		return tokens.size();
	}

	/**
	 * Gibt Anzahl aller Elemente in diesem Annotationsdokument zurueck
	 * 
	 * @return Anzahl aller Elemente
	 */
	public int getTotalCount() {
		return allElements.size();
	}

	/**
	 * Gibt Anzahl der gefundenen markierbaren Elemente in diesem Annotations-
	 * dokument zurck.
	 * 
	 * @return Anzahl der markierbaren Elemente
	 */
	public int getMarkableCount() {
		return markables.size();
	}

	/* annotation */
	public void setTagOnMarkableAt(Tag t, int at) {
		Location l = markables.elementAt(at);
		l.tag = t;
	}

	/* deselecting */
	public void setNoTagOnMarkableAt(int at) {
		Location l = markables.elementAt(at);
		l.tag = null;
	}

}
