package org.sfb441.tradisc.annotation;

import java.io.File;
import java.util.Hashtable;

import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.XMLUtilities;
import org.sfb441.tradisc.inputformat.CorpusFormat;
import org.sfb441.tradisc.inputformat.CorpusFormatReader;
import org.sfb441.tradisc.schema.SchemaModel;
import org.sfb441.tradisc.schema.SchemaReader;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class AnnotationDocumentReader {

	private Document doc;

	private Document datadoc;

	private File annotationFile;

	private File annotationDataFile;

	private CorpusFormat tokenFormat;

	public AnnotationDocumentReader(File annotationFile,
			File annotationDataFile, CorpusFormat tokenFormat) {
		this.annotationFile = annotationFile;
		this.annotationDataFile = annotationDataFile;
		this.tokenFormat = tokenFormat;
	}

	/*
	 * erzeugt ein hash mit den Annotationsinformationen aus dem
	 * Annotationsdaten-XML- Dokument.
	 */

	private Hashtable<String, Tag> getData(Element dataDocEl) throws AnnotationException {
		Hashtable<String, Tag> ret = new Hashtable<String, Tag>();
		NodeList l = dataDocEl.getElementsByTagName("ann");
		for (int i = 0; i < l.getLength(); ++i) {
			Element e = (Element) l.item(i);
			String ref = null;
			String attr = "";
			Tag t = new Tag();

			attr = e.getAttribute("annref");
			if (attr != null)
				ref = attr;
			attr = e.getAttribute("vert");
			if (attr != null)
				t.setVerticalDim(attr);
			attr = e.getAttribute("horiz");
			if (attr != null)
				t.setHorizontalDim(attr);

			if (t.getVerticalDim() == null || t.getHorizontalDim() == null
					|| ref == null)
				throw new AnnotationException("Annotation data contain errors");

			ret.put(ref, t);
		}
		return ret;
	}

	public AnnotationDocument read() throws Exception {
		AnnotationDocument ret = new AnnotationDocument();

		doc = XMLUtilities.parseXMLFile(annotationFile, false);
		if (doc == null) {
			System.err.println("Error: Couldn't load XML file.");
			throw new AnnotationException("Couldn't parse XML corpus file.\n");
		}
		// docEl ist AnnotationDocument XML element
		Element docEl = doc.getDocumentElement();

		datadoc = XMLUtilities.parseXMLFile(annotationDataFile, false);
		if (datadoc == null) {
			System.err.println("Error: Couldn't load XML data file.");
			throw new AnnotationException("Couldn't parse XML data file.");
		}
		Element dataDocEl = datadoc.getDocumentElement();

		// Schluessel vergleichen
		String docKey = docEl.getAttribute("key");
		String dataDocKey = dataDocEl.getAttribute("key");
		if (docKey == null || docKey.equals(""))
			throw new AnnotationException(
					"\"key\"-attribute in corpus document is missing");
		if (dataDocKey == null || dataDocKey.equals(""))
			throw new AnnotationException(
					"\"key\"-attribute in data document is missing");
		if (!docKey.equals(dataDocKey))
			throw new AnnotationException("\"key\"-attributes do not match");

		// Annotations-Information lesen
		Hashtable annotationInformation = getData(dataDocEl);

		// read schema
		NodeList schemaRoots = dataDocEl.getElementsByTagName("raibleschema");
		if (schemaRoots == null || schemaRoots.getLength() != 1)
			throw new AnnotationException(
					"No or more than one schema root element in data file");
		Element schemaRoot = (Element) schemaRoots.item(0);
		SchemaModel schema = SchemaReader.parseSchema(schemaRoot);		
		NodeList formatRoots = dataDocEl.getElementsByTagName("CorpusFormat");
		if (formatRoots.getLength() > 1)
			throw new AnnotationException(
					"No or more than one schema root element in data file");
		//Wenn ein Format mitgespeochert ist, dieses verwenden, sonst das gearde in TraDisc aktive
		else if (formatRoots.getLength() == 1){
			Element corpusFormatRoot = (Element)formatRoots.item(0);
			tokenFormat = CorpusFormatReader.parseCorpusFormat(corpusFormatRoot);
		}
		
		// build the document
		ret.build(doc, schema, tokenFormat, annotationInformation);

		return ret;
	}
}