package org.sfb441.tradisc.annotation;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

import org.sfb441.tradisc.Location;
import org.sfb441.tradisc.XMLUtilities;
import org.sfb441.tradisc.inputformat.CorpusFormatWriter;
import org.sfb441.tradisc.schema.SchemaException;
import org.sfb441.tradisc.schema.SchemaWriter;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class AnnotationDocumentWriter extends FileWriter {

	private BufferedWriter datafs;

	private int markableIndex = 0; // absoluter zaehler (alle markierbaren
									// Elemente (=Junktoren))

	private int taggedMarkableInd = 0; // zaehler fuer die tatsaechlich
										// getaggten elemente

	private File corpusFile;

	public AnnotationDocumentWriter(File corpusFile, File dataFile)
			throws IOException {
		super(corpusFile);
		this.corpusFile = corpusFile;
		datafs = new BufferedWriter(new FileWriter(dataFile));
	}

	public static String createID() {
		return Long.toString(System.currentTimeMillis());
	}

	public void write(AnnotationDocument annDoc) throws SchemaException,
			IOException {
		// Schlssel erzeugen
		String key = createID();
		// corpus annotation data file
		datafs.write(new String(
				"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"));
		datafs.write(new String("<raibledata key=\"" + key + "\">\n"));
		SchemaWriter.schema2xml(annDoc.getSchema(), datafs);
		datafs.write(new String("<annotationdata>\n"));
		taggedMarkableInd = 0;
		Iterator it = updateAnnotation(annDoc.markables,
				annDoc.getCorpusDocumentElement()).iterator();
		while (it.hasNext())
			datafs.write("  " + (String) it.next() + "\n");
		datafs.write(new String("</annotationdata>\n"));
		CorpusFormatWriter.corpusFormat2xml(annDoc.getTokenFormat(), datafs);
		datafs.write(new String("</raibledata>\n"));
		datafs.close();
		// corpus file
		Element docEl = annDoc.getCorpusDocumentElement();
		docEl.setAttribute("key", key);
		XMLUtilities.writeDOMDocument(docEl.getOwnerDocument(), this);
		super.close();
	}

	
	private Vector<String> updateAnnotation(Vector markables, Element node) {
		Vector<String> ret = new Vector<String>();
		if (markableIndex < markables.size()) {
			Location ml = (Location) markables.elementAt(markableIndex);
			if (ml.node.equals(node)) {
				++markableIndex;
				// wenn der aktuelle Junktor (bzw. das den Junktor enthaltende
				// xml-tag)
				// getaggt ist, dann
				if (ml.tag != null) {

					// erzeuge referenz im Korpus
					node.setAttribute("annref", new Integer(taggedMarkableInd)
							.toString());
					// erzeuge tag
					ret.add("<ann annref=\"" + taggedMarkableInd + "\" vert=\""
							+ ml.tag.getVerticalDim() + "\" horiz=\""
							+ ml.tag.getHorizontalDim() + "\"/>");
					++taggedMarkableInd;
				} else {
					if (node.hasAttribute("annref")) {
						node.removeAttribute("annref");
					}
				}
			}
		}
		NodeList children = node.getElementsByTagName("*");
		for (int i = 0; i < children.getLength(); ++i) {
			node = (Element) children.item(i);
			ret.addAll(updateAnnotation(markables, node));
		}
		return ret;
	}

	public void splitWrite(AnnotationDocument annDoc, int splitMarkable,
			int splitElement, boolean firstHalf) throws SchemaException,
			IOException {
		// Schlssel erzeugen
		String key = createID();
		// corpus annotation data file
		datafs.write(new String(
				"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"));
		datafs.write(new String("<raibledata key=\"" + key + "\">\n"));
		SchemaWriter.schema2xml(annDoc.getSchema(), datafs);
		datafs.write(new String("<annotationdata>\n"));
		taggedMarkableInd = 0;
		Iterator<String> it = splitUpdateAnnotation(annDoc.markables,
				annDoc.getCorpusDocumentElement(), splitMarkable, firstHalf)
				.iterator();
		while (it.hasNext()){
			datafs.write(it.next());
		}
		datafs.write(new String("</annotationdata>\n"));
		CorpusFormatWriter.corpusFormat2xml(annDoc.getTokenFormat(), datafs);
		datafs.write(new String("</raibledata>\n"));
		datafs.close();

		// corpus file
		datafs = new BufferedWriter(new FileWriter(corpusFile));
		datafs.write(new String(
				"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"));
		datafs.write(new String("<corpus key=\"" + key + "\">\n"));
		Element docEl = annDoc.getCorpusDocumentElement();
		NodeList children = docEl.getElementsByTagName("*");
		int elementCount = 0;
		Element element = (Element) children.item(0);
		// Element node = new Element();
		for (int i = 0; i < children.getLength(); i++) {
			element = (Element) children.item(i);

			if ((firstHalf && elementCount < splitElement)
					|| (!firstHalf && elementCount >= splitElement)) {
				datafs.write(new String("<" + element.getTagName() + " "));
				NamedNodeMap attributes = element.getAttributes();
				String attributesString = "";
				for (int j = 0; j < attributes.getLength(); j++) {
					Node attNode = attributes.item(j);
					String value = attNode.getNodeValue();
					if(value.equals("&")){
						value = "&amp;";
					}if(value.contains("\"")){
						value = value.replaceAll("\"", "&quot;");
					}
					attributesString += attNode.getNodeName() + "=\""
							+ value +  "\" ";
				}
				datafs.write(attributesString + "/>\n");
			}
			elementCount++;
		}
		datafs.write(new String("</corpus>"));
		datafs.close();
	}
	
	private Vector<String> splitUpdateAnnotation(Vector markables, Element node,
			int splitMarkable, boolean firstHalf) {
		Vector<String> ret = new Vector<String>();		
		if ((firstHalf && markableIndex < splitMarkable)
				|| (!firstHalf && markableIndex < markables.size())) {
			Location ml = (Location) markables.elementAt(markableIndex);
			if (ml.node.equals(node)) {
				++markableIndex;
				// wenn der aktuelle Junktor (bzw. das den Junktor enthaltende
				// xml-tag)
				// getaggt ist, dann
				if (ml.tag != null) {
					// nur annotieren, wenn in das MArkable in frage in der
					// richtigen Split-hlfte ist
					if (firstHalf || !firstHalf
							&& markableIndex >= splitMarkable) {
						// erzeuge referenz im Korpus
						node.setAttribute("annref", new Integer(
								taggedMarkableInd).toString());
						// erzeuge tag
						ret.add("<ann annref=\"" + taggedMarkableInd
								+ "\" vert=\"" + ml.tag.getVerticalDim()
								+ "\" horiz=\"" + ml.tag.getHorizontalDim()
								+ "\"/>");
						++taggedMarkableInd;
					}
				} else {					
					if (node.hasAttribute("annref")) {
						node.removeAttribute("annref");
					}
				}
			}
		}
		NodeList children = node.getElementsByTagName("*");
		for (int i = 0; i < children.getLength(); ++i) {
			node = (Element) children.item(i);
			ret.addAll(splitUpdateAnnotation(markables, node, splitMarkable,
					firstHalf));
		}
		return ret;
	}
	

}