/*
 * Created on 16.03.2005
 *
 */
package org.sfb441.tradisc.annotation;

import java.awt.Color;
import java.util.Iterator;
import java.util.regex.Pattern;

import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.Position;

import org.sfb441.tradisc.Location;
import org.sfb441.tradisc.inputformat.CorpusFormat;
import org.w3c.dom.Element;

/**
 * @author wmaier and malisi
 */
public class AnnotationView {

	private AnnotationModel caller;

	private int currentMarkableNumber;

	private int currentTokenNumber;

	private int currentElementNumber;

	private CorpusFormat corpusFormat;

	private JTextPane guiTextPane;

	private DefaultStyledDocument guiDoc;

	private static String FONT_BIG = "font_big";

	private static String FONT_MARKABLE_TOPROCESS = "m_toproc";

	private static String FONT_MARKABLE_PROCESSED = "m_proc";

	private static String FONT_MARKABLE_NOJUNKTOR = "m_nojunktor";

	private static String FONT_BOLD = "bold";

	private static String FONT_NOT_CURRPOS = "not_currpos";

	private static String FONT_NO_MARKABLE = "no_m";

	private static String FONT_OTHER = "other";
	
	private static final String FONT_ITAL = "italics";

	private static final Pattern PUNCT1 = Pattern
			.compile("([\\.\\,;\\)?!:\\]'])");

	private static final Pattern PUNCT2 = Pattern.compile("([\\(\\\\[])");

	public AnnotationView(AnnotationModel caller) {
		this.caller = caller;
		guiTextPane = caller.caller.getGuiTextPane();
		guiDoc = caller.caller.getGuiDoc();
		currentTokenNumber = 0;
		currentMarkableNumber = 0;
		currentElementNumber = 0;
		corpusFormat = caller.caller.getInputFormat();
		installStylesOnTextPane();
	}

	/** *************** LAYOUT ************************ */
	public void doLayout(CorpusFormat corpFormat) throws AnnotationException {
		// Iterator tokenIter = caller.annDoc.tokens.iterator();
		this.corpusFormat = corpFormat;
		Iterator iter = caller.annDoc.allElements.iterator();
		guiTextPane.setText("");
		boolean punct2 = false;
		while (iter.hasNext()) {
			Location l = (Location) iter.next();
			// boolean other =
			// !l.node.getNodeName().equals(corpusFormat.getTokenTag());
			// boolean other = false;
			boolean punct1 = false;
			Element el = (Element) l.node;
			String raw_text = "";

			if (l.isToken)
				raw_text = el.getAttribute(corpusFormat.getTokenDisplayAttr());
			else
				// has to be an other (non-token display element)
				raw_text = el.getAttribute(corpusFormat.getOtherDisplayAttr());
			String text = "";
			if (raw_text.length() > 0) {
				if (PUNCT1.matcher(raw_text).matches()) {
					text = raw_text;
					punct1 = true;
				} else if (punct2)
					text = raw_text;
				else
					text = " " + raw_text;
			}
			try {
				guiDoc.insertString(guiDoc.getLength(), text, null);
				l.start = guiDoc.createPosition(guiDoc.getLength()
						- text.length());
				if (punct1 || punct2)
					l.start = guiDoc.createPosition(guiDoc.getLength()
							- text.length());
				else
					l.start = guiDoc.createPosition(guiDoc.getLength()
							- text.length() + 1);
				guiTextPane.setCaretPosition(guiDoc.getLength());
			} catch (BadLocationException e) {
				throw new AnnotationException(
						"Could not insert text into pane: " + e.getMessage());
			}
			if (punct2)
				punct2 = false;
			if (PUNCT2.matcher(raw_text).matches())
				punct2 = true;

			if (l.marked && l.tag == null) {
				setStyle(l, FONT_MARKABLE_TOPROCESS, false);
			} else if (l.marked && l.tag != null) {
				if (l.tag.isNonMarkableTag()) {
					setStyle(l, FONT_MARKABLE_NOJUNKTOR, false);
				} else
					setStyle(l, FONT_MARKABLE_PROCESSED, false);
			} else if (!l.isToken) {
				setStyle(l, FONT_OTHER, false);
			}
		}
		
		guiDoc.setCharacterAttributes(0, guiDoc.getLength(), guiTextPane.getStyle(FONT_BIG), false);
		setPositionToMarkable(0);
	}

	// redo Layout for Annotation after schema was changed
	public void redoLayout() throws AnnotationException {
		Iterator iter = caller.annDoc.tokens.iterator();
		while (iter.hasNext()) {
			Location l = (Location) iter.next();
			if (l.marked == true) {
				this.setStyle(l, FONT_NOT_CURRPOS, false);
				if (l.tag != null) {
					if (l.tag.isNonMarkableTag()) {
						this.setStyle(l, FONT_MARKABLE_NOJUNKTOR, false);
					} else
						this.setStyle(l, FONT_MARKABLE_PROCESSED, false);
				} else
					this.setStyle(l, FONT_MARKABLE_TOPROCESS, false);
			} else {
				this.setStyle(l, FONT_NO_MARKABLE, false);
			}
		}
		jumpToElement(getCurrentElementNum());
	}

	// do Layout for black and white printing
	public JTextPane getPrintLayout() throws AnnotationException {
		Iterator iter = caller.annDoc.allElements.iterator();
		JTextPane printPane = new JTextPane();
		Style bf = printPane.addStyle(FONT_BIG, null);
		StyleConstants.setFontSize(bf, 16);
		Style style = printPane.addStyle(FONT_BOLD, bf);
		StyleConstants.setBold(style, true);
		style = printPane.addStyle(FONT_ITAL, bf);
		StyleConstants.setItalic(style, true);
		DefaultStyledDocument printDoc = (DefaultStyledDocument) printPane.getStyledDocument();
		
		printPane.setText("");
		boolean punct2 = false;
		while (iter.hasNext()) {
			Location l = (Location) iter.next();
			boolean punct1 = false;
			Element el = (Element) l.node;
			String raw_text = "";

			if (l.isToken){
				raw_text = el.getAttribute(corpusFormat.getTokenDisplayAttr());							
			} else { // has to be an other (non-token display element)				
				raw_text = el.getAttribute(corpusFormat.getOtherDisplayAttr());
			}
			String text = "";
			if (raw_text.length() > 0) {
				if (PUNCT1.matcher(raw_text).matches()) {
					text = raw_text;
					punct1 = true;
				} else if (punct2)
					text = raw_text;
				else
					text = " " + raw_text;
			}
			try {
				Position start;
				printDoc.insertString(printDoc.getLength(), text, null);

				start = printDoc.createPosition(printDoc.getLength() - text.length());
				if (punct1 || punct2)
					start = printDoc.createPosition(printDoc.getLength()
							- text.length());
				else
					start = printDoc.createPosition(printDoc.getLength()
							- text.length() + 1);
				printPane.setCaretPosition(printDoc.getLength());
				if (punct2)
					punct2 = false;
				if (PUNCT2.matcher(raw_text).matches())
					punct2 = true;
	
				if (l.marked && l.tag!= null && !l.tag.isNonMarkableTag()) {
					printDoc.setCharacterAttributes(start.getOffset(), l.len, printPane.getStyle(FONT_BOLD), false);
					
					String hor = l.tag.getHorizontalDim().split(" ")[0];
					String ver = l.tag.getVerticalDim().split(" ")[0];
					String annText = " {" + hor + "," + ver + "}";
					printDoc.insertString(printDoc.getLength(), annText, null);
					start = printDoc.createPosition(printDoc.getLength() - annText.length());
					printPane.setCaretPosition(printDoc.getLength());
					printDoc.setCharacterAttributes(start.getOffset(), annText.length(), printPane.getStyle(FONT_ITAL), false);
						
				}
			} catch (BadLocationException e) {
				throw new AnnotationException(
						"Could not insert text into pane: " + e.getMessage());
			}

		}
		printDoc.setCharacterAttributes(0, printDoc.getLength(), printPane.getStyle(FONT_BIG), false);
		return printPane;
	}
	
	
	public void setStyle(Location l, String style, boolean replace) {
		guiDoc.setCharacterAttributes(l.start.getOffset(), l.len, guiTextPane
				.getStyle(style), replace);
	}

	private void installStylesOnTextPane() {
		Style bf = guiTextPane.addStyle(FONT_BIG, null);
		StyleConstants.setFontSize(bf, 16);

		Style black = guiTextPane.addStyle(FONT_NO_MARKABLE, bf);
		StyleConstants.setForeground(black, Color.BLACK);

		Style red = guiTextPane.addStyle(FONT_MARKABLE_TOPROCESS, bf);
		StyleConstants.setForeground(red, Color.RED);

		Style otherf = guiTextPane.addStyle(FONT_OTHER, bf);
		StyleConstants.setForeground(otherf, Color.GRAY);

		Style style = guiTextPane.addStyle(FONT_NOT_CURRPOS, bf);
		StyleConstants.setBold(style, false);

		style = guiTextPane.addStyle(FONT_BOLD, bf);
		StyleConstants.setBold(style, true);

		Style annotated = guiTextPane.addStyle(FONT_MARKABLE_PROCESSED, bf);
		StyleConstants.setForeground(annotated, new Color(0, 200, 0));

		Style ann_nojunktor = guiTextPane.addStyle(FONT_MARKABLE_NOJUNKTOR, bf);
		// StyleConstants.setForeground(ann_nojunktor, new Color(0, 125, 0));
		StyleConstants.setForeground(ann_nojunktor, Color.BLUE);

	}

	/** ****************** Layout ende ******************** */

	/** ****************** Move ****************************** */

	public Location getCurrentElement() {
		return caller.annDoc.allElements.elementAt(currentElementNumber);
	}

	public Location getCurrentToken() {
		return caller.annDoc.tokens.elementAt(currentTokenNumber);
	}

	public Location getCurrentMarkable() {
		return caller.annDoc.markables.elementAt(currentMarkableNumber);
	}

	public int getCurrentMarkableNum() {
		return currentMarkableNumber;
	}

	public int getCurrentTokenNum() {
		return currentTokenNumber;
	}

	public int getCurrentElementNum() {
		return currentElementNumber;
	}

	private void setPositionToElement(int i) {
		if (i < 0 || i >= caller.annDoc.getTotalCount())
			return;

		Location l; // unmark old position
		l = getCurrentElement();
		setStyle(l, FONT_NOT_CURRPOS, false);
		// if (l.node.getNodeName().equals(corpusFormat.getOtherTag()))
		if (!l.isToken)
			setStyle(l, FONT_OTHER, false);
		if (l.marked) {
			if (l.tag != null) {
				if (l.tag.isNonMarkableTag()) {
					setStyle(l, FONT_MARKABLE_NOJUNKTOR, false);
				} else
					setStyle(l, FONT_MARKABLE_PROCESSED, false);
			} else
				setStyle(l, FONT_MARKABLE_TOPROCESS, false);
		}
		currentElementNumber = i; // move

		l = getCurrentElement(); // mark new position
		guiTextPane.setCaretPosition(l.start.getOffset());
		setStyle(l, FONT_BOLD, false);
	}

	private void setPositionToToken(int t) {
		// System.err.println("Setting token position: " + t);
		if (t < 0 || t >= caller.annDoc.getTokenCount())
			return;
		currentTokenNumber = t; // move
		Location currentTokenLoc = getCurrentToken();
		if(currentTokenLoc.marked){
			currentMarkableNumber = caller.annDoc.markables.indexOf(currentTokenLoc);
		}
		int ind = caller.annDoc.allElements.indexOf(currentTokenLoc);
		setPositionToElement(ind);

	}

	private void setPositionToMarkable(int t) {
		// System.err.println("Setting markable position: " + t);
		if (t < 0 || t >= caller.annDoc.getMarkableCount())
			return;

		currentMarkableNumber = t;
		Location currentMarkableLocation = getCurrentMarkable();
		currentTokenNumber = caller.annDoc.tokens.indexOf(currentMarkableLocation);
		int i = caller.annDoc.allElements.indexOf(currentMarkableLocation);
		setPositionToElement(i);
	}

/*	public void jumpToElement(int i) {
		setPositionToElement(i);
		Location l = (Location)caller.annDoc.allElements.elementAt(i); 
		if(l.marked){
			currentMarkableNumber = caller.annDoc.markables.indexOf(l);
			currentTokenNumber = caller.annDoc.tokens.indexOf(l);
			return;
		} 
		boolean searchToken = true;
		if(l.isToken){
			currentTokenNumber = caller.annDoc.tokens.indexOf(l);
			searchToken = false;
		}		
		while (i > 0) {
			l = (Location) caller.annDoc.allElements.elementAt(--i);
			if(searchToken && l.isToken){
				currentTokenNumber = caller.annDoc.tokens.indexOf(l);
				searchToken = false;
			}
			if (l.marked) {
					currentMarkableNumber = caller.annDoc.markables.indexOf(l);
					return;
			}			
		}
		currentMarkableNumber = 0;
		if(searchToken){
			currentTokenNumber = 0;
		}
	}*/

	public void jumpToElement(int i) {
		setPositionToElement(i);
		Location l = caller.annDoc.allElements.elementAt(i); 
		if(l.marked){
			currentMarkableNumber = caller.annDoc.markables.indexOf(l);
			currentTokenNumber = caller.annDoc.tokens.indexOf(l);
			return;
		} 		
		if(l.isToken){
			currentTokenNumber = caller.annDoc.tokens.indexOf(l);			
		}		
	}
	
	public void jumpToToken(int i) {
		setPositionToToken(i);
		i = getCurrentElementNum();
		jumpToElement(i);
	}

	
	public Location move(String command) {
		if (Annotator.TOSTART.equals(command)) {
			if (currentMarkableNumber > 0) {
				setPositionToMarkable(0);
			}
		} else if (Annotator.TOEND.equals(command)) {
			if (currentMarkableNumber < caller.annDoc.getMarkableCount() - 1) {
				setPositionToMarkable(caller.annDoc.getMarkableCount() - 1);
			}
		} else if (Annotator.FWD_MARKABLE.equals(command)) {
			if (currentMarkableNumber < caller.annDoc.getMarkableCount()) {
				Location el = getCurrentElement();
				// if location is a markable, simply go to the next one
				if (el.marked){
					setPositionToMarkable(currentMarkableNumber + 1);				
				}// if location is a token, but not a markable, search for the next markable in forward direction and go there
				else if(el.isToken){
					int i = caller.annDoc.tokens.indexOf(el);
					while(!el.marked){
						if(i < caller.annDoc.getTokenCount()-1){
							el = caller.annDoc.tokens.elementAt(++i);
						} else {
							break;
						}
					}
					if(el.marked){
						i = caller.annDoc.markables.indexOf(el);
						setPositionToMarkable(i);
					}else{
						setPositionToMarkable(caller.annDoc.getMarkableCount()-1);
					}															
				}// if location is neither a token nor a markable, search for the next markable in forward direction and go there
				else{
					int i = caller.annDoc.allElements.indexOf(el);
					while(!el.marked){
						if(i < caller.annDoc.allElements.size() - 1){
							el = caller.annDoc.allElements.elementAt(++i);
						} else {
							break;
						}
					}
					if(el.marked){
						i = caller.annDoc.markables.indexOf(el);
						setPositionToMarkable(i);
					}else{
						setPositionToMarkable(caller.annDoc.getMarkableCount()-1);
					}																				
				}									
			}
		} else if (Annotator.BWD_MARKABLE.equals(command)) {
			Location el = getCurrentElement();
			// if location is a markable, simply go to the previous one
			if (el.marked){
				if (currentMarkableNumber > 0) {
					setPositionToMarkable(currentMarkableNumber - 1);
				}
			}// if location is a token, but not a markable, search for the previous markable
			else if(el.isToken){
				int i = caller.annDoc.tokens.indexOf(el);
				while(!el.marked){
					if(i > 0){
						el = caller.annDoc.tokens.elementAt(--i);
					} else {
						break;
					}
				}
				if(el.marked){
					i = caller.annDoc.markables.indexOf(el);
					setPositionToMarkable(i);
				}else{
					setPositionToMarkable(0);
				}															
			}// if location is neither a token nor a markable, search for the previous markable
			else{
				int i = caller.annDoc.allElements.indexOf(el);
				while(!el.marked){
					if(i > 0){
						el = caller.annDoc.allElements.elementAt(--i);
					} else {
						break;
					}
				}
				if(el.marked){
					i = caller.annDoc.markables.indexOf(el);
					setPositionToMarkable(i);
				} else {
					setPositionToMarkable(0);
				}																				
			}									
								
		} else if (Annotator.FWD_TOKEN.equals(command)) {
			if (currentTokenNumber < caller.annDoc.getTokenCount()) {
				Location el = getCurrentElement();
				// if location is a token, go to the next one
				if(el.isToken){
					setPositionToToken(currentTokenNumber + 1);
					
				}// if location is neither a token nor a markable, search for the next token in forward direction and go there
				else{
					int i = caller.annDoc.allElements.indexOf(el);
					while(!el.isToken){
						if(i < caller.annDoc.allElements.size() - 1){
							el = caller.annDoc.allElements.elementAt(++i);
						} else {
							break;
						}
					}
					if(el.isToken){
						i = caller.annDoc.tokens.indexOf(el);
						setPositionToToken(i);
					}else{
						setPositionToToken(caller.annDoc.getTokenCount()-1);
					}																				
				}									
			}												
		} else if (Annotator.BWD_TOKEN.equals(command)) {
			if (currentTokenNumber > 0) {
				Location el = getCurrentElement();
				// if location is a token, go to the next one
				if(el.isToken){
					setPositionToToken(currentTokenNumber - 1);
				}// if location is neither a token nor a markable, search for the next token in forward direction and go there
				else{
					int i = caller.annDoc.allElements.indexOf(el);
					while(!el.isToken){
						if(i > 0){
							el = caller.annDoc.allElements.elementAt(--i);
						} else {
							break;
						}
					}
					if(el.isToken){
						i = caller.annDoc.tokens.indexOf(el);
						setPositionToToken(i);
					}else{
						setPositionToToken(0);
					}																				
				}									
			}			}
		if (Annotator.TOSTART_TOKEN.equals(command)) {
			if (currentTokenNumber > 0) {
				setPositionToToken(0);
				currentMarkableNumber = 0;
			}
		} else if (Annotator.TOEND_TOKEN.equals(command)) {
			if (currentTokenNumber < caller.annDoc.getTokenCount() - 1) {
				setPositionToToken(caller.annDoc.getTokenCount() - 1);
				currentMarkableNumber = caller.annDoc.getMarkableCount() - 1;
			}
		} else if (Annotator.FWD_ALL.equals(command)) {
			if (currentElementNumber < caller.annDoc.getTotalCount()) {
				setPositionToElement(currentElementNumber + 1);
				Location l = getCurrentElement();
				if (l.isToken) {
					currentTokenNumber = caller.annDoc.tokens.indexOf(l);
					if (l.marked) {
						currentMarkableNumber = caller.annDoc.markables.indexOf(l);
					}
				}
				System.err.println("Current Element: " + currentElementNumber
						+ "Current token: " + currentTokenNumber
						+ ", Current markable: " + currentMarkableNumber);
			}
		} else if (Annotator.BWD_ALL.equals(command)) {
			if (currentElementNumber > 0) {
				setPositionToElement(currentElementNumber - 1);
				Location l = getCurrentElement();
				if (l.isToken) {
					currentTokenNumber = caller.annDoc.tokens.indexOf(l);
					if (l.marked) {
						currentMarkableNumber = caller.annDoc.markables.indexOf(l);
					}
				}
				System.err.println("Current Element: " + currentElementNumber
						+ "Current token: " + currentTokenNumber
						+ ", Current markable: " + currentMarkableNumber);

			}
		} else if (Annotator.TOSTART_ALL.equals(command)) {
			if (currentElementNumber > 0) {
				setPositionToElement(0);
				currentMarkableNumber = 0;
				currentTokenNumber = 0;
			}
		} else if (Annotator.TOEND_ALL.equals(command)) {
			if (currentElementNumber < caller.annDoc.getTotalCount() - 1) {
				setPositionToElement(caller.annDoc.getTotalCount() - 1);
				currentMarkableNumber = caller.annDoc.getMarkableCount() - 1;
				currentTokenNumber = caller.annDoc.getTokenCount() - 1;
			}
		}
		System.err.println("Current token: " + currentTokenNumber);
		return getCurrentElement();
	}

	
}
