/*
 * Created on 16.01.2005
 *  
 */
package org.sfb441.tradisc.annotation;

import javax.swing.text.*;
import javax.swing.*;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.Location;
import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.inputformat.CorpusFormat;
import org.sfb441.tradisc.schema.SchemaModel;
import org.sfb441.tradisc.schema.SchemaException;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

public class Annotator implements ActionListener {

	protected final static String TOEND = "toend";
	protected final static String TOSTART = "tostart";
	protected final static String FWD_MARKABLE = "forward_m";
	protected final static String FWD_DECISION = "forward_d";
	protected final static String BWD_MARKABLE = "backward_m";
	protected final static String BWD_DECISION = "backward_d";
	protected final static String FWD_UNDECIDED = "forward_u";
	protected final static String BWD_UNDECIDED = "backward_u";
	protected final static String FWD_TOKEN = "forward_t";
	protected final static String BWD_TOKEN = "backward_t";
	protected final static String TOEND_TOKEN = "toend_t";
	protected final static String TOSTART_TOKEN = "tostart_t";
	protected final static String FWD_ALL = "forward_a";
	protected final static String BWD_ALL = "backward_a";
	protected final static String TOEND_ALL = "toend_a";
	protected final static String TOSTART_ALL = "tostart_a";
	protected final static String ACTIVE_MARKABLE = "active markable changed";
	protected final static String JUMP_TO = "jump to token by number";

	// private static Pattern emptyStringPattern = Pattern.compile("^\\s*$");

	private GUI caller;

	private JButton toEndButton;
	private JButton toStartButton;
	private JButton fwdDecisionButton;
	private JButton bwdDecisionButton;
	private JButton fwdUndecidedButton;
	private JButton bwdUndecidedButton;
	private JButton fwdMarkableButton;
	private JButton bwdMarkableButton;

	private JDialog annotatorDialog;
	
	private JPanel tagChooserPane;
	private JComboBox markableBox;
	private JCheckBox ignoreNoJunktors;

	private JLabel tokenLabel;

	private JLabel numLabel;

	private JTextField tokenNumberField;

	private AnnotationModel ann;

	private boolean annotationHasChanged;

	private boolean annotationOpen;

	/**
	 * Annotations-Kontroll-Klasse: Konstruktor
	 * 
	 * @param caller
	 *            The calling GUI
	 */
	public Annotator(GUI caller) {
		this.caller = caller;
		createControllDialog();
	}

	public JTextPane getGuiTextPane() {
		return caller.getGuiTextPane();
	}

	public DefaultStyledDocument getGuiDoc() {
		return (DefaultStyledDocument) caller.getGuiTextPane()
				.getStyledDocument();
	}

	/* control frame inhalt */
	private void createControllDialog() {

		annotatorDialog = new JDialog(caller.getGuiFrame(), "annotation control");
		annotatorDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		annotatorDialog.setLocationRelativeTo(null);
		annotatorDialog.setResizable(false);

		// markable information
		JPanel markablePanel = new JPanel();
		markablePanel.setBorder(BorderFactory
				.createTitledBorder("token information"));
		tokenLabel = new JLabel(" ");
		numLabel = new JLabel(" ");
		markablePanel.add(numLabel);
		markablePanel.add(tokenLabel);
		markablePanel.setLayout(new GridLayout(2, 1));

		// Bewegungen im Korpus
		JPanel movementPanel = new JPanel(new BorderLayout());
		movementPanel.setBorder(BorderFactory.createTitledBorder("navigation"));
		JPanel buttonPane = new JPanel();
		// buttonPane.setBorder(BorderFactory.createTitledBorder("Movement
		// Buttons"));
		toStartButton = new JButton("|<");
		toStartButton.addActionListener(this);
		toStartButton.setActionCommand(TOSTART);
		buttonPane.add(toStartButton);
		toStartButton.setToolTipText("first markable");
		toEndButton = new JButton(">|");
		toEndButton.addActionListener(this);
		toEndButton.setActionCommand(TOEND);
		buttonPane.add(toEndButton);
		toEndButton.setToolTipText("last markable");
		bwdUndecidedButton = new JButton("?<");
		bwdUndecidedButton.addActionListener(this);
		bwdUndecidedButton.setActionCommand(BWD_UNDECIDED);
		buttonPane.add(bwdUndecidedButton);
		bwdUndecidedButton.setToolTipText("previous unannotated markable");
		fwdUndecidedButton = new JButton(">?");
		fwdUndecidedButton.addActionListener(this);
		fwdUndecidedButton.setActionCommand(FWD_UNDECIDED);
		buttonPane.add(fwdUndecidedButton);
		fwdUndecidedButton.setToolTipText("next unannotated markable");
		bwdDecisionButton = new JButton("<<");
		bwdDecisionButton.addActionListener(this);
		bwdDecisionButton.setActionCommand(BWD_DECISION);
		buttonPane.add(bwdDecisionButton);
		bwdDecisionButton.setToolTipText("previous ambivalent markable");
		fwdDecisionButton = new JButton(">>");
		fwdDecisionButton.addActionListener(this);
		fwdDecisionButton.setActionCommand(FWD_DECISION);
		fwdDecisionButton.setToolTipText("next ambivalent markable");
		buttonPane.add(fwdDecisionButton);
		bwdMarkableButton = new JButton("<");
		bwdMarkableButton.addActionListener(this);
		bwdMarkableButton.setActionCommand(BWD_MARKABLE);
		bwdMarkableButton.setToolTipText("previous markable");
		buttonPane.add(bwdMarkableButton);
		fwdMarkableButton = new JButton(">");
		fwdMarkableButton.addActionListener(this);
		fwdMarkableButton.setActionCommand(FWD_MARKABLE);
		fwdMarkableButton.setToolTipText("next markable");
		buttonPane.add(fwdMarkableButton);

		buttonPane.setLayout(new GridLayout(4, 2));

		// Markable whlen
		markableBox = new JComboBox();
		markableBox.addActionListener(this);
		markableBox.setActionCommand(ACTIVE_MARKABLE);

		ignoreNoJunktors = new JCheckBox("ignore non-junctors");
		JPanel p = new JPanel(new BorderLayout());
		JPanel markableChooserPane = new JPanel(/* new BorderLayout() */);
		markableChooserPane.setBorder(BorderFactory
				.createTitledBorder("active markable"));
		markableChooserPane.add(markableBox);
		p.add(markableChooserPane, BorderLayout.CENTER);
		p.add(ignoreNoJunktors, BorderLayout.SOUTH);

		JLabel label = new JLabel("jump to token nr:");
		tokenNumberField = new JTextField(3);
		JPanel fieldp = new JPanel();
		JButton go = new JButton("go!");
		go.addActionListener(this);
		go.setActionCommand(JUMP_TO);
		fieldp.add(label);
		fieldp.add(tokenNumberField);
		fieldp.add(go);
		p.add(fieldp, BorderLayout.NORTH);

		movementPanel.add(buttonPane, BorderLayout.CENTER);
		movementPanel.add(p, BorderLayout.SOUTH);
		// Tag auswhlen: JPanel dafr
		tagChooserPane = new JPanel();
		tagChooserPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		// contentPanel: alles zusammenpacken
		JPanel annotContentPane = new JPanel(new BorderLayout());
		annotContentPane.add(markablePanel, BorderLayout.NORTH);
		annotContentPane.add(movementPanel, BorderLayout.CENTER);
		// annotContentPane.add(markableChooserPane, BorderLayout.EAST);
		// annotContentPane.add(label);
		annotContentPane.add(tagChooserPane, BorderLayout.SOUTH);
		// contentPanel: alles zusammenpacken

		// alles zusammenpacken
		annotatorDialog.setContentPane(annotContentPane);
		annotatorDialog.pack();

		// Tastatursteuerung aktivieren
		// linker pfeil --> backward markable
		Action leftAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				System.err.println("[Annotator] " + BWD_MARKABLE);
				if (markableBox.getSelectedItem().equals("ALL MARKABLES")) {
					if (ignoreNoJunktors.isSelected()) {
						Location l = ann.move(BWD_MARKABLE);
						while ((l.tag != null && l.tag.isNonMarkableTag())
								&& !ann.isCurrentPositionEnd()
								&& !ann.isCurrentPositionStart())
							l = ann.move(BWD_MARKABLE);
					} else
						ann.move(BWD_MARKABLE);
				} else if (markableBox.getSelectedItem().equals("ALL TOKENS")) {
					ann.move(BWD_TOKEN);
				} else if (markableBox.getSelectedItem().equals("ALL ELEMENTS")) {
					ann.move(BWD_ALL);
				} else {
					Location l = ann.move(BWD_MARKABLE);
					Markable m = Markable.element2markable(l.node);
					Markable navm = (Markable) markableBox.getSelectedItem();
					while (((ignoreNoJunktors.isSelected() && l.tag != null && l.tag.isNonMarkableTag()
							) || !m.isCompatibleWith(navm))
							&& !ann.isCurrentPositionEnd()
							&& !ann.isCurrentPositionStart()) {
						l = ann.move(BWD_MARKABLE);
						m = Markable.element2markable(l.node);
					}
				}
				annotate(ann.getCurrentElementLocation());
			}
		};
		KeyStroke left = KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0, false);
		annotatorDialog.getRootPane().getInputMap(
				JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(left, "LEFT");
		annotatorDialog.getRootPane().getInputMap(
				JComponent.WHEN_FOCUSED).put(left, "LEFT");
		annotatorDialog.getRootPane().getActionMap().put("LEFT", leftAction);

		// rechter pfeil --> forward markable
		Action rightAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				System.err.println("[Annotator] " + FWD_MARKABLE);
				if (markableBox.getSelectedItem().equals("ALL MARKABLES")) {
					if (ignoreNoJunktors.isSelected()) {
						Location l = ann.move(FWD_MARKABLE);
						while ((l.tag != null && l.tag.isNonMarkableTag())
								&& !ann.isCurrentPositionEnd()
								&& !ann.isCurrentPositionStart())
							l = ann.move(FWD_MARKABLE);
					} else

						ann.move(FWD_MARKABLE);
				} else if (markableBox.getSelectedItem().equals("ALL TOKENS")) {
					ann.move(FWD_TOKEN);
				} else if (markableBox.getSelectedItem().equals("ALL ELEMENTS")) {
					ann.move(FWD_ALL);
				} else {
					Location l = ann.move(FWD_MARKABLE);
					Markable m = Markable.element2markable(l.node);
					Markable navm = (Markable) markableBox.getSelectedItem();
					while (((ignoreNoJunktors.isSelected() && l.tag != null && l.tag
							.isNonMarkableTag()) || !m.isCompatibleWith(navm))
							&& !ann.isCurrentPositionEnd()
							&& !ann.isCurrentPositionStart()) {
						l = ann.move(FWD_MARKABLE);
						m = Markable.element2markable(l.node);
					}
				}
				annotate(ann.getCurrentElementLocation());
			}
		};

		KeyStroke right = KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0, false);
		annotatorDialog.getRootPane().getInputMap(
				JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(right, "RIGHT");
		annotatorDialog.getRootPane().getInputMap(
				JComponent.WHEN_FOCUSED).put(right, "RIGHT");

		annotatorDialog.getRootPane().getActionMap().put("RIGHT", rightAction);

	}

	/**
	 * annotator frame anzeigen
	 */
	public void show() {
		annotatorDialog.setVisible(true);
	}

	/**
	 * annotator frame verbergen
	 */
	public void hide() {
		// wenn noch eine Tag-Auswahl angezeigt wird,
		// diese verbergen
		if (this.choicesShown) {
			choicesShown = false;
			tagChooserPane.removeAll();
			annotatorDialog.pack();
		}
		// textpane leeren
		caller.getGuiTextPane().setText("");
		// Annotation ist jetzt zu
		annotationOpen = false;
		// Frame verstecken
		annotatorDialog.setVisible(false);
	}

	/**
	 * true wenn sich seit dem Lade-/Erstellvorgang die Annotation verndert
	 * hat.
	 * 
	 * @return
	 */
	public boolean isAnnotationChanged() {
		return annotationOpen && annotationHasChanged;
	}

	/**
	 * true wenn eine Annotation geffnet ist
	 * 
	 * @return
	 */
	public boolean isAnnotationOpen() {
		return annotationOpen;
	}

	/**
	 * wenn eine schon bestehende Annotation geladen wird, wird das in der
	 * Annotations-Datendatei enthaltene Schema mitgeladen. Dieses muss auch in
	 * den Schema-Editor geladen werden.
	 */
	public void schemaReload(SchemaModel schema) {
		caller.getSchemaEditor().setSchema(schema); // Tabelle im Schema-Editor
													// mit neuem Schema versehen
	}
	
	/**
	 * wenn eine schon bestehende Annotation geladen wird, wird das in der
	 * Annotations-Datendatei enthaltene CorpusFormat mitgeladen. Dieses muss auch in
	 * den CorpusFormat-Editor geladen werden.
	 */
	public void corpusFormatReload(CorpusFormat corpusFormat) {
		caller.getCorpusFormatEditor().setCorpusFormat(corpusFormat);
	}
	

	public void reloadAnnotation() {
		ann.rebuildAnnotation();
		annotationHasChanged = true;
		caller.setHeader();
		loadActiveMarkables();
	}

	public JTextPane getPrintLayout() throws AnnotationException {
		try {
			JTextPane ret = ann.annView.getPrintLayout();
			return ret;
		} catch (AnnotationException e) {

			throw new AnnotationException(e.getMessage());
		}	
	}
	
	public void doStandardLayout(){
		try {
			ann.annView.doLayout(getInputFormat());
		} catch (AnnotationException e) {
			e.printStackTrace();
		}
	}
	
	private void setButtonToolTipTexts() {
		if (markableBox.getItemCount() > 0) {
			if (markableBox.getSelectedItem().equals("ALL TOKENS")) {
				toStartButton.setToolTipText("first token");
				toEndButton.setToolTipText("last token");
				bwdUndecidedButton.setToolTipText("previous token");
				fwdUndecidedButton.setToolTipText("next token");
				bwdDecisionButton.setToolTipText("previous token");
				fwdDecisionButton.setToolTipText("next token");
				bwdMarkableButton.setToolTipText("previous token");
				fwdMarkableButton.setToolTipText("next token");
			} else if (markableBox.getSelectedItem().equals("ALL ELEMENTS")) {
				toStartButton.setToolTipText("first element");
				toEndButton.setToolTipText("last element");
				bwdUndecidedButton.setToolTipText("previous element");
				fwdUndecidedButton.setToolTipText("next element");
				bwdDecisionButton.setToolTipText("previous element");
				fwdDecisionButton.setToolTipText("next element");
				bwdMarkableButton.setToolTipText("previous element");
				fwdMarkableButton.setToolTipText("next element");
			} else {
				toStartButton.setToolTipText("first markable");
				toEndButton.setToolTipText("last markable");
				bwdUndecidedButton
						.setToolTipText("previous unannotated markable");
				fwdUndecidedButton.setToolTipText("next unannotated markable");
				bwdDecisionButton
						.setToolTipText("previous ambivalent markable");
				fwdDecisionButton.setToolTipText("next ambivalent markable");
				bwdMarkableButton.setToolTipText("previous markable");
				fwdMarkableButton.setToolTipText("next markable");
			}
		}
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		System.err.println("[Annotator] " + command);
		if (JUMP_TO.equals(command)) {
			String str = tokenNumberField.getText();
			try {
				int i = Integer.parseInt(str);
				if (i < 1 || i > tokenCount())
					throw new Exception("Token number out of bounds");
				ann.annView.jumpToToken(--i);
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(annotatorDialog,
						"Enter a number between 1 and " + tokenCount() + "!",
						"Illegal input", JOptionPane.ERROR_MESSAGE);
				ex.printStackTrace();
			}
			tokenNumberField.setText("");
		} else if (ACTIVE_MARKABLE.equals(command)) {
			setButtonToolTipTexts();
		}
		// Button Movements
		else if (markableBox.getSelectedItem().equals("ALL MARKABLES")) {
			if (FWD_DECISION.equals(command)) {
				Vector v = null;
				Location l = new Location();
				do {
					l = ann.move(FWD_MARKABLE);
					v = getTagsForLocation(l);
					if (v.size() == 1) {
						Tag t = (Tag) v.elementAt(0);
						ann.setTagOnCurrentMarkable(t);
						System.err.println("Setting tag " + t.toString());
					}
				} while (v.size() < 2
						&& !ann.isCurrentPositionEnd()
						|| (ignoreNoJunktors.isSelected() && l.tag != null && l.tag.isNonMarkableTag()));
			} else if (BWD_DECISION.equals(command)) {
				Vector v = null;
				Location l = new Location();
				do {
					l = ann.move(BWD_MARKABLE);
					v = getTagsForLocation(l);
					if (v.size() == 1) {
						Tag t = (Tag) v.elementAt(0);
						ann.setTagOnCurrentMarkable(t);
						System.err.println("Setting tag " + t.toString());
					}
				} while (v.size() < 2
						&& !ann.isCurrentPositionStart()
						|| (ignoreNoJunktors.isSelected() && l.tag != null && l.tag.isNonMarkableTag()));
			} else if (FWD_UNDECIDED.equals(command)) {
				do
					ann.move(FWD_MARKABLE);
				while (ann.isCurrentMarkableTagged()
						&& !ann.isCurrentPositionEnd());
			} else if (BWD_UNDECIDED.equals(command)) {
				do
					ann.move(BWD_MARKABLE);
				while (ann.isCurrentMarkableTagged()
						&& !ann.isCurrentPositionStart());
			} else if (FWD_MARKABLE.equals(command)
					|| BWD_MARKABLE.equals(command)) {
				if (ignoreNoJunktors.isSelected()) {
					Location l = ann.move(command);
					while ((l.tag != null && l.tag.isNonMarkableTag())
							&& !ann.isCurrentPositionEnd()
							&& !ann.isCurrentPositionStart())
						l = ann.move(command);
				} else
					ann.move(command);
			} else
				ann.move(command);
		} else if (markableBox.getSelectedItem().equals("ALL TOKENS")) {
			if (FWD_DECISION.equals(command) || FWD_UNDECIDED.equals(command)
					|| FWD_MARKABLE.equals(command)) {
				ann.move(FWD_TOKEN);
			} else if (BWD_DECISION.equals(command)
					|| BWD_UNDECIDED.equals(command)
					|| BWD_MARKABLE.equals(command)) {
				ann.move(BWD_TOKEN);
			} else if (TOEND.equals(command)) {
				ann.move(TOEND_TOKEN);
			} else if (TOSTART.equals(command)) {
				ann.move(TOSTART_TOKEN);
			}
		} else if (markableBox.getSelectedItem().equals("ALL ELEMENTS")) {
			if (FWD_DECISION.equals(command) || FWD_UNDECIDED.equals(command)
					|| FWD_MARKABLE.equals(command)) {
				ann.move(FWD_ALL);
			} else if (BWD_DECISION.equals(command)
					|| BWD_UNDECIDED.equals(command)
					|| BWD_MARKABLE.equals(command)) {
				ann.move(BWD_ALL);
			} else if (TOEND.equals(command)) {
				ann.move(TOEND_ALL);
			} else if (TOSTART.equals(command)) {
				ann.move(TOSTART_ALL);
			}
		} else {
			Markable navm = (Markable) markableBox.getSelectedItem();
			if (FWD_DECISION.equals(command)) {
				Location l = ann.move(FWD_MARKABLE);
				Markable m = Markable.element2markable(l.node);
				Vector v = getTagsForLocation(l);
				while ((v.size() == 1
						|| (ignoreNoJunktors.isSelected() && l.tag != null && l.tag.isNonMarkableTag())
								|| !m.isCompatibleWith(navm))
						&& !ann.isCurrentPositionEnd()) {
					if (v.size() == 1 && m.isCompatibleWith(navm)) {
						Tag t = (Tag) v.elementAt(0);
						ann.setTagOnCurrentMarkable(t);
						System.err.println("Setting tag " + t.toString());
					}
					l = ann.move(FWD_MARKABLE);
					m = Markable.element2markable(l.node);
					v = getTagsForLocation(l);
				}
			} else if (BWD_DECISION.equals(command)) {
				Location l = ann.move(BWD_MARKABLE);
				Markable m = Markable.element2markable(l.node);
				Vector v = getTagsForLocation(l);
				while ((v.size() == 1
						|| (ignoreNoJunktors.isSelected() && l.tag != null && l.tag.isNonMarkableTag())
								 || !m.isCompatibleWith(navm))
						&& !ann.isCurrentPositionStart()) {
					if (v.size() == 1 && m.isCompatibleWith(navm)) {
						Tag t = (Tag) v.elementAt(0);
						ann.setTagOnCurrentMarkable(t);
						System.err.println("Setting tag " + t.toString());
					}
					l = ann.move(BWD_MARKABLE);
					m = Markable.element2markable(l.node);
					v = getTagsForLocation(l);
				}
			} else if (FWD_UNDECIDED.equals(command)) {
				Location l = ann.move(FWD_MARKABLE);
				Markable m = Markable.element2markable(l.node);
				while (!(m.isCompatibleWith(navm) && !ann
						.isCurrentMarkableTagged())
						&& !ann.isCurrentPositionEnd()) {
					l = ann.move(FWD_MARKABLE);
					m = Markable.element2markable(l.node);
				}
			} else if (BWD_UNDECIDED.equals(command)) {
				Location l = ann.move(BWD_MARKABLE);
				Markable m = Markable.element2markable(l.node);
				while (!(m.isCompatibleWith(navm) && !ann
						.isCurrentMarkableTagged())
						&& !ann.isCurrentPositionStart()) {
					l = ann.move(BWD_MARKABLE);
					m = Markable.element2markable(l.node);
				}
			} else if (FWD_MARKABLE.equals(command)
					|| BWD_MARKABLE.equals(command)) {
				Location l = ann.move(command);
				Markable m = Markable.element2markable(l.node);

				while (((ignoreNoJunktors.isSelected() && l.tag != null && l.tag.isNonMarkableTag())
						 || !m.isCompatibleWith(navm))
						&& !ann.isCurrentPositionEnd()
						&& !ann.isCurrentPositionStart()) {
					l = ann.move(command);
					m = Markable.element2markable(l.node);
				}
			} else
				ann.move(command);

		}
		/*
		 * if(markableBox.getSelectedItem().equals("ALL TOKENS")){
		 * clearChoices(ann.getCurrentTokenLocation()); }else
		 * annotate(ann.getCurrentLocation());
		 */
		annotate(ann.getCurrentElementLocation());
	}

	public void newAnnotation(File corpusFile, SchemaModel schema)
			throws AnnotationException {
		ann = new AnnotationModel(this);
		// Markable tokenInfo = askForTokenFormat();
		ann.build(corpusFile, schema);
		annotationOpen = true;
		annotationHasChanged = true;		
		loadActiveMarkables();
		show(); // show control frame
		annotate(ann.getCurrentMarkableLocation());
	}

	private void loadActiveMarkables() {
		markableBox.removeAllItems();
		Vector v = ann.annDoc.getSchema().getAllMarkables();
		markableBox.addItem("ALL MARKABLES");
		markableBox.addItem("ALL TOKENS");
		markableBox.addItem("ALL ELEMENTS");
		Iterator it = v.iterator();
		while (it.hasNext()) {
			markableBox.addItem(it.next());
		}

	}

	public String getSelectedMarkable() {
		return markableBox.getSelectedItem().toString();
	}

	public CorpusFormat getInputFormat() {
		return caller.getCorpusFormatEditor().getCorpusFormat();
	}

	public void openAnnotation(File annotationFile, File annotationDataFile)
			throws Exception {
		ann = new AnnotationModel(this);
		// Markable tokenInfo = askForTokenFormat();
		ann.rebuild(annotationFile, annotationDataFile);
		annotationOpen = true;
		annotationHasChanged = false;
		loadActiveMarkables();
		// annotatorFrame.pack();
		show(); // show control frame
		annotatorDialog.requestFocus();
		annotate(ann.getCurrentMarkableLocation());
	}

	public void mergeWithAnnotation(File mergeCorpusFile, File mergeDataFile)
			throws Exception {
		try {
			ann.mergeWith(mergeCorpusFile, mergeDataFile);

			// neues, zusammengefgtes Schema in den Schemaeditor laden
			schemaReload(ann.annDoc.getSchema());
			annotationOpen = true;
			loadActiveMarkables();
			reloadAnnotation();

			// annotatorFrame.pack();
			show(); // show control frame
			annotate(ann.getCurrentMarkableLocation());
		} catch (Exception e) {
			throw new Exception(e.getMessage());
		}
	}

	public void splitAnnotation(File corpusFile1, File dataFile1,
			File corpusFile2, File dataFile2) throws Exception {
		try {
			ann.splitAnnotation(corpusFile1, dataFile1, corpusFile2, dataFile2);
		} catch (Exception e) {
			throw new Exception(e.getMessage());
		}
	}

	public void saveAnnotation(File annotationFile, File annotationDataFile)
			throws AnnotationException, IOException, SchemaException {
		ann.save(annotationFile, annotationDataFile);
		annotationHasChanged = false;
	}

	private boolean choicesShown = false;

	private Tag currentChoice = null;

	private Vector choices = null;

	public Vector<Location> getMarkableLocations() {
		return ann.annDoc.markables;
	}

	public Vector<Location> getTokenLocations() {
		return ann.annDoc.tokens;
	}

	public Vector<Location> getAllElementLocations(){
		return ann.annDoc.allElements;
	}
	

	
	public int getTokenNumberFor(Location l) {
		return ann.annDoc.getTokenNumberFor(l);
	}

	private Vector getTagsForLocation(Location l) {
		// Vector v = l.tags;

		Markable m = Markable.element2markable(l.node);
		Vector v = caller.getSchemaEditor().getSchema().getTagsFor(m);
		l.tags = v;

		if (v.size() == 0) {
			System.err
					.println("Keine Tags gefunden, Element war aber markiert.");
			System.exit(1);
		}
		// show choices
		/*
		 * if (l.tag == null) { // default: choose first available tag l.tag =
		 * (Tag)l.tags.elementAt(0); annotationHasChanged = true;
		 * System.err.println("l.tag was null"); }
		 */
		return v;
	}

	public void setPositionToElement(int index){
		ann.annView.jumpToElement(index);
		annotate(ann.getCurrentElementLocation());
	}
	
	
	private void annotate(Location l) {
		// falls die aktuelle Position kein Markable ist, darf man nicht
		// annotieren, keine auswahl zeigen
		if (!l.marked) {
			choices = new Vector();
			choicesShown = false;
			tagChooserPane.removeAll();
			tokenLabel.setText(ann.getCurrentElementLocation().toString());
			if (l.isToken){
				numLabel.setText("token nr.: " + (ann.getCurrentTokenNumber() + 1));
			} else {
				numLabel.setText("token nr.: -- ");
			}
			annotatorDialog.pack();
			annotatorDialog.requestFocus();
			tagChooserPane.requestFocus();
			return;
		}
		choicesShown = true;
		choices = getTagsForLocation(l);
		JPanel choicePanel = new JPanel();
		choicePanel.setBorder(BorderFactory.createTitledBorder("select tag"));
		choicePanel.setLayout(new BoxLayout(choicePanel, BoxLayout.PAGE_AXIS));
		int actionCmdCreateCounter = 0;
		ButtonGroup bg = new ButtonGroup();
		if (l.tag != null) {
			Tag t = l.tag;
			if (t.isNonMarkableTag()){				
				l.tag = Tag.nonTag();
				t = Tag.nonTag();
			}
			if (!choices.contains(t)) {				
				l.tag = null;
			}
		}
		Iterator i = choices.iterator();
		while (i.hasNext()) {
			Tag buttonTag= (Tag)i.next();
			JRadioButton rb = new JRadioButton(buttonTag.toString());
			// Wenn schon ein Tag vorhanden ist und der gerade erstellte
			// Knopf dieses darstellt,
			if (l.tag != null && rb.getText().equals(l.tag.toString())) {
				// dann whle diesen Knopf aus
				rb.setSelected(true);
				currentChoice = l.tag;
			}
			// die annotation muss von nderungen in der Tagwahl benachrichtigt
			// werden.
			class ChoiceListener implements ActionListener {
				private Tag t;

				public ChoiceListener(Tag t) {
					this.t = t;
				}

				public void actionPerformed(ActionEvent e) {
					String cmd = e.getActionCommand();
					currentChoice = (Tag) choices.elementAt(Integer
							.parseInt(cmd));
					if (!currentChoice.equals(t)) {
						annotationHasChanged = true;
						caller.setHeader();
					}
					ann.setTagOnCurrentMarkable(currentChoice);
					System.err.println("[Annotator] Choice: " + cmd + " -> "
							+ currentChoice.toString());
				}
			}
			;
			rb.addActionListener(new ChoiceListener(l.tag));
			rb.setActionCommand(new Integer(actionCmdCreateCounter++)
					.toString());
			bg.add(rb);
			choicePanel.add(rb);
		}

		// einen Knopf zum deselecten einfgen
		JRadioButton rb = new JRadioButton("no tag selected");
		if (l.tag == null) {
			// dann whle diesen Knopf aus
			rb.setSelected(true);
		}
		// die annotation muss von nderungen in der Tagwahl benachrichtigt
		// werden.
		class ChoiceListener implements ActionListener {
			public ChoiceListener() {
			}

			public void actionPerformed(ActionEvent e) {
				ann.setNoTagOnCurrentMarkable();
				annotationHasChanged = true;
				caller.setHeader();
				System.err.println("[Annotator] Choice: No Tag selected");
			}
		}
		;
		rb.addActionListener(new ChoiceListener());
		rb.setActionCommand(new Integer(actionCmdCreateCounter++).toString());
		bg.add(rb);

		choicePanel.add(rb);
		tagChooserPane.removeAll();
		tagChooserPane.add(choicePanel);
		tagChooserPane.requestFocus();
		tagChooserPane.setFocusCycleRoot(true);
		tokenLabel.setText(ann.getCurrentMarkableLocation().toString());
		numLabel.setText("token nr.: " + (ann.getCurrentTokenNumber() + 1));
		annotatorDialog.pack();
	}

	// fr das evaluations-Modul
	public double getTagDistribution(Tag t, int constant) {
		// keine Annotation offen?
		if (this.ann == null)
			return 0.0;
		// not normalized
		if (constant <= 0) {
			return ann.getTagCount(t);
		}
		// normalized
		return ann.getNormalizedTagCount(t, constant);
	}

	public double getWindowTagDistribution(Tag t, int start, int end) {
		// keine Annotation offen?
		if (this.ann == null){
			return 0.0;
		}
		// not normalized
		return ann.getWindowTagCount(t, start, end);
		// normalized
	}

	public int getWindowMarkableCount(int start, int end) {
		// keine Annotation offen?
		if (this.ann == null){
			return 0;
		}
		return ann.getWindowMarkableCount(start, end);
	}

	
	public double getDistributionFor(Vector<Markable> v, Tag t, int normfactor) {
		// keine Annotation offen?
		if (this.ann == null)
			return 0.0;
		// not normalized
		if (normfactor <= 0) {
			return ann.getDistributionFor(v, t);
		}
		// normalized
		return ann.getNormalizedDistributionFor(v, t, normfactor);
	}

	public int countAnnotatedMarkables() {
		return ann.countAnnotatedMarkables();
	}
	
	public int tokenCount() {
		if (this.ann == null)
			return 0;
		return ann.annDoc.getTokenCount();
	}

	public void getFocus(){
		annotatorDialog.requestFocus();
		tagChooserPane.requestFocus();
	}

}