package org.sfb441.tradisc.eval;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import org.sfb441.tradisc.schema.SchemaModel;

	class CombinationList implements ActionListener {

		private double globaltokencount;

		private File currentDir;

		private JDialog dialog;

		private JList combineList;
		
		private EvaluationTable caller;

		private DefaultListModel listModel;

		private Hashtable<String, double[][]> datas;

		private Hashtable<String, Integer> normfactors;

		private Hashtable<String, Integer> tokencounts;

		private static final String LOADCSV = "load evaluation .csv-file";

		private static final String REMOVE = "remove evaluation .csv-file form list";

		private static final String CLOSELIST = "close evaluation file list window";

		public CombinationList(EvaluationTable caller) {
			this.caller = caller;
			globaltokencount = 0.0;
			datas = new Hashtable<String, double[][]>();
			normfactors = new Hashtable<String, Integer>();
			tokencounts = new Hashtable<String, Integer>();
			listModel = new DefaultListModel();
			combineList = new JList(listModel);
			combineList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			combineList.setLayoutOrientation(JList.VERTICAL);
			combineList.setVisibleRowCount(-1);
			JScrollPane listScroller = new JScrollPane(combineList);
			listScroller.setPreferredSize(new Dimension(250, 80));
			JButton loadButton = new JButton("add evaluation file");
			loadButton.setActionCommand(LOADCSV);
			loadButton.addActionListener(this);
			JButton removeButton = new JButton("remove evaluation file");
			removeButton.setActionCommand(REMOVE);
			removeButton.addActionListener(this);
			JButton closeb = new JButton("close");
			closeb.setActionCommand(CLOSELIST);
			closeb.addActionListener(this);
			JPanel buttonPane = new JPanel();
			buttonPane.add(loadButton);
			buttonPane.add(removeButton);
			buttonPane.add(closeb);
			JPanel contentPane = new JPanel();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(listScroller, BorderLayout.CENTER);
			contentPane.add(buttonPane, BorderLayout.SOUTH);
			dialog = new JDialog(caller.getDialog(), "current evaluation files",
					true);
			// dialog.setTitle("Current evaluation files");
			dialog.setContentPane(contentPane);
			dialog.setLocationRelativeTo(caller.getDialog());
			dialog.pack();
		}

		public void show() {
			dialog.setVisible(true);
		}

		private void load() {
			JFileChooser fc = new JFileChooser();
			if (currentDir != null)
				fc.setCurrentDirectory(currentDir);
			int n = fc.showOpenDialog(dialog);
			// wenn nicht "cancel"
			File csvfile = null;
			fc.setDialogTitle("open evaluation .csv file");
			if (n == JFileChooser.APPROVE_OPTION) {
				csvfile = fc.getSelectedFile();
				currentDir = fc.getCurrentDirectory();
			} else {
				JOptionPane.showMessageDialog(dialog, "Operation canceled!");
				return;
			}
			if (listModel.contains(csvfile.getName())) {
				JOptionPane.showMessageDialog(dialog,
						"File is already loaded!", "",
						JOptionPane.WARNING_MESSAGE);
				return;
			}
			try {
				SchemaModel schema = caller.getSchema();
				FileReader fr = new FileReader(csvfile);
				BufferedReader reader = new BufferedReader(fr);
				String s = reader.readLine();
				String[] colstrings = s.split(";", -1);

				if (schema.getColumnCount() != colstrings.length) {
					JOptionPane.showMessageDialog(dialog,
							"File is not compatible!", "File loading Error",
							JOptionPane.ERROR_MESSAGE);
					return;
				}
				for (int i = 0; i < colstrings.length; i++) {
					if (!schema.getColumnName(i).equals(colstrings[i])) {
						JOptionPane
								.showMessageDialog(dialog,
										"File is not compatible!",
										"File loading Error",
										JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
				s = reader.readLine();
				String[] spline = s.split(";", -1);
				int rownum = 0;
				double[][] data = new double[schema.getRowCount()][schema
						.getColumnCount() - 1];
				while (s != null && rownum < schema.getRowCount()) {
					spline = s.split(";", -1);
					if (spline[0].equals(schema.getRowName(rownum))) {
						for (int i = 1; i < spline.length; i++) {
							data[rownum][i - 1] = Double.parseDouble(spline[i]);
						}
						rownum++;
					} else {
						JOptionPane
								.showMessageDialog(dialog,
										"File is not compatible!",
										"File loading Error",
										JOptionPane.ERROR_MESSAGE);
						return;
					}
					s = reader.readLine();
				}
				spline = s.split(";", -1);
				Integer normFactor = Integer.valueOf(Integer
						.parseInt(spline[1]));
				s = reader.readLine();
				spline = s.split(";", -1);
				Integer tokenCount = Integer.valueOf(Integer
						.parseInt(spline[1]));
				caller.setCombinedLoaded(true);
				String filename = csvfile.getName();
				datas.put(filename, data);
				normfactors.put(filename, normFactor);
				tokencounts.put(filename, tokenCount);
				listModel.addElement(filename);
				updateGlobalTokenCount();
				combineList.setModel(listModel);
				caller.setViewCombined(true);
				dialog.pack();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(dialog,
						"File is not compatible!", "File loading Error",
						JOptionPane.ERROR_MESSAGE);
			}
		}

		private void remove() {
			if (combineList.isSelectionEmpty()) {
				JOptionPane.showMessageDialog(dialog, "Select a filename",
						"No file selected", JOptionPane.WARNING_MESSAGE);
			} else {
				String filename = (String) combineList.getSelectedValue();
				datas.remove(filename);
				normfactors.remove(filename);
				tokencounts.remove(filename);
				listModel.removeElement(filename);
				combineList.setModel(listModel);
				updateGlobalTokenCount();
				if (listModel.getSize() == 0) {
					caller.setCombinedLoaded(false);
					caller.setViewCombined(false);
				} else{
					caller.setViewCombined(true);
				}
				dialog.pack();
			}
		}

		public void clear() {
			datas.clear();
			normfactors.clear();
			tokencounts.clear();
			listModel.clear();
			combineList.setModel(listModel);
			caller.setCombinedLoaded(false);
			updateGlobalTokenCount();

			dialog.pack();
		}

		public Integer getTokenCount(String filename) {
			if (tokencounts.containsKey(filename))
				return tokencounts.get(filename);
			else
				return null;
		}

		public Integer getNormFactor(String filename) {
			if (normfactors.containsKey(filename))
				return normfactors.get(filename);
			else
				return null;
		}

		public Double getValueAt(String filename, int row, int col) {
			if (this.datas.containsKey(filename)) {
				double[][] dat =  datas.get(filename);
				return Double.valueOf(dat[row][col]);
			} else
				return null;
		}

		public Enumeration getLoadedFiles() {
			return listModel.elements();
		}

		private void updateGlobalTokenCount() {
			double res = caller.getAnnotator().tokenCount();
			Enumeration e = listModel.elements();
			while (e.hasMoreElements()) {
				String f = (String) e.nextElement();
				res += ( normfactors.get(f)).doubleValue();
			}
			globaltokencount = res;
		}

		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			if (cmd.equals(LOADCSV)) {
				load();
			} else if (cmd.equals(REMOVE)) {
				remove();
			} else if (cmd.equals(CLOSELIST)) {
				dialog.setVisible(false);
			}
		}


		public double[][] getData(String filename) {
			if(datas.containsKey(filename)){
				return datas.get(filename);		
			}else{
				return null;
			}
		
		}

		protected double getGlobalTokenCount() {
			return globaltokencount;
		}


	}
