package org.sfb441.tradisc.eval;

import java.util.Vector;

import javax.swing.table.*;

import java.text.NumberFormat;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.AbstractAction;
import javax.swing.DefaultCellEditor;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.schema.SchemaModel;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

public class ComplexityTable implements ActionListener {

	private JTable table;

	private JDialog tableDialog;

	private JLabel label;

	private GUI caller;

	private ComplexityModel model;

	private String filename;

	private boolean changed;

	private static final String LOAD = "Load complexity score table";

	private static final String SAVE = "Save complexity score table";

	private static final String SET_DEFAULT = "Set default complexity score table";

	private static final String CLOSE = "Close";

	public ComplexityTable(GUI caller) {
		this.caller = caller;
		model = new ComplexityModel(caller);
		table = new JTable(model);
		changed = false;
		tableDialog = new JDialog(caller.getGuiFrame(),
				"complexity score editor", true);
		tableDialog.setLocationRelativeTo(caller.getGuiFrame());
		tableDialog.setResizable(true);
		tableDialog.setContentPane(createContentPane());
		tableDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		tableDialog.pack();
	}

	private JPanel createContentPane() {
		table = new JTable(model);
		table.getTableHeader().setReorderingAllowed(false);
		table.setDragEnabled(false);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setCellSelectionEnabled(true);
		table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {
			public void setValue(Object value) {
				if (value.getClass() == String.class) {
					this.setOpaque(true);
					setBackground(Color.LIGHT_GRAY);
				} else {
					setBackground(Color.WHITE);
				}
				setText(value.toString());
			}
		});
		table.setDefaultEditor(Object.class, new DoubleEditor());

		table.setPreferredScrollableViewportSize(new Dimension(500, 120));
		tableDialog.addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent ce) {
				table.requestFocusInWindow();
				table.changeSelection(0, 1, false, false);
			}
		});
		// Create the scroll pane and add the table to it.
		JPanel contentPane = new JPanel(new BorderLayout());
		label = new JLabel("default complexity table");
		JPanel p = new JPanel();
		p.add(label);
		contentPane.add(p, BorderLayout.NORTH);
		JScrollPane scrollPane = new JScrollPane(table);
		p = new JPanel();
		p.add(scrollPane);
		contentPane.add(p);
		JButton loadButton = new JButton("load");
		loadButton.setActionCommand(LOAD);
		loadButton.addActionListener(this);
		p = new JPanel();
		p.add(loadButton);
		JButton saveButton = new JButton("save");
		saveButton.setActionCommand(SAVE);
		saveButton.addActionListener(this);
		p.add(saveButton);
		JButton standardButton = new JButton("set default scores");
		standardButton.setActionCommand(SET_DEFAULT);
		standardButton.addActionListener(this);
		p.add(standardButton);
		JButton closeButton = new JButton("Close");
		closeButton.setActionCommand(CLOSE);
		closeButton.addActionListener(this);
		p.add(closeButton);
		contentPane.add(p, BorderLayout.SOUTH);
		return contentPane;
	}

	public void actionPerformed(ActionEvent ae) {
		String cmd = ae.getActionCommand();
		System.err.println("[ComplexityTable] " + cmd);
		if (cmd.equals(CLOSE)) {
			dispose();
		} else if (cmd.equals(SAVE)) {
			model.save();
		} else if (cmd.equals(LOAD)) {
			model.load();
		} else if (cmd.equals(SET_DEFAULT)) {
			model.setDefault();
		}

	}

	public void ensureCompatibility() {
		if (!model.isCompatibleWithSchema(caller.getSchemaEditor().getSchema())) {
			if (isComplexityTableChanged()) {
				int n = JOptionPane
						.showConfirmDialog(
								caller.getGuiFrame(),
								new String(
										"Current complexity table is not compatible with schema\n"
												+ "and has unsaved changes.\n"
												+ "Save complexity table before setting to compatible default tabel?"),
								"Complexity table not compatible",
								JOptionPane.YES_NO_OPTION,
								JOptionPane.WARNING_MESSAGE);
				if (n == JOptionPane.YES_OPTION)
					model.save();
				model.setDefault();
			} else
				model.setDefault();
		}
	}

	public ComplexityModel getComplexityModel() {
		return model;
	}

	public void show() {
		// if(!model.isCompatibleWithSchema(caller.getSchemaEditor().getSchema()))
		// model.setDefault();
		tableDialog.pack();
		tableDialog.setVisible(true);
	}

	public void dispose() {
		if (!model.isCompatibleWithSchema(caller.getSchemaEditor().getSchema()))
			model.setDefault();
		tableDialog.setVisible(false);
	}

	public boolean isComplexityTableChanged() {
		return changed;
	}

	private void setHeader() {
		if (label != null) {
			if (changed)
				label.setText("unsaved complexity table");
			else {
				if (filename != null)
					label.setText("table in file '" + filename + "'");
				else
					label.setText("default complexity table");
			}
			tableDialog.pack();
		}
	}

	public class ComplexityModel extends AbstractTableModel {

		private Object[][] data;

		private GUI gui;

		private int rows;

		private int cols;

		private Vector<String> colnames;

		public ComplexityModel(GUI gui) {
			this.gui = gui;
			setDefault();
		}

		public void setDefault() {
			System.err.println("setting standard complexity table");
			SchemaModel schema = gui.getSchemaEditor().getSchema();
			rows = schema.getRowCount();
			cols = schema.getColumnCount();
			data = new Object[rows][cols];
			colnames = new Vector<String>();
			colnames.add("");
			for (int row = 0; row < rows; row++) {
				data[row][0] = schema.getRowName(row);
			}
			for (int col = 1; col < cols; col++)
				colnames.add(schema.getColumnName(col));
			for (int row = 0; row < rows; row++)
				for (int col = 1; col < cols; col++) {
					Double score = Double.valueOf((col + 1) * (row + 1));
					data[row][col] = score;
				}
			changed = false;
			filename = null;
			setHeader();
			this.fireTableStructureChanged();
		}

		private void load() {
			JFileChooser fc = new JFileChooser();
			int n = fc.showOpenDialog(tableDialog);
			// wenn nicht "cancel"
			File csvfile = null;
			fc.setDialogTitle("open complexity table .csv file");
			if (n == JFileChooser.APPROVE_OPTION) {
				csvfile = fc.getSelectedFile();
			} else {
				JOptionPane.showMessageDialog(tableDialog,
						"operation canceled!");
				return;
			}
			try {
				BufferedReader reader = new BufferedReader(new FileReader(
						csvfile));
				String s = reader.readLine();
				String[] colstrings = s.split(";", -1);
				SchemaModel schema = gui.getSchemaEditor().getSchema();
				if (schema.getColumnCount() != colstrings.length) {
					JOptionPane.showMessageDialog(tableDialog, "Table in '"
							+ csvfile.getName()
							+ "' has different dimensions than schema table",
							"File not compatible!", JOptionPane.ERROR_MESSAGE);
					return;
				}
				for (int i = 0; i < colstrings.length; i++) {
					if (!schema.getColumnName(i).equals(colstrings[i])) {
						JOptionPane
								.showMessageDialog(
										tableDialog,
										"Table in '"
												+ csvfile.getName()
												+ "' has different column names than schema table",
										"File not compatible!",
										JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
				s = reader.readLine();
				String[] spline = s.split(";", -1);
				int rownum = 0;
				Object[][] newData = new Object[schema.getRowCount()][schema
						.getColumnCount()];
				Vector<String> newColnames = new Vector<String>();
				newColnames.add("");
				for (int col = 1; col < cols; col++)
					newColnames.add(schema.getColumnName(col));

				while (s != null && rownum < schema.getRowCount()) {
					spline = s.split(";", -1);
					if (spline[0].equals(schema.getRowName(rownum))) {
						newData[rownum][0] = spline[0];
						for (int i = 1; i < spline.length; i++) {
							newData[rownum][i] = Double.valueOf(spline[i]);
						}
						rownum++;
					} else {
						JOptionPane
								.showMessageDialog(
										tableDialog,
										"Table in '"
												+ csvfile.getName()
												+ "' has different row names than schema table",
										"File not compatible!",
										JOptionPane.ERROR_MESSAGE);
						return;
					}
					s = reader.readLine();
				}
				if (s != null) {
					JOptionPane.showMessageDialog(tableDialog, "Table in '"
							+ csvfile.getName()
							+ "' has different dimensions than schema table",
							"File not compatible!", JOptionPane.ERROR_MESSAGE);
					return;
				}
				data = newData;
				colnames = newColnames;
				rows = schema.getRowCount();
				cols = schema.getColumnCount();
				changed = false;
				filename = csvfile.getName();
				setHeader();
				fireTableStructureChanged();
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(tableDialog,
						"File is not compatible!", "Error Message",
						JOptionPane.ERROR_MESSAGE);
			}
		}

		private void save() {
			JFileChooser fc = new JFileChooser();
			fc.setDialogTitle("save complexity score table");
			int n = fc.showSaveDialog(tableDialog);
			if (n != JFileChooser.APPROVE_OPTION)
				return;
			File file = fc.getSelectedFile();
			boolean exists = file.exists();
			while (exists) {
				n = JOptionPane.showConfirmDialog(tableDialog, file.getName()
						+ " already exists. Overwrite?", "Please confirm",
						JOptionPane.YES_NO_OPTION);
				if (n == JOptionPane.YES_OPTION)
					exists = false;
				else if (n == JOptionPane.NO_OPTION) {
					n = fc.showSaveDialog(tableDialog);
					if (n == JFileChooser.APPROVE_OPTION)
						file = fc.getSelectedFile();
					exists = file.exists();
					if (n == JFileChooser.CANCEL_OPTION) {
						JOptionPane.showMessageDialog(tableDialog,
								"Operation canceled!");
						return;
					}
				}
			}
			BufferedWriter w;
			try {
				w = new BufferedWriter(new FileWriter(file));
				for (int i = 1; i < model.getColumnCount(); i++) {
					w.write(";");
					w.write(model.getColumnName(i));
				}
				w.write("\n");
				for (int row = 0; row < model.getRowCount(); row++) {
					w.write(model.getRowName(row));
					for (int c = 1; c < model.getColumnCount(); c++) {
						w.write(";");
						w.write(model.getValueAt(row, c).toString());
					}
					w.write("\n");
				}
				w.close();
				changed = false;
				filename = file.getName();
				setHeader();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null,
						"Error while saving Complexity table!");
				e.printStackTrace();
				return;
			}
		}

		/* from super */
		public String getColumnName(int col) {
			return colnames.get(col);
		}

		/* extra */
		public String getRowName(int row) {
			return (String) data[row][0];
		}

		public int getRowCount() {
			return rows;
		}

		public int getColumnCount() {
			return cols;
		}

		public void setValueAt(int row, int col, Object value) {
			if (row < rows && col < cols && col > 0 && row >= 0) {
				if (!data[row][col].equals(value)) {
					data[row][col] = value;
					changed = true;
					setHeader();
				}
			}
			fireTableCellUpdated(row, col);
		}

		public Object getValueAt(int row, int col) {
			if (row < rows && col < cols && col >= 0 && row >= 0)
				return data[row][col];
			return null;
		}

		public boolean isCellEditable(int row, int col) {
			if (col == 0) {
				return false;
			} else {
				return true;
			}
		}

		public boolean isCompatibleWithSchema(SchemaModel schema) {
			if (cols != schema.getColumnCount() || rows != schema.getRowCount())
				return false;
			for (int r = 0; r < rows; r++) {
				if (!getRowName(r).equals(schema.getRowName(r)))
					return false;
			}
			for (int c = 1; c < cols; c++) {
				if (!getColumnName(c).equals(schema.getColumnName(c)))
					return false;
			}
			return true;
		}

	}

	/**
	 * Implements a cell editor that uses a formatted text field to edit Double
	 * values.
	 */
	private class DoubleEditor extends DefaultCellEditor {
		JFormattedTextField ftf;

		NumberFormat doubleFormat;

		int row;

		int col;

		public DoubleEditor() {
			super(new JFormattedTextField());
			ftf = (JFormattedTextField) getComponent();
			doubleFormat = NumberFormat.getInstance();
			NumberFormatter doubleFormatter = new NumberFormatter(doubleFormat);
			doubleFormatter.setFormat(doubleFormat);
			ftf.setValue(Double.valueOf(0.0));
			ftf.setFocusLostBehavior(JFormattedTextField.PERSIST);
			ftf
					.setFormatterFactory(new DefaultFormatterFactory(
							doubleFormatter));
			ftf.setHorizontalAlignment(JTextField.TRAILING);
			// React when the user presses Enter while the editor is
			// active. (Tab is handled as specified by
			// JFormattedTextField's focusLostBehavior property.)
			ftf.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
					"check");
			ftf.getActionMap().put("check", new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					if (!ftf.isEditValid()) { // The text is invalid.
						// ftf.selectAll();
						JOptionPane.showMessageDialog(table,
								"The value must be a number. ",
								"Invalid Text Entered",
								JOptionPane.ERROR_MESSAGE);

						ftf.setValue(ftf.getValue());
						ftf.postActionEvent(); // inform the editor
					} else
						try { // The text is valid,
							ftf.commitEdit(); // so use it.
							ftf.postActionEvent(); // stop editing
						} catch (java.text.ParseException exc) {
						}
				}
			});
		}

		// Override to invoke setValue on the formatted text field.
		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int column) {
			this.row = row;
			this.col = column;
			JFormattedTextField ftf = (JFormattedTextField) super
					.getTableCellEditorComponent(table, value, isSelected, row,
							column);
			ftf.setValue(value);
			return ftf;
		}

		// Override to ensure that the value remains an Double.
		public Object getCellEditorValue() {
			JFormattedTextField ftf = (JFormattedTextField) getComponent();
			Object o = ftf.getValue();
			if (ftf.isEditValid()) {
				if (o.getClass().equals(Double.class)) {
					System.err.println("returning" + o.toString());
					model.setValueAt(row, col, o);
					return o;
				} else {
					o = Double.valueOf(o.toString());
					System.err.println("returning" + o.toString());
					model.setValueAt(row, col, o);
					return o;
				}
			} else {
				try {
					return doubleFormat.parseObject(o.toString());
				} catch (Exception exc) {
					System.err.println("getCellEditorValue: can't parse o: "
							+ o);
					return null;
				}
			}
		}

		// Override to check whether the edit is valid,
		// setting the value if it is and complaining if
		// it isn't. If it's OK for the editor to go
		// away, we need to invoke the superclass's version
		// of this method so that everything gets cleaned up.
		public boolean stopCellEditing() {
			JFormattedTextField ftf = (JFormattedTextField) getComponent();
			if (ftf.isEditValid()) {
				try {
					ftf.commitEdit();
				} catch (java.text.ParseException exc) {
				}
			} else { // text is invalid
				JOptionPane.showMessageDialog(table,
						"The value must be a number. ", "Invalid Text Entered",
						JOptionPane.ERROR_MESSAGE);
				ftf.setValue(ftf.getValue());
				return false; // don't let the editor go away
			}
			return super.stopCellEditing();
		}

	}
}