package org.sfb441.tradisc.eval;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.AbstractCellEditor;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;

import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;

class EvaluationCellViewer extends AbstractCellEditor implements TableCellEditor,
		ActionListener {

	// system
	private EvaluationTable caller;

	// command constants
	protected final static String VIEW_CELL = "view cell";

	protected final static String CLOSE = "close";

	protected final static String NORMALIZE = "normalize";

	protected final static String EXPORT = "export";

	protected final static String NORM_EXPORT = "normalized export";

	protected JDialog editorDialog;

	private JButton tableCellButton;

	private JLabel normLabel;

	// manipulate display mode buttons
	private JButton normalizeButton;

	private JButton exportButton;

	private JButton nexportButton;

	private boolean normalized;

	// cell content

	private JTable cellContentTable;

	private DefaultTableModel cellContentTableModel;

	private Vector<Markable> currentMarkables;

	private Tag currentTag;

	public EvaluationCellViewer(EvaluationTable caller) {
		this.caller = caller;

		// bring up cell "button". Alle Zellen sind ein Knopf.
		// Auf diese Art kann man leichter Ereignisse dranhngen.
		tableCellButton = new JButton();
		tableCellButton.setActionCommand(VIEW_CELL);
		tableCellButton.addActionListener(this);
		tableCellButton.setBorderPainted(false);

		// "close editor" button with its panel
		JButton closeButton = new JButton("close");
		closeButton.setActionCommand(CLOSE);
		closeButton.addActionListener(this);
		JPanel buttonPane = new JPanel();
		buttonPane.add(closeButton);

		// table als zellinhalt
		cellContentTableModel = new DefaultTableModel();
		cellContentTable = new JTable(cellContentTableModel);
		cellContentTable.setPreferredScrollableViewportSize(new Dimension(250,
				100));
		JScrollPane cellContentScrollPane = new JScrollPane(cellContentTable);

		// the buttons
		// normalize
		normalizeButton = new JButton("normalize");
		normalizeButton.setMnemonic(KeyEvent.VK_N);
		normalizeButton.setActionCommand(NORMALIZE);
		normalizeButton.addActionListener(this);
		normalized = false;

		// normalized label
		normLabel = new JLabel("");
		setLabel();
		JPanel normalizedPanel = new JPanel();
		normalizedPanel.setBorder(BorderFactory.createEmptyBorder());
		normalizedPanel.add(normLabel);

		// exportieren
		exportButton = new JButton("export");
		exportButton.setMnemonic(KeyEvent.VK_E);
		exportButton.setActionCommand(EXPORT);
		exportButton.addActionListener(this);

		nexportButton = new JButton("norm. export");
		nexportButton.setMnemonic(KeyEvent.VK_X);
		nexportButton.setActionCommand(NORM_EXPORT);
		nexportButton.addActionListener(this);

		// panel for the buttons
		JPanel modPanel = new JPanel();
		modPanel.setLayout(new GridLayout(4, 1));
		modPanel.setBorder(BorderFactory.createTitledBorder("controls"));
		modPanel.add(normalizeButton);
		modPanel.add(exportButton);
		modPanel.add(nexportButton);
		modPanel.add(new JLabel(" "));

		// stuff everything together
		JPanel dialogContentPane = new JPanel(new BorderLayout());
		dialogContentPane.add(normalizedPanel, BorderLayout.NORTH);
		dialogContentPane.add(cellContentScrollPane, BorderLayout.CENTER);
		dialogContentPane.add(modPanel, BorderLayout.LINE_END);
		dialogContentPane.add(buttonPane, BorderLayout.PAGE_END);

		// the dialog itself
		editorDialog = new JDialog(caller.getDialog(), "distribution", false);
		editorDialog.setLocationRelativeTo(caller.getDialog());
		editorDialog.setContentPane(dialogContentPane); // content pane
		// setzen
		editorDialog.pack();

		// enable dialog closing via ESC
		Action escapeAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		};
		KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
		editorDialog.getRootPane().getInputMap(
				JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
		editorDialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);
	}

	public void dispose() {
		// notify the renderer, he has to take over again
		fireEditingStopped();
		// hide the dialog
		editorDialog.setVisible(false);
		// show table
		caller.getDialog().setVisible(true);
	}

	private void setLabel() {
		String s = "";
		if (!normalized) {
			s = "not normalized.";
		} else {
			s = "normalized to " + Integer.toString(caller.getNormfactor())
					+ " words.";
		}
		normLabel.setText(s);
	}

	private void export(boolean normalize) {
		// wohin?
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save cell Evaluation");
		int n = fc.showSaveDialog(editorDialog);
		if (n != JFileChooser.APPROVE_OPTION)
			return;
		File file = fc.getSelectedFile();
		boolean exists = file.exists();
		while (exists) {
			n = JOptionPane.showConfirmDialog(caller.getDialog(), file.getName()
					+ " already exists. Overwrite?", "Please confirm",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION)
				exists = false;
			else if (n == JOptionPane.NO_OPTION) {
				n = fc.showSaveDialog(editorDialog);
				if (n == JFileChooser.APPROVE_OPTION)
					file = fc.getSelectedFile();
				exists = file.exists();
				if (n == JFileChooser.CANCEL_OPTION) {
					JOptionPane.showMessageDialog(editorDialog,
							"Operation canceled!");
					return;
				}
			}
		}
		BufferedWriter w;
		try {
			w = new BufferedWriter(new FileWriter(file));
			if (normalize)
				Exporter.cvsCellExport(currentTag, currentMarkables, w,
						caller.getNormfactor(), caller.getAnnotator());
			else
				Exporter.cvsCellExport(currentTag, currentMarkables, w, 0,
						caller.getAnnotator());
			w.close();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Error during export process");
			e.printStackTrace();
			return;
		}
	}

	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int col) {
		currentMarkables = (Vector<Markable>) caller.getSchema().getValueAt(row, col);
		currentTag = caller.getSchema().getTagAtPosition(row, col);
		tableCellButton.setText(new String("selected"));
		return tableCellButton;
	}

	public Object getCellEditorValue() {
		return currentMarkables;
	}

	//
	public void renderCell() {
		Vector<Object> dataVector = new Vector<Object>();
		int normFact = 0;
		if (normalized)
			normFact = caller.getNormfactor();

		Iterator<Markable> it = currentMarkables.iterator();
		while (it.hasNext()) {
			Markable m = it.next();
			Vector<Markable> mv = new Vector<Markable>();
			mv.add(m);
			double d = caller.getAnnotator().getDistributionFor(mv,
					currentTag, normFact);
			Vector<Object> v = new Vector<Object>();
			v.add(m.toString());
			v.add(Double.valueOf(d));
			dataVector.add(v);
		}
		Vector<String> colIdent = new Vector<String>();
		colIdent.add(new String("markables"));
		colIdent.add(new String("count"));
		cellContentTableModel.setDataVector(dataVector, colIdent);
		cellContentTableModel.fireTableStructureChanged();
		return;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (VIEW_CELL.equals(command)) {
			// user clicks on cell: launch cell editor
			tableCellButton.setText("selected");
			renderCell();

			String title = "distribution for: ";
			title = title + currentTag.toString();
			editorDialog.setTitle(title);
			// caller.dialog.setVisible(false);
			caller.getDialog().setVisible(false);
			editorDialog.setVisible(true);
		} else if (CLOSE.equals(command)) {
			dispose();
		} else if (NORMALIZE.equals(command)) {
			normalized = !normalized;
			renderCell();
			setLabel();
		} else if (EXPORT.equals(command)) {
			export(false);
			editorDialog.requestFocus();
			this.normalizeButton.requestFocus();
			//editorDialog.setVisible(true);
		} else if (NORM_EXPORT.equals(command)) {
			export(true);
			editorDialog.setVisible(true);
		}

	}
}

