/*
 * Created on Apr 6, 2005
 *
 */
package org.sfb441.tradisc.eval;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.annotation.Annotator;
import org.sfb441.tradisc.schema.SchemaModel;
import org.sfb441.tradisc.Markable;

/**
 * @author wmaier and malisi
 * 
 */
public class EvaluationTable implements ActionListener {

	private GUI caller;

	private final static String CLOSE = "close";

	private final static String UPDATE = "update";

	private final static String COMB_LOAD = "load evaluation data to combine";

	private final static String COMB_TOGGLE = "toggle Combination view";

	private final static String NORMALIZE = "normalize";

	private static final String SETFACTOR = "set normalization factor";

	private static final String EXPORT = "export";

	private static final String MARKABLE_CHANGED = "markable changed";

	private static final String M_CHOOSER = "show markable chooser dialog";

	private static final String M_TOGGLE = "toggle selected markables evaluation";

	private JDialog evaluationDialog;

	private JPanel tablePanel;

	private JPanel controlPanel;

	private JTable evalTable;

	private SchemaModel schema;

	private EvaluationTableCellRenderer cellRenderer;

	private int normfactor;

	private JLabel normalizedLabel;

	private JButton setNormFactorButton;

	private NormFactorInputDialog normfinput;

	private JButton exportButton;

	private JButton closeWindowButton;

	private JCheckBox normalizeCheckbox;

	private JCheckBox toggleCombinationCheckbox;

	private JCheckBox toggleMarkableSelectionCheckbox;

	private CombinationList combinationList;

	private boolean isCombinedLoaded;

	private JButton loadCombinationButton;

	private MarkableChooser markableChooser;

	private JButton chooseMarkablesButton;

	private boolean allMarkables;

	public EvaluationTable(GUI caller) {
		this.caller = caller;
		evaluationDialog = new JDialog(caller.getGuiFrame(), "evaluation", true);
		evaluationDialog.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		evaluationDialog.setLocationByPlatform(true);
		evaluationDialog.setResizable(true);
		evaluationDialog.setSize(400, 200);
		startEvaluationTools();
		// combineEvals = new Hashtable();
		evaluationDialog.setContentPane(createContentPane());
		normfinput = new NormFactorInputDialog(evaluationDialog);
		normfinput.setLocationRelativeTo(null);
		isCombinedLoaded = false;
		allMarkables = true;
		combinationList = new CombinationList(this);
		markableChooser = new MarkableChooser(this);
	}

	private void startEvaluationTools() {
		schema = new SchemaModel();
		normfactor = 1000;
		evalTable = new JTable(schema);
		evalTable.setDragEnabled(false);
		evalTable.getTableHeader().setReorderingAllowed(false);
		evalTable.changeSelection(0, 1, false, false);
		evalTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		evalTable.setCellSelectionEnabled(true);
		// daten
		cellRenderer = new EvaluationTableCellRenderer(this, false);
		evalTable.setDefaultRenderer(Vector.class, cellRenderer);
		// zeilennamen
		evalTable.setDefaultRenderer(String.class,
				new DefaultTableCellRenderer() {
					public void setValue(Object value) {
						setOpaque(true);
						setBackground(Color.LIGHT_GRAY);
						setText((String) value);
					}
				});
		evalTable.setDefaultEditor(Vector.class, new EvaluationCellViewer(this));
	}

	private void setLabel() {
		String s = "";
		if (!cellRenderer.isNormalized()) {
			s += "not normalized.";
			if (cellRenderer.isViewCombined()) {
				s += " combined evaluation based on "
						+ combinationList.getGlobalTokenCount();
			} else {
				s += " corpus contains "
						+ Integer.toString(caller.getAnnotator().tokenCount());
			}
			s += " words.";
		} else {
			s += "normalized to " + Integer.toString(normfactor) + " words.";
		}
		normalizedLabel.setText(s);
	}

	private JPanel createContentPane() {
		// normalized label
		normalizedLabel = new JLabel("");
		setLabel();
		JPanel normalizedPanel = new JPanel();
		normalizedPanel.setBorder(BorderFactory.createEmptyBorder());
		normalizedPanel.add(normalizedLabel);

		// table
		tablePanel = new JPanel(new BorderLayout());
		evalTable.setPreferredScrollableViewportSize(new Dimension(500, 130));
		JScrollPane tableSp = new JScrollPane(evalTable);
		tablePanel.add(tableSp);
		// controls
		loadCombinationButton = new JButton("manage evaluation files");
		loadCombinationButton.setActionCommand(COMB_LOAD);
		loadCombinationButton.addActionListener(this);
		setNormFactorButton = new JButton(SETFACTOR);
		setNormFactorButton.setActionCommand(SETFACTOR);
		setNormFactorButton.addActionListener(this);
		exportButton = new JButton(EXPORT);
		exportButton.setActionCommand(EXPORT);
		exportButton.addActionListener(this);
		closeWindowButton = new JButton(CLOSE);
		closeWindowButton.setActionCommand(CLOSE);
		closeWindowButton.addActionListener(this);
		chooseMarkablesButton = new JButton("select markables");
		chooseMarkablesButton.setActionCommand(M_CHOOSER);
		chooseMarkablesButton.addActionListener(this);

		// create panels
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(BorderFactory.createTitledBorder("controls"));

		buttonPanel.add(setNormFactorButton);
		buttonPanel.add(exportButton);
		buttonPanel.add(loadCombinationButton);
		buttonPanel.add(chooseMarkablesButton);
		buttonPanel.add(closeWindowButton);

		normalizeCheckbox = new JCheckBox("normalized");
		normalizeCheckbox.setActionCommand(NORMALIZE);
		normalizeCheckbox.addActionListener(this);
		toggleCombinationCheckbox = new JCheckBox("combined with loaded tables");
		toggleCombinationCheckbox.setActionCommand(COMB_TOGGLE);
		toggleCombinationCheckbox.addActionListener(this);
		toggleMarkableSelectionCheckbox = new JCheckBox(
				"only selected markables");
		toggleMarkableSelectionCheckbox.setActionCommand(M_TOGGLE);
		toggleMarkableSelectionCheckbox.addActionListener(this);
		JPanel checkPane = new JPanel();
		checkPane.setBorder(BorderFactory
				.createTitledBorder("evaluation display options"));
		checkPane.add(normalizeCheckbox);
		checkPane.add(toggleCombinationCheckbox);
		checkPane.add(this.toggleMarkableSelectionCheckbox);

		controlPanel = new JPanel();
		controlPanel.setLayout(new BorderLayout());
		controlPanel.add(checkPane, BorderLayout.NORTH);
		controlPanel.add(buttonPanel, BorderLayout.CENTER);

		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(normalizedPanel, BorderLayout.NORTH);
		contentPane.add(tablePanel, BorderLayout.CENTER);
		contentPane.add(controlPanel, BorderLayout.SOUTH);
		return contentPane;
	}

	public void show() {
		setLabel();
		update();
		loadActiveMarkables();
		evaluationDialog.pack();
		evaluationDialog.setVisible(true);
	}

	public void hide() {
		evaluationDialog.setVisible(false);
	}

	public void update() {
		schema = caller.getSchemaEditor().getSchema();
		evalTable.setModel(schema);
		schema.fireTableStructureChanged();
		setLabel();
		// loadActiveMarkables();
	}

	public void loadActiveMarkables() {
		markableChooser.loadActiveMarkables();
		/*
		 * markableBox.removeAllItems(); Vector v = schema.getAllMarkables();
		 * markableBox.addItem("ALL MARKABLES"); Iterator it = v.iterator();
		 * while(it.hasNext()){ markableBox.addItem(it.next()); }
		 */
	}

	private void export(boolean normalize) {
		// wohin?
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save evaluation table");
		int n = fc.showSaveDialog(evaluationDialog);
		if (n != JFileChooser.APPROVE_OPTION)
			return;
		File file = fc.getSelectedFile();
		boolean exists = file.exists();
		while (exists) {
			n = JOptionPane.showConfirmDialog(evaluationDialog, file.getName()
					+ " already exists. Overwrite?", "Please confirm",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION)
				exists = false;
			else if (n == JOptionPane.NO_OPTION) {
				n = fc.showSaveDialog(evaluationDialog);
				if (n == JFileChooser.APPROVE_OPTION)
					file = fc.getSelectedFile();
				exists = file.exists();
				if (n == JFileChooser.CANCEL_OPTION) {
					JOptionPane.showMessageDialog(evaluationDialog,
							"Operation canceled!");
					return;
				}
			}
		}
		BufferedWriter w;
		try {
			w = new BufferedWriter(new FileWriter(file));
			if (cellRenderer.isViewCombined()) {
				if (normalize)
					Exporter.cvsCombinedExport(schema, allMarkables,
							markableChooser.getSelectedMarkables(), w,
							normfactor, caller.getAnnotator(), combinationList);
				else
					Exporter.cvsCombinedExport(schema, allMarkables,
							markableChooser.getSelectedMarkables(), w, 0,
							caller.getAnnotator(), combinationList);
			} else {
				if (normalize)
					Exporter.cvsExport(schema, allMarkables, markableChooser
							.getSelectedMarkables(), w, normfactor, caller
							.getAnnotator());
				else
					Exporter.cvsExport(schema, allMarkables, markableChooser
							.getSelectedMarkables(), w, 0, caller
							.getAnnotator());
			}
			w.close();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Error during export process");
			e.printStackTrace();
			return;
		}
		JOptionPane.showMessageDialog(null, "Finished!");
	}

	protected void setViewCombined(boolean b) {
		toggleCombinationCheckbox.setSelected(b);
		cellRenderer.setCombined(b);
	}

	public void resetDisplayOptions() {
		markableChooser.clearSelectedMarkables();
		normalizeCheckbox.setSelected(false);
		if (cellRenderer.isNormalized())
			cellRenderer.toggleNormalized();
		combinationList.clear();
		setViewCombined(false);
		update();
	}

	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		System.err.println("[EvaluationManager] " + cmd);
		if (CLOSE.equals(cmd)) {
			hide();
		} else if (UPDATE.equals(cmd)) {
			update();
		} else if (SETFACTOR.equals(cmd)) {
			normfinput.setVisible(true);
			int temp = normfinput.getValidatedData();
			if (temp > 0)
				normfactor = temp;
			else {
				JOptionPane.showMessageDialog(null,
						"Normalization Factor must be greater than zero.");
			}
			update();
			// normfactor = normfinput.getValidatedData();

		} else if (NORMALIZE.equals(cmd)) {
			cellRenderer.toggleNormalized();
			setLabel();
			update();
		} else if (EXPORT.equals(cmd)) {
			export(cellRenderer.isNormalized());
		} else if (MARKABLE_CHANGED.equals(cmd)) {
			update();
		} else if (COMB_LOAD.equals(cmd)) {
			combinationList.show();
			update();
		} else if (COMB_TOGGLE.equals(cmd)) {
			if (isCombinedLoaded) {
				setViewCombined(!cellRenderer.isViewCombined());
				update();
			} else {
				toggleCombinationCheckbox.setSelected(false);
				JOptionPane.showMessageDialog(null,
						"No evaluation file(s) loaded!", "",
						JOptionPane.WARNING_MESSAGE);
			}
		} else if (M_CHOOSER.equals(cmd)) {
			markableChooser.show();
			if (!markableChooser.isSelectionEmpty()) {
				allMarkables = false;
				toggleMarkableSelectionCheckbox.setSelected(true);
			}
		} else if (M_TOGGLE.equals(cmd)) {
			if (markableChooser.isSelectionEmpty()) {
				toggleMarkableSelectionCheckbox.setSelected(false);
				JOptionPane.showMessageDialog(null, "No markable(s) selected!",
						"", JOptionPane.WARNING_MESSAGE);
				allMarkables = true;
			} else {
				allMarkables = !allMarkables;
				toggleMarkableSelectionCheckbox.setSelected(!allMarkables);
			}
			update();
		}
	}

	protected void selectMarkableSelectionCheckBox(boolean b){
		toggleMarkableSelectionCheckbox.setSelected(b);
	}
	
	protected JDialog getDialog() {
		return evaluationDialog;
	}

	protected SchemaModel getSchema() {		
		return this.schema;
	}

	protected void setAllMarkables(boolean allMarkables) {
		this.allMarkables = allMarkables;
	}

	protected boolean isCombinedLoaded() {
		return isCombinedLoaded;
	}

	protected void setCombinedLoaded(boolean isCombinedLoaded) {
		this.isCombinedLoaded = isCombinedLoaded;
	}

	protected Annotator getAnnotator() {
		return caller.getAnnotator();
	}

	protected int getNormfactor() {
		return normfactor;
	}

	protected boolean isAllMarkables() {
		return allMarkables;
	}

	protected MarkableChooser getMarkableChooser() {
		return markableChooser;
	}

	protected CombinationList getCombinationList() {
		return combinationList;
	}

}
