package org.sfb441.tradisc.eval;

import java.awt.Component;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;

class EvaluationTableCellRenderer extends DefaultTableCellRenderer {

	private boolean normalized;

	private boolean combined;
	
	private EvaluationTable evaluationTable;

	public EvaluationTableCellRenderer(EvaluationTable evaluationTable, boolean normalized) {
		this.evaluationTable = evaluationTable;
		this.normalized = normalized;
		combined = false;
	}

	public void toggleNormalized() {
		normalized = !normalized;
	}

	public void setCombined(boolean b) {
		combined = b;
	}

	public boolean isViewCombined() {
		return combined;
	}

	public boolean isNormalized() {
		return normalized;
	}

	public Component getTableCellRendererComponent(JTable table,
			Object value, boolean isSelected, boolean hasFocus, int row,
			int column) {
		JLabel ret = new JLabel();
		String text = "";
		Tag tag = evaluationTable.getSchema().getTagAtPosition(row, column);
		// String mark = markableBox.getSelectedItem().toString();
		if (!combined) {
			// if(mark.equals("ALL MARKABLES")){
			if (evaluationTable.isAllMarkables()) {
				if (normalized)
					text = Double.toString(evaluationTable.getAnnotator()
							.getTagDistribution(tag, evaluationTable.getNormfactor()));
				else
					text = Double.toString(evaluationTable.getAnnotator()
							.getTagDistribution(tag, 0));
			} else {
				Vector<Markable> v = evaluationTable.getMarkableChooser().getSelectedMarkables();
				if (normalized) {
					// Markable m = schema.getMarkableForString(mark);
					text = Double.toString(evaluationTable.getAnnotator()
							.getDistributionFor(v, tag, evaluationTable.getNormfactor()));
				} else {
					// Markable m = schema.getMarkableForString(mark);
					text = Double.toString(evaluationTable.getAnnotator()
							.getDistributionFor(v, tag, 0));
				}
			}
		} else {
			// if(mark.equals("ALL MARKABLES")){
			if (evaluationTable.isAllMarkables()) {
				if (normalized) {
					// Additionsfromel:
					// X3 = (X1*T1 + ((X2/N2)*N1)*T2 + ... + ((Xn/Nn)*N1)*Tn
					// ) / (T1 + T2 + ... + Tn)

					double val = evaluationTable.getAnnotator().getTagDistribution(
							tag, evaluationTable.getNormfactor());
					Enumeration files = evaluationTable.getCombinationList().getLoadedFiles();
					double globaltokencount = evaluationTable.getAnnotator()
							.tokenCount();
					double res = val * evaluationTable.getAnnotator().tokenCount();
					// normalisieren aller werte nach this.evaluationTable.getNormfactor() und
					// gewichten der werte jeder zelle nach anzahl der
					// tokens der jeweilige tabelle
					while (files.hasMoreElements()) {
						String file = (String) files.nextElement();
						double tokenc = evaluationTable.getCombinationList().getTokenCount(file).doubleValue();
						double normf = evaluationTable.getCombinationList().getNormFactor(file).doubleValue();
						double[][] dat = evaluationTable.getCombinationList().getData(file);
						res += ((dat[row][column - 1] / normf) * evaluationTable.getNormfactor())
								* tokenc;
						globaltokencount += tokenc;
					}
					res /= globaltokencount;
					res = ((double) Math.round(res * 100) / 100);
					text = Double.toString(res);
				} else {
					double val = evaluationTable.getAnnotator().getTagDistribution(
							tag, 0);
					Enumeration files = evaluationTable.getCombinationList().getLoadedFiles();
					double res = val;
					// Da aktuelle Evaluation nicht normalisiert ist,
					// einfach die jeweiligen Zellen zusammenzhlen zum
					// Addieren.
					while (files.hasMoreElements()) {
						String file = (String) files.nextElement();
						double[][] dat = evaluationTable.getCombinationList().getData(file);
						res += dat[row][column - 1];
					}
					res = ((double) Math.round(res * 100) / 100);
					text = Double.toString(res);
				}
			} else {
				Vector<Markable> v = evaluationTable.getMarkableChooser().getSelectedMarkables();
				if (normalized) {
					// Markable m = schema.getMarkableForString(mark);
					double val = evaluationTable.getAnnotator().getDistributionFor(
							v, tag, evaluationTable.getNormfactor());
					Enumeration files = evaluationTable.getCombinationList().getLoadedFiles();
					double globaltokencount = evaluationTable.getAnnotator()
							.tokenCount();
					double res = val * evaluationTable.getAnnotator().tokenCount();
					// normalisieren aller werte nach this.evaluationTable.getNormfactor() und
					// gewichten der werte jeder zelle nach anzahl der
					// tokens der jeweilige tabelle
					while (files.hasMoreElements()) {
						String file = (String) files.nextElement();
						double tokenc = evaluationTable.getCombinationList().getTokenCount(file).doubleValue();
						double normf = evaluationTable.getCombinationList().getNormFactor(file).doubleValue();
						double[][] dat = evaluationTable.getCombinationList().getData(file);
						res += ((dat[row][column - 1] / normf) * evaluationTable.getNormfactor())
								* tokenc;
						globaltokencount += tokenc;
					}
					res /= globaltokencount;
					res = ((double) Math.round(res * 100) / 100);
					text = Double.toString(res);
				} else {
					// Markable m = schema.getMarkableForString(mark);
					double val = evaluationTable.getAnnotator().getDistributionFor(
							v, tag, 0);
					Enumeration files = evaluationTable.getCombinationList().getLoadedFiles();
					double res = val;
					// Da aktuelle Evaluation nicht normalisiert ist,
					// einfach die jeweiligen Zellen zusammenzhlen zum
					// Addieren.
					while (files.hasMoreElements()) {
						String file = (String) files.nextElement();
						double[][] dat = evaluationTable.getCombinationList().getData(file);
						res += dat[row][column - 1];
					}
					res = ((double) Math.round(res * 100) / 100);
					text = Double.toString(res);
				}
			}
		}
		ret.setText(" " + text);
		return ret;
	}
}
