/*
 * Created on 25.07.2005
 */
package org.sfb441.tradisc.eval;

import java.io.IOException;
import java.io.Writer;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Iterator;

import org.sfb441.tradisc.Location;
import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.annotation.Annotator;
import org.sfb441.tradisc.schema.SchemaModel;
import org.sfb441.tradisc.Markable;

/**
 * @author wolfgang
 */
public class Exporter {

	private Exporter() {
	}

	public static void cvsExport(SchemaModel schema, boolean all_markables,
			Vector<Markable> selectedMarkables, Writer w, int normfactor, Annotator a)
			throws IOException {
		int rows = schema.getRowCount();
		int cols = schema.getColumnCount();
		// print col heads
		for (int col = 1; col < cols; ++col) {
			w.write(";" + schema.getColumnName(col));
		}
		w.write("\n");
		for (int row = 0; row < rows; ++row) {
			w.write(schema.getRowName(row));
			for (int col = 1; col < cols; ++col) {
				Tag tag = schema.getTagAtPosition(row, col);
				String text = "";
				if (all_markables)
					text = Double.toString(a
							.getTagDistribution(tag, normfactor));
				else {
					text = Double.toString(a.getDistributionFor(
							selectedMarkables, tag, normfactor));
				}
				w.write(";" + text);
			}
			w.write("\n");
		}

		if (normfactor == 0)
			w.write("normalization factor;" + a.tokenCount() + "\n");
		else
			w.write("normalization factor;" + normfactor + "\n");
		w.write("number of tokens;" + a.tokenCount() + "\n");

	}

	public static void cvsCombinedExport(SchemaModel schema,
			boolean all_markables, Vector<Markable> selectedMarkables, Writer w,
			int normfactor, Annotator a, CombinationList cl)
			throws IOException {
		int rows = schema.getRowCount();
		int cols = schema.getColumnCount();
		// print col heads
		for (int col = 1; col < cols; ++col) {
			w.write(";" + schema.getColumnName(col));
		}
		w.write("\n");
		for (int row = 0; row < rows; ++row) {
			w.write(schema.getRowName(row));
			for (int col = 1; col < cols; ++col) {
				Tag tag = schema.getTagAtPosition(row, col);
				String text = "";
				if (all_markables) {
					if (normfactor != 0) {
						// Additionsfromel:
						// X3 = (X1*T1 + ((X2/N2)*N1)*T2 + ... + ((Xn/Nn)*N1)*Tn
						// ) / (T1 + T2 + ... + Tn)

						double val = a.getTagDistribution(tag, normfactor);
						Enumeration files = cl.getLoadedFiles();
						double globaltokencount = a.tokenCount();
						double res = val * a.tokenCount();
						// normalisieren aller werte nach this.normfactor und
						// gewichten der werte jeder zelle nach anzahl der
						// tokens der jeweilige tabelle
						while (files.hasMoreElements()) {
							String file = (String) files.nextElement();
							double tokenc = (cl.getTokenCount(file))
									.doubleValue();
							double normf = (cl.getNormFactor(file))
									.doubleValue();
							double combvalue = cl.getValueAt(file, row,
									(col - 1)).doubleValue();
							res += ((combvalue / normf) * normfactor) * tokenc;
							globaltokencount += tokenc;
						}
						res /= globaltokencount;
						res = ((double) Math.round(res * 100) / 100);
						text = Double.toString(res);
					} else {
						double val = a.getTagDistribution(tag, 0);
						Enumeration files = cl.getLoadedFiles();
						double res = val;
						// Da aktuelle Evaluation nicht normalisiert ist,
						// einfach die jeweiligen Zellen zusammenzhlen zum
						// Addieren.
						while (files.hasMoreElements()) {
							String file = (String) files.nextElement();
							double combvalue = cl.getValueAt(file, row,
									(col - 1)).doubleValue();
							res += combvalue;
						}
						res = ((double) Math.round(res * 100) / 100);
						text = Double.toString(res);
					}
				} else {
					if (normfactor != 0) {
						double val = a.getDistributionFor(selectedMarkables,
								tag, normfactor);
						Enumeration files = cl.getLoadedFiles();
						double globaltokencount = a.tokenCount();
						double res = val * a.tokenCount();
						// normalisieren aller werte nach this.normfactor und
						// gewichten der werte jeder zelle nach anzahl der
						// tokens der jeweilige tabelle
						while (files.hasMoreElements()) {
							String file = (String) files.nextElement();
							double tokenc = (cl.getTokenCount(file))
									.doubleValue();
							double normf = (cl.getNormFactor(file))
									.doubleValue();
							double combvalue = cl.getValueAt(file, row,
									(col - 1)).doubleValue();
							res += ((combvalue / normf) * normfactor) * tokenc;
							globaltokencount += tokenc;
						}
						res /= globaltokencount;
						res = ((double) Math.round(res * 100) / 100);
						text = Double.toString(res);
					} else {
						double val = a.getDistributionFor(selectedMarkables,
								tag, 0);
						Enumeration files = cl.getLoadedFiles();
						double res = val;
						// Da aktuelle Evaluation nicht normalisiert ist,
						// einfach die jeweiligen Zellen zusammenzhlen zum
						// Addieren.
						while (files.hasMoreElements()) {
							String file = (String) files.nextElement();
							double combvalue = cl.getValueAt(file, row,
									(col - 1)).doubleValue();
							res += combvalue;
						}
						res = ((double) Math.round(res * 100) / 100);
						text = Double.toString(res);
					}
				}
				w.write(";" + text);
			}
			w.write("\n");
		}
		// tatschliche Anzahl der Tokens der zugrundeliegenden Korpora
		int actualCombTokenCount = a.tokenCount();
		// Anzahl der Tokens, auf der die Tabelle basiert (also summe der
		// Normalisierungsfaktoren)
		int tokenSum = a.tokenCount();
		Enumeration e = cl.getLoadedFiles();
		while (e.hasMoreElements()) {
			String file = (String) e.nextElement();
			actualCombTokenCount += cl.getTokenCount(file)
					.intValue();
			tokenSum += cl.getNormFactor(file).intValue();
		}
		if (normfactor == 0)
			w.write("normalization factor;" + tokenSum + "\n");
		else
			w.write("normalization factor;" + normfactor + "\n");
		w.write("number of tokens;" + actualCombTokenCount + "\n");

	}

	public static void cvsCellExport(Tag tag, Vector<Markable> markables, Writer w,
			int normfactor, Annotator a) throws IOException {
		w.write("distribution for: " + tag.toString() + ";\n");
		// print col heads
		w.write("markables;count\n");
		Iterator<Markable> it = markables.iterator();
		while (it.hasNext()) {
			String text = "";
			Markable m = it.next();
			Vector<Markable> v = new Vector<Markable>();
			v.add(m);
			text = Double.toString(a.getDistributionFor(v, tag, normfactor));
			w.write(m.toString() + ";" + text + "\n");
		}
	}

	public static void cvsComplexityDistributionExport(Writer w,
			int windowsize, int steplength, Annotator a, SchemaModel schema)
			throws IOException {
		int start = 0;
		int end = windowsize;
		w.write("start token;end token;complexity score\n");
		if (windowsize > a.tokenCount()) {
			end = a.tokenCount();
		}
		boolean goon = true;
		double score = 0.0;
		while (goon) {
			for (int row = 0; row < schema.getRowCount(); row++) {
				for (int col = 1; col < schema.getColumnCount(); col++) {
					double mult = (col + 1) * (row + 1);
					Tag t = schema.getTagAtPosition(row, col);
					double distr = a.getWindowTagDistribution(t, start, end);
					score += mult * distr;
				}
			}
			score = ((double) Math.round(score * 100.0)) / 100;
			w.write(new String((start + 1) + ";" + end + ";" + score + "\n"));
			score = 0.0;
			start += steplength;
			if (end >= a.tokenCount())
				goon = false;
			if ((end + steplength) > a.tokenCount())
				end = a.tokenCount();
			else
				end += steplength;
		}
	}

	public static void cvsMarkableCountDistributionExport(Writer w,
			int windowsize, int steplength, Annotator a)
			throws IOException {
		int start = 0;
		int end = windowsize;
		w.write("start token;end token;number of annotated markables\n");
		if (windowsize > a.tokenCount()) {
			end = a.tokenCount();
		}
		boolean goon = true;
		int count = 0;
		while (goon) {
			count = a.getWindowMarkableCount(start, end);
			w.write(new String((start + 1) + ";" + end + ";" + count + "\n"));
			start += steplength;
			if (end >= a.tokenCount()){
				goon = false;
			}
			if ((end + steplength) > a.tokenCount()){
				end = a.tokenCount();
			}
			else{
				end += steplength;
			}
		}
	}

	
	public static void cvsJunctogramExport(Writer w, Annotator a,
			SchemaModel schema) throws IOException {
		Hashtable<String, Integer> yvalues = new Hashtable<String, Integer>();
		w.write("dimension;associated y value\n");
		for (int i = 1; i < schema.getColumnCount(); i++) {
			w.write(schema.getColumnName(i) + ";" + i + "\n");
			yvalues.put(schema.getColumnName(i), Integer.valueOf(i));
		}
		for (int i = 0; i < schema.getRowCount(); i++) {
			w.write(schema.getRowName(i) + ";" + (-i - 1) + "\n");
			yvalues.put(schema.getRowName(i), Integer.valueOf(-i - 1));
		}

		w.write("\n\n");
		w.write("token number;token;vertical dimension value;horizontal dimension value\n");

		Vector<Location> tokens = a.getTokenLocations();
		Iterator<Location> it = tokens.iterator();
		String displayAttribute = a.getInputFormat().getTokenDisplayAttr();
		while (it.hasNext()) {
			Location l = it.next();
			String displayAttrValue = (Markable.element2markable(l.node)).getAttributeValue(displayAttribute);
			if (l.tag != null && !l.tag.isNonMarkableTag()) {
				w.write((a.getTokenNumberFor(l) + 1) + ";");				
				w.write(displayAttrValue + ";");
				w.write(yvalues.get(l.tag.getVerticalDim()) + ";");
				w.write(yvalues.get(l.tag.getHorizontalDim()) + "\n");								
			}else{
				w.write((a.getTokenNumberFor(l) + 1) +";" + displayAttrValue + ";0;0\n");
			}
		}
	}

	public static void cvsJunctogramSelectedExport(Writer w, Annotator a,
			SchemaModel schema, Vector selectedMarkables) throws IOException {
		Hashtable<String, Integer> yvalues = new Hashtable<String, Integer>();
		w.write("dimension;associated y value\n");
		for (int i = 1; i < schema.getColumnCount(); i++) {
			w.write(schema.getColumnName(i) + ";" + i + "\n");
			yvalues.put(schema.getColumnName(i), Integer.valueOf(i));
		}
		for (int i = 0; i < schema.getRowCount(); i++) {
			w.write(schema.getRowName(i) + ";" + (-i - 1) + "\n");
			yvalues.put(schema.getRowName(i), Integer.valueOf(-i - 1));
		}

		w.write("\n\n");
		w.write("token number;vertical dimension value;horizontal dimension value\n");
		Vector<Location> tokens = a.getTokenLocations();
		Iterator<Location> it = tokens.iterator();
		String displayAttribute = a.getInputFormat().getTokenDisplayAttr();
		while (it.hasNext()) {
			Location l = it.next();
			Markable m = Markable.element2markable(l.node);
			Iterator selIt = selectedMarkables.iterator();
			boolean selected = false;
			while(selIt.hasNext()){
				Markable selM = (Markable)selIt.next();
				if(m.isCompatibleWith(selM)){
					selected = true;
					break;
				}
			}			
			String displayAttrValue = (Markable.element2markable(l.node)).getAttributeValue(displayAttribute);
			if (l.tag != null && selected && !l.tag.isNonMarkableTag()){									
				w.write((a.getTokenNumberFor(l)+1) + ";");
				w.write(displayAttrValue + ";");
				w.write(yvalues.get(l.tag.getVerticalDim()) + ";");
				w.write(yvalues.get(l.tag.getHorizontalDim()) + "\n");
			} else {
				w.write((a.getTokenNumberFor(l)+1) + ";" + displayAttrValue + ";0;0\n");
			}			 			
		}

	}
}
