package org.sfb441.tradisc.eval;

import java.awt.*;
import java.awt.event.*;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.*;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.annotation.Annotator;
import org.sfb441.tradisc.schema.SchemaModel;

public class GlobalComplexityScorer implements ActionListener {
	private GUI caller;

	private JDialog dialog;

	private JFormattedTextField factorField;

	private JLabel globalResultLabel;

	private JCheckBox normalizeCheckBox;

	private final static String COMPUTE_GLOBAL = "compute global complexity score";

	private int normfactor;

	public GlobalComplexityScorer(GUI caller) {
		this.caller = caller;
		normfactor = 1000;

		dialog = new JDialog(caller.getGuiFrame(), "global complexity score",
				true);
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		// verhindern, dass jemand den JDialog einfach so zumacht
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				dispose();
			}
		});
		dialog.setLocationRelativeTo(null);
		dialog.setResizable(true);
		dialog.setContentPane(createContentPane());
		dialog.pack();

	}

	protected JPanel createContentPane() {
		JPanel contentPane = new JPanel(new BorderLayout());
		JPanel optionPanel = new JPanel(new BorderLayout());

		JLabel factorLabel = new JLabel("normalization factor:");
		factorField = new JFormattedTextField();
		factorField.setValue(new Integer(normfactor));
		
		NumberFormat integerFormat =  NumberFormat.getIntegerInstance();
		NumberFormatter integerFormatter =
	        new NumberFormatter(integerFormat) {
				public String valueToString(Object o)
					throws ParseException {
			        Integer number = (Integer)o;
			        if(number == null){
			        	return "";
			        }
			        String s = super.valueToString(o);
			        s = s.replaceAll("\\.", " ");
			        return s;
			    }
			    public Object stringToValue(String s)
			           throws ParseException {
			    	try{			    	
			    		s = s.replaceAll(" ", "");
			    		Integer i = Integer.parseInt(s);
			    		if( i.intValue() == 0){
			    			throw new NumberFormatException("Number has to be greater than zero");
			    		} else if(i.intValue() < 0){
			    			factorField.setValue(new Integer(Math.abs(i)));
			    			i =Math.abs(i);
			    		}	
			    		normfactor = i;
			    		if(normalizeCheckBox.isSelected()){
			    			computeGlobalComplexityScore();
			    		}
			    		return i;
			    	}catch(NumberFormatException e){
			    		e.printStackTrace();
			    		throw new ParseException(e.getMessage(), 0);
			    	}
			        
			    }
			};
		integerFormatter.setAllowsInvalid(false);
		factorField.setFormatterFactory(new DefaultFormatterFactory(integerFormatter));
		//factorField = new JFormattedTextField( new DefaultFormatterFactory(integerFormatter));										
		factorField.setColumns(6);
		factorField.setValue(new Integer(normfactor));
		JPanel p = new JPanel();
		p.add(factorLabel);
		p.add(factorField);
		normalizeCheckBox = new JCheckBox("normalize score", true);
		normalizeCheckBox.addActionListener(this);
		normalizeCheckBox.setActionCommand(COMPUTE_GLOBAL);
		optionPanel.add(p, BorderLayout.NORTH);
		optionPanel.add(normalizeCheckBox, BorderLayout.CENTER);

		JPanel resPanel = new JPanel(new BorderLayout());
		resPanel.setBorder(BorderFactory.createTitledBorder("result"));
		globalResultLabel = new JLabel();
		resPanel.add(globalResultLabel, BorderLayout.NORTH);
		contentPane.add(optionPanel, BorderLayout.NORTH);
		contentPane.add(resPanel, BorderLayout.CENTER);
		return contentPane;
	}

	public void dispose() {
		dialog.setVisible(false);
	}

	public void computeGlobalComplexityScore() {
		Annotator ann = caller.getAnnotator();
		SchemaModel schema = caller.getSchemaEditor().getSchema();
		double score = 0.0;
		ComplexityTable ctable = caller.getComplexityTable();
		for (int row = 0; row < schema.getRowCount(); row++) {
			for (int col = 1; col < schema.getColumnCount(); col++) {
				double mult = ((Double) ctable.getComplexityModel().getValueAt(
						row, col)).doubleValue();
				Tag t = schema.getTagAtPosition(row, col);
				// System.err.print(mult + "\t");
				int constant = 0;
				if (normalizeCheckBox.isSelected())
					constant = normfactor;
				double dist = ann.getTagDistribution(t, constant);
				score += mult * dist;
			}
			// System.err.println("");
		}
		score = ((double) Math.round(score * 100.0)) / 100;
		System.err.println("complexity score: " + score);
		
		String info = "<html>" + "<center>" + "complexity Score is: <br>"
				+ "<font size=+2>" + score + "</font> <br>";
		if (normalizeCheckBox.isSelected()){
			info = info + "normalized to " + normfactor + " tokens"
					+ "</center></html>";
		} else {
			info = info + "number of tokens: "
					+ caller.getAnnotator().tokenCount() + "</center></html>";
		}

		globalResultLabel.setText(info);
		// globalResultLabel.setSize(100, 100);
		dialog.pack();
		if (!dialog.isVisible())
			dialog.setVisible(true);
		return;
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		if (command.equals(COMPUTE_GLOBAL)) {
			computeGlobalComplexityScore();
		}

	}

}
