package org.sfb441.tradisc.eval;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.Markable;
//import org.sfb441.tradisc.schema.*;

public class Junctogram implements ActionListener {

	private JDialog dialog;
	private GUI gui;
	
	private ButtonGroup buttonGroup;
	private JRadioButton allButton;
	
	private DefaultListModel cmListModel;
	private DefaultListModel allListModel;


	private JList chosenMarkablesList;
	private JList allMarkablesList;

	private static final String ADD = "add selected markable(s) to list";
	private static final String REMOVE = "remove selected markable(s) from list";
	private static final String CANCEL = "cancel & close junctogram dialog";
	private static final String EXPORT = "export junctogram";
	private static final String ALL = "use all markables for junctogram";
	private static final String SELECTED = "use selected markables for junctogram";
	

	
	public Junctogram(GUI gui) {
		this.gui = gui;
		cmListModel = new DefaultListModel();
		chosenMarkablesList = new JList(cmListModel);
		chosenMarkablesList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		chosenMarkablesList.setLayoutOrientation(JList.VERTICAL);
		chosenMarkablesList.setVisibleRowCount(-1);
		allListModel = new DefaultListModel();
		allMarkablesList = new JList(allListModel);
		allMarkablesList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		allMarkablesList.setLayoutOrientation(JList.VERTICAL);
		allMarkablesList.setVisibleRowCount(-1);
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BorderLayout());

		JPanel radioPane = new JPanel();
	    allButton = new JRadioButton("all markables");
	    allButton.setActionCommand(ALL);
	    JRadioButton selectedButton = new JRadioButton("markables selected below:");
	    selectedButton.setActionCommand(SELECTED);
	    buttonGroup = new ButtonGroup();
	    buttonGroup.add(allButton);
	    buttonGroup.add(selectedButton);
	    allButton.setSelected(true);
	    radioPane.add(allButton);
	    radioPane.add(selectedButton);
	    radioPane.setBorder(BorderFactory.createTitledBorder("which markables to use for junctogram?"));
	    contentPane.add(radioPane, BorderLayout.NORTH);
	    
		JScrollPane chosenListScroller = new JScrollPane(
				chosenMarkablesList);
		chosenListScroller.setPreferredSize(new Dimension(300, 250));
		JScrollPane allListScroller = new JScrollPane(allMarkablesList);
		allListScroller.setPreferredSize(new Dimension(250, 250));
		JButton removeButton = new JButton("remove markable(s)");
		removeButton.setActionCommand(REMOVE);
		removeButton.addActionListener(this);
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		JPanel p = new JPanel(new BorderLayout());
		JLabel l = new JLabel("all markables");
		JButton addButton = new JButton("add markable(s)");
		addButton.setActionCommand(ADD);
		addButton.addActionListener(this);
		p.add(l, BorderLayout.NORTH);
		p.add(allListScroller, BorderLayout.CENTER);
		p.add(addButton, BorderLayout.SOUTH);
		splitPane.setLeftComponent(p);
		p = new JPanel(new BorderLayout());
		l = new JLabel("selected markables");
		p.add(l, BorderLayout.NORTH);
		p.add(chosenListScroller, BorderLayout.CENTER);
		p.add(removeButton, BorderLayout.SOUTH);
		splitPane.setRightComponent(p);
		contentPane.add(splitPane, BorderLayout.CENTER);

		JButton okb = new JButton("export");
		okb.setActionCommand(EXPORT);
		okb.addActionListener(this);
		JButton cancelb = new JButton("cancel");
		cancelb.setActionCommand(CANCEL);
		cancelb.addActionListener(this);
		p = new JPanel();
		p.add(okb);
		p.add(cancelb);
		contentPane.add(p, BorderLayout.SOUTH);
		dialog = new JDialog(gui.getGuiFrame(), "markables to include in junctogram", true);
		dialog.setContentPane(contentPane);
		dialog.setLocationRelativeTo(gui.getGuiFrame());
		dialog.pack();
	}

	
	public void show() {
		loadActiveMarkables();
		dialog.setVisible(true);
	}

	public Vector<Markable> getSelectedMarkables() {
		Enumeration e = cmListModel.elements();
		Vector<Markable> v = new Vector<Markable>();
		while (e.hasMoreElements())
			
			v.add((Markable)e.nextElement());
		return v;
	}

	public void loadActiveMarkables() {
		Vector v = gui.getSchemaEditor().getSchema().getAllMarkables();
		allListModel.clear();
		Iterator it = v.iterator();
		while (it.hasNext())
			allListModel.addElement(it.next());
		dialog.pack();
	}

	public void clearSelectedMarkables() {
		cmListModel.clear();
		chosenMarkablesList.setModel(cmListModel);
		dialog.pack();
//		all_markables = true;
//		toggleMarkableSelectionCheckbox.setSelected(false);
	}

	public boolean isSelectionEmpty() {
		return cmListModel.isEmpty();
	}

	private void exportJunctogram() {
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("save junctogram data");
		int n = fc.showSaveDialog(dialog);
		if (n != JFileChooser.APPROVE_OPTION)
			return;
		File file = fc.getSelectedFile();
		boolean exists = file.exists();
		while (exists) {
			n = JOptionPane.showConfirmDialog(dialog, file.getName()
					+ " already exists. Overwrite?", "Please confirm",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION)
				exists = false;
			else if (n == JOptionPane.NO_OPTION) {
				n = fc.showSaveDialog(dialog);
				if (n == JFileChooser.APPROVE_OPTION)
					file = fc.getSelectedFile();
				exists = file.exists();
				if (n == JFileChooser.CANCEL_OPTION) {
					JOptionPane.showMessageDialog(dialog,
							"Operation canceled!");
					return;
				}
			}
		}
		BufferedWriter w;
		try {
			Vector markables = getSelectedMarkables();
			w = new BufferedWriter(new FileWriter(file));
			if(allButton.isSelected())
				Exporter.cvsJunctogramExport(w, gui.getAnnotator(), gui.getSchemaEditor().getSchema());
			else				
				Exporter.cvsJunctogramSelectedExport(w, gui.getAnnotator(), gui.getSchemaEditor().getSchema(),  markables);					
			w.close();
		} catch (IOException e) {
			// TODO fehlgeschlagen hinweis
			e.printStackTrace();
		}

	}
	
	
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		if (CANCEL.equals(cmd)) {
			clearSelectedMarkables();
			allButton.setSelected(true);			
			dialog.setVisible(false);
		} else if (ADD.equals(cmd)) {
			Object[] selection = allMarkablesList.getSelectedValues();
			for (int i = 0; i < selection.length; i++)
				if (!cmListModel.contains(selection[i]))
					cmListModel.addElement(selection[i]);
		} else if (REMOVE.equals(cmd)) {
			Object[] selection = chosenMarkablesList.getSelectedValues();
			for (int i = 0; i < selection.length; i++)
				cmListModel.removeElement(selection[i]);
		} else if(EXPORT.equals(cmd)){
			if(!allButton.isSelected() && getSelectedMarkables().isEmpty()){
				JOptionPane.showMessageDialog(dialog, "<html>Please select some markables <br> or choose option \"all markables\"</html>",
				"Markable list is empty", JOptionPane.WARNING_MESSAGE);
				return;
			}
			dialog.setVisible(false);
			exportJunctogram();
			allButton.setSelected(true);
			clearSelectedMarkables();
		}
	}
}
	
	
	



