package org.sfb441.tradisc.eval;

import java.awt.*;
import java.awt.event.*;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.*;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.annotation.Annotator;

public class MarkableCount implements ActionListener {
	private GUI caller;

	private JDialog dialog;

	private JFormattedTextField factorField;

	private JLabel resultLabel;

	private JCheckBox normalizeCheckBox;

	private final static String COUNT_ANNOTATED_MARKABLES = "count annotated markables";

	private int normfactor;

	public MarkableCount(GUI caller) {
		this.caller = caller;
		normfactor = 1000;

		dialog = new JDialog(caller.getGuiFrame(), "number of annotated markables",
				true);
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		// verhindern, dass jemand den JDialog einfach so zumacht
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				dispose();
			}
		});
		dialog.setLocationRelativeTo(null);
		dialog.setResizable(true);
		dialog.setContentPane(createContentPane());
		dialog.pack();

	}

	protected JPanel createContentPane() {
		JPanel contentPane = new JPanel(new BorderLayout());
		JPanel optionPanel = new JPanel(new BorderLayout());

		JLabel factorLabel = new JLabel("normalization factor:");
		factorField = new JFormattedTextField();
		factorField.setValue(new Integer(normfactor));
		
		NumberFormat integerFormat =  NumberFormat.getIntegerInstance();
		NumberFormatter integerFormatter =
	        new NumberFormatter(integerFormat) {
				public String valueToString(Object o)
					throws ParseException {
			        Integer number = (Integer)o;
			        if(number == null){
			        	return "";
			        }
			        String s = super.valueToString(o);
			        s = s.replaceAll("\\.", " ");
			        return s;
			    }
			    public Object stringToValue(String s)
			           throws ParseException {
			    	try{			    	
			    		s = s.replaceAll(" ", "");
			    		Integer i = Integer.parseInt(s);
			    		if( i.intValue() == 0){
			    			throw new NumberFormatException("Number has to be greater than zero");
			    		} else if(i.intValue() < 0){
			    			factorField.setValue(new Integer(Math.abs(i)));
			    			i =Math.abs(i);
			    		}	
			    		normfactor = i;
			    		countAnnotatedMarkables();
			    		return i;
			    	}catch(NumberFormatException e){
			    		e.printStackTrace();
			    		throw new ParseException(e.getMessage(), 0);
			    	}
			        
			    }
			};
		integerFormatter.setAllowsInvalid(false);
		factorField.setFormatterFactory(new DefaultFormatterFactory(integerFormatter));
		//factorField = new JFormattedTextField( new DefaultFormatterFactory(integerFormatter));										
		factorField.setColumns(6);
		factorField.setValue(new Integer(normfactor));
		JPanel p = new JPanel();
		p.add(factorLabel);
		p.add(factorField);
		normalizeCheckBox = new JCheckBox("normalize count", false);		
		normalizeCheckBox.addActionListener(this);
		normalizeCheckBox.setActionCommand(COUNT_ANNOTATED_MARKABLES);
		optionPanel.add(p, BorderLayout.NORTH);
		optionPanel.add(normalizeCheckBox, BorderLayout.CENTER);

		JPanel resPanel = new JPanel(new BorderLayout());
		resPanel.setBorder(BorderFactory.createTitledBorder("result"));
		resultLabel = new JLabel();
		resPanel.add(resultLabel, BorderLayout.NORTH);
		contentPane.add(optionPanel, BorderLayout.NORTH);
		contentPane.add(resPanel, BorderLayout.CENTER);
		return contentPane;
	}

	public void dispose() {
		dialog.setVisible(false);
	}

	public void countAnnotatedMarkables() {
		Annotator ann = caller.getAnnotator();
		double count = 0.0;
		count = ann.countAnnotatedMarkables();
		System.err.println("count: " + count);
		int corpuslength = caller.getAnnotator().tokenCount();
		if (normalizeCheckBox.isSelected()){
			count = (count / corpuslength) * normfactor;
			System.err.println("normcount: " + count);
			count = ((double) Math.round(count * 100.0)) / 100;
		}				
		System.err.println("markable count: " + count);
		
		String info = "<html>" + "<center>" + "Number of annotated markables is: <br>"
				+ "<font size=+2>" + count + "</font> <br>";
		if (normalizeCheckBox.isSelected()){
			info = info + "Corpus length normalized to " + normfactor + " tokens"
					+ "</center></html>";
		}else{
			info = info + "Corpus length: "	+ corpuslength 
			         + " tokens" + "</center></html>";
		}
		resultLabel.setText(info);
		dialog.pack();
		if (!dialog.isVisible())
			dialog.setVisible(true);
		return;
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		if (command.equals(COUNT_ANNOTATED_MARKABLES)) {
			countAnnotatedMarkables();
		}

	}

}
