package org.sfb441.tradisc.eval;

import java.awt.*;
import java.awt.event.*;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.SpringLayout;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.SwingUtilities;

public class MarkableCountDistribution implements ActionListener {
	private GUI caller;

	private JDialog parameterDialog;

	private JDialog dialog;

	private JButton setParametersButton;

	private JTextField sizeField;

	private JTextField stepField;

	private final static String SET_PARAM = "compute distribution of annotated markables";

	private final static String EXPORT = "Export distribution of annotated markables";

	private final static String PARAM_CHANGE = "Change parameters";

	private final static String CLOSE = "close window";

	private final static String CANCEL_PARAM = "cancel parameter change";

	private JLabel infoLabel;

	private int windowsize;

	private int steplength;

	private JTable table;

	private DefaultTableModel tableModel;

	public MarkableCountDistribution(GUI caller) {
		this.caller = caller;
		windowsize = 100;
		steplength = 100;
		dialog = new JDialog(caller.getGuiFrame(), "markable count distribution", true);
		dialog.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		dialog.setLocationRelativeTo(null);
		dialog.setResizable(true);
		dialog.setSize(200, 300);
		dialog.setContentPane(createContentPane());
		parameterDialog = new JDialog(dialog, "markable count distribution", true);
		// create dialog frame
		parameterDialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		// verhindern, dass jemand den JDialog einfach so zumacht
		parameterDialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				dispose();
			}
		});
		parameterDialog.setLocationRelativeTo(null);
		parameterDialog.setResizable(true);
		parameterDialog.setContentPane(createParameterPane());
		parameterDialog.pack();

	}

	protected JPanel createContentPane() {
		JPanel contentPane = new JPanel(new BorderLayout());

		// buttons
		JButton b = new JButton("export");
		b.setActionCommand(EXPORT);
		b.addActionListener(this);
		JButton c = new JButton("change parameters");
		c.setActionCommand(PARAM_CHANGE);
		c.addActionListener(this);
		JPanel p = new JPanel(new GridLayout(2, 1));
		// panel for the buttons
		JPanel buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.setBorder(BorderFactory.createTitledBorder("settings"));
		p.add(b);
		p.add(c);
		infoLabel = new JLabel(getInfoLabel());
		buttonPanel.add(p, BorderLayout.NORTH);
		buttonPanel.add(infoLabel, BorderLayout.CENTER);
		tableModel = new DefaultTableModel();
		table = new JTable(tableModel);
		table.setPreferredScrollableViewportSize(new Dimension(250, 200));
		JScrollPane scrollPane = new JScrollPane(table);

		// "close editor" button with its panel
		JButton closeButton = new JButton("close");
		closeButton.setActionCommand(CLOSE);
		closeButton.addActionListener(this);
		JPanel closePane = new JPanel();
		closePane.add(closeButton);

		contentPane.add(scrollPane, BorderLayout.CENTER);
		contentPane.add(buttonPanel, BorderLayout.LINE_END);
		contentPane.add(closePane, BorderLayout.PAGE_END);
		return contentPane;
	}

	protected JPanel createParameterPane() {
		// Create and populate the panel.
		JPanel contentPane = new JPanel(new BorderLayout());
		JPanel p = new JPanel(new SpringLayout());

		JLabel l = new JLabel("set size of sliding window: ", JLabel.TRAILING);
		p.add(l);
		sizeField = new JTextField(5);
		sizeField.setText(Integer.toString(windowsize));
		l.setLabelFor(sizeField);
		p.add(sizeField);
		l = new JLabel("set step length of window: ", JLabel.TRAILING);
		p.add(l);
		stepField = new JTextField(5);
		stepField.setText(Integer.toString(steplength));
		l.setLabelFor(stepField);
		p.add(stepField);
		// Lay out the panel.
		SwingUtilities.makeCompactGrid(p, 2, 2, // rows, cols
				6, 6, // initX, initY
				6, 6); // xPad, yPad
		contentPane.add(p, BorderLayout.NORTH);
		setParametersButton = new JButton("set parameters");
		setParametersButton.addActionListener(this);
		setParametersButton.setActionCommand(SET_PARAM);
		JButton b = new JButton("cancel");
		b.addActionListener(this);
		b.setActionCommand(CANCEL_PARAM);

		p = new JPanel();
		p.add(setParametersButton);
		p.add(b);
		contentPane.add(p, BorderLayout.CENTER);
		return contentPane;
	}

	public void show() {
		computeDistribution();
		dialog.setVisible(true);		
	}

	public void dispose() {
		dialog.setVisible(false);
	}

	public void showParameterDialog() {
		parameterDialog.setVisible(true);
	}

	public void disposeParameterDialog() {
		parameterDialog.setVisible(false);
	}

	private String getInfoLabel() {
		String info = "<html>" + "Current parameters: <br>" + "Windowsize: "
				+ windowsize + " <br>" + "Step length: " + steplength + " <br>"
				+ "</html>";
		return info;
	}

	private void computeDistribution() {
		Vector<Object> dataVector = new Vector<Object>();
		int start = 0;
		int end = windowsize;
		if (windowsize > caller.getAnnotator().tokenCount()) {
			end = caller.getAnnotator().tokenCount();
		}
		boolean goon = true;
		int count = 0;
		while (goon) {
			count = caller.getAnnotator().getWindowMarkableCount(start, end);			
			Vector<Integer> v = new Vector<Integer>();
			v.add(Integer.valueOf(start + 1));
			v.add(Integer.valueOf(end));
			v.add(Integer.valueOf(count));
			dataVector.add(v);
			start += steplength;
			if (start >= caller.getAnnotator().tokenCount()){
				goon = false;
			}
			if (end >= caller.getAnnotator().tokenCount()){
				goon = false;
			}
			if ((end + steplength) > caller.getAnnotator().tokenCount()){
				end = caller.getAnnotator().tokenCount();
			}
			else{
				end += steplength;
			}
		}
		Vector<String> colIdent = new Vector<String>();
		colIdent.add(new String("start token"));
		colIdent.add(new String("end token"));
		colIdent.add(new String("annotated markables"));
		tableModel.setDataVector(dataVector, colIdent);
		tableModel.fireTableStructureChanged();
		dialog.pack();
		return;
	}

	private void export() {
		// wohin?
		JFileChooser fc = new JFileChooser();
		int n = fc.showSaveDialog(dialog);
		if (n != JFileChooser.APPROVE_OPTION)
			return;
		BufferedWriter w;
		try {
			w = new BufferedWriter(new FileWriter(fc.getSelectedFile()));
			Exporter.cvsMarkableCountDistributionExport(w, windowsize, steplength, caller.getAnnotator());
			w.close();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Error during export process");
			e.printStackTrace();
			return;
		}
		JOptionPane.showMessageDialog(null, "Finished!");
	}

	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		System.err.println("[MarkableCountDistribution] Command: " + command);
		if (command.equals(SET_PARAM)) {
			try {
				windowsize = Integer.parseInt(sizeField.getText());
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(caller.getGuiFrame(), new String(
						"Please enter a positive integer number"),
						"Windowsize", JOptionPane.WARNING_MESSAGE);
			}
			try {
				steplength = Integer.parseInt(stepField.getText());
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(caller.getGuiFrame(), new String(
						"Please enter a positive integer number"),
						"Steplength", JOptionPane.WARNING_MESSAGE);
			}
			infoLabel.setText(getInfoLabel());
			computeDistribution();
			disposeParameterDialog();

		} else if (command.equals(PARAM_CHANGE)) {
			showParameterDialog();
		} else if (command.equals(CLOSE)) {
			dispose();
		} else if (command.equals(EXPORT)) {
			export();
		} else if (command.equals(CANCEL_PARAM)) {
			disposeParameterDialog();
			sizeField.setText(Integer.toString(windowsize));
			stepField.setText(Integer.toString(steplength));
		}

	}

}
