package org.sfb441.tradisc.inputformat;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.SwingUtilities;

public class CorpusFormatEditor implements ActionListener {

	private GUI gui;
	
	private JDialog dialog;

	private JTabbedPane tabby;

	private JTextField tokenTagField;

	private JTextField tokenDisplayField;

	private JTextField tokenAttrNameField;

	private JTextField tokenAttrValueField;

	private JTextField otherTagField;

	private JTextField otherDisplayField;

	private JTextField otherAttrNameField;

	private JTextField otherAttrValueField;

	private JLabel currentFormatsLabel;

	private CorpusFormat corpusFormat;

	private Hashtable tokenAttribHash;

	private DefaultTableModel tokenAttribModel;

	private JTable tokenAttribTable;

	private Hashtable otherAttribHash;

	private DefaultTableModel otherAttribModel;

	private JTable otherAttribTable;

	private JButton applyButton;

	private JButton okButton;

	private JButton cancelButton;

	private static final String TOKEN_ATTRIBADD = "add attribute to token format";

	private static final String TOKEN_ATTRIBDELETE = "delete attribute from token format";

	private static final String OTHER_ATTRIBADD = "add attribute to other format";

	private static final String OTHER_ATTRIBDELETE = "delete attribute from other format";

	private static final String OK = "apply changes and exit dialog";

	private static final String APPLY = "apply changes";

	private static final String CANCEL = "discard changes and exit dialog";

	private static final String SETSTANDARD = "set format to TraDisc standard";

	public CorpusFormatEditor(GUI gui) {
		this.gui = gui;
		corpusFormat = new CorpusFormat();
		corpusFormat.setTraDiscStandard();
		tokenAttribHash = new Hashtable();
		otherAttribHash = new Hashtable();
		dialog = new JDialog(gui.getGuiFrame(), "edit corpus input format",
				true);
		dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		dialog.setTitle("edit corpus input format");
		dialog.setContentPane(createContentPane());
		dialog.pack();
	}

	private JPanel createContentPane() {
		JPanel contentPane = new JPanel(new BorderLayout());

		tokenTagField = new JTextField(20);
		tokenDisplayField = new JTextField(20);
		tokenAttrNameField = new JTextField(20);
		tokenAttrValueField = new JTextField(20);
		tokenAttribModel = new DefaultTableModel();
		tokenAttribTable = new JTable(tokenAttribModel);
		otherTagField = new JTextField(20);
		;
		otherDisplayField = new JTextField(20);
		;
		otherAttrNameField = new JTextField(20);
		otherAttrValueField = new JTextField(20);
		otherAttribModel = new DefaultTableModel();
		otherAttribTable = new JTable(otherAttribModel);

		tabby = new JTabbedPane();
		tabby.addTab("tokens", createInputPanel(true));
		tabby.addTab("other elements to display", createInputPanel(false));

		JPanel buttonPane = new JPanel();
		okButton = new JButton("OK");
		okButton.setActionCommand(OK);
		okButton.addActionListener(this);
		applyButton = new JButton("apply changes");
		applyButton.setActionCommand(APPLY);
		applyButton.addActionListener(this);
		cancelButton = new JButton("cancel");
		cancelButton.setActionCommand(CANCEL);
		cancelButton.addActionListener(this);
		buttonPane.add(okButton);
		buttonPane.add(applyButton);
		buttonPane.add(cancelButton);

		currentFormatsLabel = new JLabel("formatofrchterlich");
		JPanel labelPanel = new JPanel();
		labelPanel.add(currentFormatsLabel);
		labelPanel.setBorder(BorderFactory
				.createTitledBorder("current XML input formats"));
		contentPane.add(labelPanel, BorderLayout.NORTH);
		contentPane.add(tabby, BorderLayout.CENTER);
		JPanel p = new JPanel(new BorderLayout());
		p.add(labelPanel, BorderLayout.NORTH);
		p.add(buttonPane, BorderLayout.SOUTH);
		contentPane.add(p, BorderLayout.SOUTH);
		return contentPane;
	}

	private JPanel createInputPanel(boolean forToken) {

		JTextField tagField = tokenTagField;
		JTextField displayField = tokenDisplayField;
		JTextField attrNameField = tokenAttrNameField;
		JTextField attrValueField = tokenAttrValueField;
		DefaultTableModel attribTableModel = tokenAttribModel;
		JTable attributesTable = tokenAttribTable;
		if (!forToken) {
			tagField = otherTagField;
			displayField = otherDisplayField;
			attrNameField = otherAttrNameField;
			attrValueField = otherAttrValueField;
			attribTableModel = otherAttribModel;
			attributesTable = otherAttribTable;
		}

		JPanel inputPanel = new JPanel(new BorderLayout());
		JLabel tagDescr = new JLabel("XML tag name: ");
		JLabel displayDescr = new JLabel("XML attribute to display: ");
		JPanel tagNameAndDisplayPanel = new JPanel(new SpringLayout());
		tagNameAndDisplayPanel.add(tagDescr);
		tagNameAndDisplayPanel.add(tagField);
		tagNameAndDisplayPanel.add(displayDescr);
		tagNameAndDisplayPanel.add(displayField);
		SwingUtilities
				.makeCompactGrid(tagNameAndDisplayPanel, 2, 2, 3, 3, 2, 2);

		// Attributes
		attribTableModel.addColumn("attribute name");
		attribTableModel.addColumn("attribute value");
		attributesTable.setPreferredScrollableViewportSize(new Dimension(300,
				100));
		attributesTable.getTableHeader().setReorderingAllowed(false);
		JScrollPane attribListSp = new JScrollPane(attributesTable);

		JButton addAttributesButton = new JButton("add");
		JButton delAttributesButton = new JButton("remove");

		addAttributesButton.addActionListener(this);
		if (forToken)
			addAttributesButton.setActionCommand(TOKEN_ATTRIBADD);
		else
			addAttributesButton.setActionCommand(OTHER_ATTRIBADD);
		addAttributesButton.setMnemonic(KeyEvent.VK_A);
		delAttributesButton.addActionListener(this);
		if (forToken)
			delAttributesButton.setActionCommand(TOKEN_ATTRIBDELETE);
		else
			delAttributesButton.setActionCommand(OTHER_ATTRIBDELETE);
		addAttributesButton.setMnemonic(KeyEvent.VK_R);

		JPanel attributeButtons = new JPanel();
		attributeButtons.setLayout(new GridLayout(2, 0));
		attributeButtons.add(addAttributesButton);
		attributeButtons.add(delAttributesButton);

		JPanel contentPanelTagAttribsInput1 = new JPanel();
		contentPanelTagAttribsInput1.setBorder(BorderFactory.createEmptyBorder(
				5, 5, 5, 5));
		contentPanelTagAttribsInput1.setLayout(new GridLayout(2, 2));
		contentPanelTagAttribsInput1.add(new JLabel(
				"required XML attributes (name/value pairs):"));
		contentPanelTagAttribsInput1.add(new JLabel(" "));
		contentPanelTagAttribsInput1.add(attrNameField);
		contentPanelTagAttribsInput1.add(attrValueField);
		JPanel contentPanelTagAttribsInput2 = new JPanel();
		contentPanelTagAttribsInput2.setLayout(new BoxLayout(
				contentPanelTagAttribsInput2, BoxLayout.PAGE_AXIS));
		// contentPanelTagAttribsInput2.add(contentPanelTagAttribsInput1);
		contentPanelTagAttribsInput2.add(attribListSp);

		JPanel contentPanelTagAttribsInput = new JPanel();
		contentPanelTagAttribsInput.setLayout(new BoxLayout(
				contentPanelTagAttribsInput, BoxLayout.PAGE_AXIS));
		contentPanelTagAttribsInput.add(contentPanelTagAttribsInput1);
		contentPanelTagAttribsInput.add(contentPanelTagAttribsInput2);

		JPanel contentPanelTagAttribs = new JPanel();
		contentPanelTagAttribs.add(contentPanelTagAttribsInput);
		contentPanelTagAttribs.add(attributeButtons);

		inputPanel.add(tagNameAndDisplayPanel, BorderLayout.NORTH);
		inputPanel.add(contentPanelTagAttribs, BorderLayout.CENTER);
		return inputPanel;
	}

	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		System.err.println("[attribute edit] " + cmd);
		if (TOKEN_ATTRIBADD.equals(cmd)) {
			String an = tokenAttrNameField.getText();
			String av = tokenAttrValueField.getText();
			if (an.equals("")) {
				JOptionPane.showMessageDialog(dialog, "Enter attribute name");
				return;

			} else if (tokenAttribHash.containsKey(an)) {
				JOptionPane.showMessageDialog(dialog,
						"Attribute with this name already exists");
				return;
			}
			Object[] data = { an, av };
			tokenAttribModel.addRow(data);
			tokenAttribHash.put(an, av);
			tokenAttrNameField.setText("");
			tokenAttrValueField.setText("");
		} else if (TOKEN_ATTRIBDELETE.equals(cmd)) {
			if (tokenAttribTable.getRowCount() == 0)
				return;
			else if (tokenAttribTable.getSelectedRowCount() == 0) {
				// TODO messages mit berschrift und warning, auch enstpr. im
				// markableeditor
				JOptionPane.showMessageDialog(dialog,
						"Select attribute to remove");
				return;
			}
			String an = (String) tokenAttribModel.getValueAt(tokenAttribTable
					.getSelectedRow(), 0);
			tokenAttribHash.remove(an);
			tokenAttribModel.removeRow(tokenAttribTable.getSelectedRow());
		} else if (OTHER_ATTRIBADD.equals(cmd)) {
			String an = otherAttrNameField.getText();
			String av = otherAttrValueField.getText();
			if (an.equals("")) {
				JOptionPane.showMessageDialog(dialog, "Enter attribute name");
				return;
			} else if (otherAttribHash.containsKey(an)) {
				JOptionPane.showMessageDialog(dialog,
						"Attribute with this name already exists");
				return;
			}
			Object[] data = { an, av };
			otherAttribModel.addRow(data);
			otherAttribHash.put(an, av);
			otherAttrNameField.setText("");
			otherAttrValueField.setText("");
		} else if (OTHER_ATTRIBDELETE.equals(cmd)) {
			if (otherAttribTable.getRowCount() == 0)
				return;
			else if (otherAttribTable.getSelectedRowCount() == 0) {
				JOptionPane.showMessageDialog(dialog,
						"Select attribute to remove");
				return;
			}
			String an = (String) otherAttribModel.getValueAt(otherAttribTable
					.getSelectedRow(), 0);
			otherAttribHash.remove(an);
			otherAttribModel.removeRow(otherAttribTable.getSelectedRow());
		} else if (OK.equals(cmd)) {
			applyChanges();
			dispose();
		} else if (CANCEL.equals(cmd)) {
			dispose();
		} else if (APPLY.equals(cmd)) {
			applyChanges();
		}
	}

	public CorpusFormat getCorpusFormat() {
		return corpusFormat;
	}

	public void setCorpusFormat(CorpusFormat newFormat){
		corpusFormat = newFormat;
	}
	
	public void show() {
		loadCurrentCorpusFormat();
		dialog.pack();
		dialog.setVisible(true);
	}

	private void applyChanges() {
		// noch ausnahmebehandlung machen: illegale characters entfernen (auf
		// jeden fall " > < = / \ rest vielleicht schauen)
		// und bei leerem string auch meckern
		corpusFormat.setTokenTag(tokenTagField.getText());
		corpusFormat.setTokenDisplayAttr(tokenDisplayField.getText());
		corpusFormat.clearTokenAttributes();
		Enumeration e = tokenAttribHash.keys();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String value = (String) tokenAttribHash.get(name);
			corpusFormat.addTokenAttribute(name, value);
		}
		corpusFormat.setOtherTag(otherTagField.getText());
		corpusFormat.setOtherDisplayAttr(otherDisplayField.getText());
		corpusFormat.clearOtherAttributes();
		e = otherAttribHash.keys();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String value = (String) otherAttribHash.get(name);
			corpusFormat.addOtherAttribute(name, value);
		}
		setCurrentLabel();
	}

	private void loadCurrentCorpusFormat() {
		tokenTagField.setText(corpusFormat.getTokenTag());
		tokenDisplayField.setText(corpusFormat.getTokenDisplayAttr());
		tokenAttribModel.setRowCount(0);
		tokenAttribHash.clear();
		Enumeration e = corpusFormat.getTokenAttributeNames();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String val = corpusFormat.getTokenAttribute(name);
			Object[] data = { name, val };
			tokenAttribModel.addRow(data);
			tokenAttribHash.put(name, val);
		}
		tokenAttribTable.setModel(tokenAttribModel);
		otherTagField.setText(corpusFormat.getOtherTag());
		otherDisplayField.setText(corpusFormat.getOtherDisplayAttr());
		otherAttribModel.setRowCount(0);
		otherAttribHash.clear();
		e = corpusFormat.getOtherAttributeNames();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String val = corpusFormat.getOtherAttribute(name);
			Object[] data = { name, val };
			otherAttribModel.addRow(data);
			otherAttribHash.put(name, val);
		}
		otherAttribTable.setModel(otherAttribModel);
		setCurrentLabel();
	}

	public void setTraDiscStandard() {
		corpusFormat.setTraDiscStandard();
		JOptionPane.showMessageDialog(gui.getGuiFrame(), "TraDisc standard corpus format loaded!");
		setCurrentLabel();
	}

	private void setCurrentLabel() {
		String text = "";
		text = "<html>";
		text += "Token XML format:<br><center>" + "&lt;"
				+ corpusFormat.getTokenTag() + " "
				+ corpusFormat.getTokenDisplayAttr()
				+ "=\"<i>DISPLAY_THIS</i>\" ";
		Enumeration e = corpusFormat.getTokenAttributeNames();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String val = corpusFormat.getTokenAttribute(name);
			text += name + "=\"" + val + "\" ";
		}

		text += "/&gt;<br><br>"
				+ "</center>Other display element XML format:<br><center>"
				+ "&lt;" + corpusFormat.getOtherTag() + " "
				+ corpusFormat.getOtherDisplayAttr()
				+ "=\"<i>DISPLAY_THIS</i>\" ";

		e = corpusFormat.getOtherAttributeNames();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String val = corpusFormat.getOtherAttribute(name);
			text += name + "=\"" + val + "\" ";
		}

		text += "/&gt;<br></center></html>";
		currentFormatsLabel.setText(text);
	}

	private void dispose() {
		tokenAttrNameField.setText("");
		tokenAttrValueField.setText("");
		otherAttrNameField.setText("");
		otherAttrValueField.setText("");
		tabby.setSelectedIndex(0);
		dialog.setVisible(false);
	}

	public void openCorpusFormat(File file) throws FileNotFoundException, Exception{
		CorpusFormatReader reader = new CorpusFormatReader(file);		
		CorpusFormat cf = reader.readCorpusFormat();
		corpusFormat = cf;
	}
	
	public void saveCorpusFormat(File file) throws FileNotFoundException, Exception{
		CorpusFormatWriter writer = new CorpusFormatWriter(file);
		writer.write(this.corpusFormat);
		writer.close();
	}
	
}
