package org.sfb441.tradisc.inputformat;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;

import org.sfb441.tradisc.XMLUtilities;
import org.sfb441.tradisc.schema.SchemaException;
import org.sfb441.tradisc.schema.SchemaModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


public class CorpusFormatReader extends FileReader{

	private File file;
	
	public CorpusFormatReader(File file) throws FileNotFoundException {
		super(file);
		this.file = file;		
	}

	
	public static CorpusFormat parseCorpusFormat(Element inputFormatDocEl) throws Exception{
		CorpusFormat ret = new CorpusFormat();
		NodeList nl = inputFormatDocEl.getElementsByTagName("token");
		if(nl.getLength() != 1)
			throw new Exception("Couldn't load corpus format XML file.");
		Element tokenEl = (Element)nl.item(0);
		String tag = tokenEl.getAttribute("tag");
		if(tag.length() == 0)
			throw new Exception("Couldn't load corpus format XML file.");
		String displ = tokenEl.getAttribute("display");
		if(displ.length() == 0)
			throw new Exception("Couldn't load corpus format XML file.");
		ret.setTokenTag(tag);
		ret.setTokenDisplayAttr(displ);
		
		nl = tokenEl.getElementsByTagName("attribute");
		for(int i = 0; i<nl.getLength();i++){
			Element attribute = (Element)nl.item(i);
			String name = attribute.getAttribute("name");
			String value = attribute.getAttribute("value");
			if(name.length() == 0)
				throw new Exception("Couldn't load corpus format XML file.");
			ret.addTokenAttribute(name, value);
		}
		
		nl = inputFormatDocEl.getElementsByTagName("otherDisplayElements");
		if(nl.getLength() != 1)
			throw new Exception("Couldn't load corpus format XML file.");
		Element otherEl = (Element)nl.item(0);
		tag = otherEl.getAttribute("tag");
		if(tag.length() == 0)
			throw new Exception("Couldn't load corpus format XML file.");
		displ = otherEl.getAttribute("display");
		if(displ.length() == 0)
			throw new Exception("Couldn't load corpus format XML file.");
		ret.setOtherTag(tag);
		ret.setOtherDisplayAttr(displ);
		
		nl = otherEl.getElementsByTagName("attribute");
		for(int i = 0; i<nl.getLength();i++){
			Element attribute = (Element)nl.item(i);
			String name = attribute.getAttribute("name");
			String value = attribute.getAttribute("value");
			if(name.length() == 0)
				throw new Exception("Couldn't load corpus format XML file.");
			ret.addOtherAttribute(name, value);
		}
		return ret;
	}
	
	public CorpusFormat readCorpusFormat() throws Exception {
		Document doc;
		CorpusFormat ret = new CorpusFormat();
		doc = XMLUtilities.parseXMLFile(file, false);
		if (doc == null)
			throw new Exception("Couldn't load XML file.");
		ret = parseCorpusFormat(doc.getDocumentElement());
		return ret;
	}
	
}
