package org.sfb441.tradisc.schema;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.regex.Pattern;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

class DimensionInputDialog extends JDialog implements
		ActionListener, PropertyChangeListener {

	private JOptionPane optionPane;

	private String btnString1 = "enter";

	private String btnString2 = "cancel";

	private JTextField textField;

	private String typedText = null;

	private Pattern emptyStringPattern = Pattern.compile("^\\s*$");

	/** Creates the dialog. */
	public DimensionInputDialog(Frame aFrame) {
		super(aFrame, true);

		this.setTitle("");
		textField = new JTextField(10);

		// das soll angezeigt werden:
		String msgString = "The name of the raible dimension:";
		Object[] array = { msgString, textField };

		// Dialog-Knoepfe + Text
		Object[] options = { btnString1, btnString2 };

		// Create the JOptionPane.
		optionPane = new JOptionPane(array, JOptionPane.QUESTION_MESSAGE,
				JOptionPane.YES_NO_OPTION, null, options, options[0]);

		// dieser Dialog zeigt es an
		setContentPane(optionPane);

		// window-schlieen korrekt hndeln
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
			}
		});

		// textfeld kriegt immer ersten Fokus
		addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent ce) {
				textField.requestFocusInWindow();
			}
		});

		// Eingabe kommt in die OptionPane. Das macht dieser actionlistener.
		textField.addActionListener(this);

		// Jetzt brauchen wir noch einen event handler, der merkt, wenn
		// irgendwas
		// in die OptionPane gestopft wird.
		optionPane.addPropertyChangeListener(this);
	}

	/**
	 * Benutzermitteilung setzen, z.B.: Bitte Daten eingeben:
	 * 
	 * @param message
	 */
	public void setMessage(String message) {
		optionPane.setMessage(new Object[] { message, textField });
	}

	/**
	 * Eingabefeld mit Vorgabe belegen.
	 * 
	 * @param text
	 */
	public void setText(String text) {
		textField.setText(text);
	}

	/**
	 * die validierte Eingabe zurckfordern
	 * 
	 * @return
	 */
	public String getValidatedData() {
		if (typedText == null
				|| emptyStringPattern.matcher(typedText).find()) {
			return null;
		} else {
			return typedText;
		}
	}

	/** This method handles events for the text field. S.o. */
	public void actionPerformed(ActionEvent e) {
		optionPane.setValue(btnString1);
	}

	/**
	 * This method reacts to state changes in the option pane. nur property
	 * change CLOSED kann passieren
	 */
	public void propertyChange(PropertyChangeEvent e) {
		String prop = e.getPropertyName();

		if (isVisible()
				&& (e.getSource() == optionPane)
				&& (JOptionPane.VALUE_PROPERTY.equals(prop) || JOptionPane.INPUT_VALUE_PROPERTY
						.equals(prop))) {
			Object value = optionPane.getValue();

			if (value == JOptionPane.UNINITIALIZED_VALUE) {
				// ignore reset
				return;
			}

			// Reset the JOptionPane's value.
			// If we don't do this, then if the user
			// presses the same button next time, no
			// property change event will be fired.
			optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

			if (btnString1.equals(value)) {
				typedText = textField.getText();
				// we're done; clear and dismiss the dialog
				clearAndHide();
			} else { // user closed dialog or clicked cancel
				typedText = null;
				clearAndHide();
			}
		}
	}

	/** dialog clearen und verbergen */
	public void clearAndHide() {
		textField.setText(null);
		setVisible(false);
	}

}

