package org.sfb441.tradisc.schema;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.AbstractCellEditor;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;

import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;

class SchemaCellEditor extends AbstractCellEditor implements
		TableCellEditor, ActionListener {

	// system
	private SchemaEditor caller;

	// command constants
	protected final static String EDIT_CLOSE = "close";

	protected final static String EDIT_CELL = "editcell";

	protected final static String LIST_DELETE = "listdelete";

	protected final static String LIST_ADD = "listadd";

	protected final static String LIST_EDIT = "listedit";

	// data
	protected Vector<Markable> currentVector;

	protected final JDialog editorDialog;

	private JButton tableCellButton;

	// input
	private JButton addElementButton;

	private JButton editElementButton;

	private JButton deleteElementButton;

	private MarkableInputDialog markableInputDialog;

	// cell content
	private JList cellContentList;

	private DefaultListModel cellContentListModel;

	private Tag currentTag;
	
	// non elements markable.toString() --> Boolean
	private Hashtable<String, Boolean> localNonElementList;

	private Hashtable<String, Markable> nonElementStringToMarkable;

	public SchemaCellEditor(SchemaEditor caller) {
		this.caller = caller;

		// bring up cell "button". Alle Zellen sind ein Knopf.
		// Auf diese Art kann man leichter Ereignisse dranhngen.
		tableCellButton = new JButton();
		tableCellButton.setActionCommand(EDIT_CELL);
		tableCellButton.addActionListener(this);
		tableCellButton.setBorderPainted(false);

		// bring up the editor dialog

		// "close editor" button with its panel
		JButton closeButton = new JButton("close");
		closeButton.setActionCommand(EDIT_CLOSE);
		closeButton.addActionListener(this);
		JPanel buttonPane = new JPanel();
		buttonPane.add(closeButton);

		// lokale non-elements-liste
		localNonElementList = new Hashtable<String, Boolean>();
		nonElementStringToMarkable = new Hashtable<String, Markable>();

		// the list
		cellContentListModel = new DefaultListModel();
		cellContentList = new JList(cellContentListModel);
		cellContentList
				.addListSelectionListener(new CellElementsSelectionListener());
		JScrollPane cellContentScrollPane = new JScrollPane(cellContentList);
		// the buttons
		// delete element: generic, deletes everything
		deleteElementButton = new JButton("delete element");
		deleteElementButton.setMnemonic(KeyEvent.VK_D);
		deleteElementButton.setActionCommand(LIST_DELETE);
		deleteElementButton.addActionListener(this);
		deleteElementButton.setEnabled(false);
		// add an element
		addElementButton = new JButton("add element");
		addElementButton.setMnemonic(KeyEvent.VK_A);
		addElementButton.setActionCommand(LIST_ADD);
		addElementButton.addActionListener(this);
		// edit an element
		editElementButton = new JButton("edit element");
		editElementButton.setMnemonic(KeyEvent.VK_E);
		editElementButton.setActionCommand(LIST_EDIT);
		editElementButton.addActionListener(this);
		editElementButton.setEnabled(false);

		// panel for the buttons
		JPanel modPanel = new JPanel();
		modPanel.setLayout(new GridLayout(4, 1));
		modPanel.setBorder(BorderFactory.createTitledBorder("edit"));
		modPanel.add(addElementButton);
		modPanel.add(editElementButton);
		modPanel.add(deleteElementButton);
		modPanel.add(new JLabel(" "));

		// cell contents/cell editing
		markableInputDialog = new MarkableInputDialog(this);
		markableInputDialog.setLocationRelativeTo(caller
				.getSchemaEditorDialog().getParent());
		markableInputDialog.pack();

		// selector box + buttons together
		JPanel editPane = new JPanel();
		editPane.setLayout(new BoxLayout(editPane, BoxLayout.PAGE_AXIS));
		editPane.add(modPanel);

		// stuff everything together
		JPanel dialogContentPane = new JPanel(new BorderLayout());
		dialogContentPane.add(cellContentScrollPane, BorderLayout.CENTER);
		dialogContentPane.add(editPane, BorderLayout.LINE_END);
		dialogContentPane.add(buttonPane, BorderLayout.PAGE_END);

		// the dialog itsself
		editorDialog = new JDialog(caller.getSchemaEditorDialog(),
				"edit cell value", false);
		editorDialog.setLocationRelativeTo(caller.getSchemaEditorDialog());
		editorDialog.setContentPane(dialogContentPane); // content pane
														// setzen
		editorDialog.pack();

		// enable dialog closing via ESC
		Action escapeAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		};
		KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0,
				false);
		editorDialog.getRootPane().getInputMap(
				JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
		editorDialog.getRootPane().getActionMap().put("ESCAPE",
				escapeAction);
	}

	/**
	 * Wenn wir die Zelle fertig editiert haben, knnen wir den Editor
	 * zumachen, mssen aber vorher alle Daten zurck in das Modell kopieren
	 * (d.h. das SchemaModel)
	 */
	public void dispose() {
		// bearbeitete Daten zurck in das Tabellenmodell kopieren
		currentVector.clear();
		currentVector.ensureCapacity(cellContentListModel.size());
		Enumeration enumeration = cellContentListModel.elements();
		while (enumeration.hasMoreElements()){
			currentVector.addElement((Markable)enumeration.nextElement());
		}
		// non-Elements
		caller.getNonElements().putAll(localNonElementList);
		// storeNonElements();

		// notify the renderer, he has to take over again
		fireEditingStopped();
		// hide the dialog

		editorDialog.setVisible(false);
		// show table
		caller.getSchemaEditorDialog().setVisible(true);

	}

	/*
	 * String representation of the cell content. Das erscheint als Text in
	 * der Zelle. Wenn mehr als ein Element an einer Tabellenposition
	 * vorhanden ist, erscheint das erste Element und "...".
	 */
	private String tabLabelText() {
		String s = "";
		if (currentVector != null && currentVector.size() > 0) {
			s = currentVector.elementAt(0).toString();
			if (currentVector.size() > 1)
				s += "...";
		}
		return s;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue() {
		return currentVector;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table,
			Object value, boolean isSelected, int row, int col) {
		
		currentTag = caller.getSchema().getTagAtPosition(row, col);
		currentVector = (Vector<Markable>) value;
		tableCellButton.setText(tabLabelText());
		return tableCellButton;
	}

	/*
	 * non - Elements
	 */
	private void loadNonElements(Vector<Markable> markables) {
		Iterator<Markable> it = markables.iterator();
		while (it.hasNext()) {
			Markable m = it.next();
			localNonElementList.put(m.toString(), new Boolean(caller.getSchema()
					.isNonElement(m.toString())));
			nonElementStringToMarkable.put(m.toString(), m);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (EDIT_CELL.equals(command)) {
			// user clicks on cell: launch cell editor
			tableCellButton.setText(tabLabelText());
			// fill data model of editor
			cellContentListModel.clear();
			Iterator it = currentVector.iterator();
			while (it.hasNext()) {
				cellContentListModel.addElement(it.next());
			}
			loadNonElements(currentVector);
			caller.getSchemaEditorDialog().setVisible(false);
			editorDialog.setTitle("edit markables for \"" + currentTag.getHorizontalDim()
					+ "-" + currentTag.getVerticalDim() + "\"");
			editorDialog.setVisible(true);
			// fireEditingstopped();
			return;
		} else if (EDIT_CLOSE.equals(command)) {
			dispose();
			return;
		}		

		if (LIST_DELETE.equals(command)) {
			// delete one position in the list
			// this.caller.
			int ind = cellContentList.getSelectedIndex();
			cellContentListModel.remove(ind);
			caller.setSchemaChanged(true);
			int size = cellContentListModel.getSize();
			if (size == 0) {
				deleteElementButton.setEnabled(false);
			} else {
				if (ind == cellContentListModel.getSize())
					ind--;
				cellContentList.setSelectedIndex(ind);
				cellContentList.ensureIndexIsVisible(ind);
			}
		} else if (LIST_ADD.equals(command)) {
			markableInputDialog.setNoMarkableCheckBox(true);
			markableInputDialog.showMarkableEditDialog(false);
			Markable m = markableInputDialog.getValidatedMarkable();
			if (m != null) {
				boolean boo = markableInputDialog.isNoMarkableBoxSelected();					
				localNonElementList.put(m.toString(), new Boolean(boo));
				cellContentListModel.addElement(m);
				this.nonElementStringToMarkable.put(m.toString(), m);
				caller.setSchemaChanged(true);
			}
		} else if (LIST_EDIT.equals(command)) {
			int index = cellContentList.getSelectedIndex();
			Markable markableToEdit = (Markable) cellContentListModel
					.getElementAt(index);
			boolean c = localNonElementList.get(markableToEdit
					.toString()).booleanValue();
			String name = markableToEdit.getXmlElementName();				
			Hashtable<String, String> attributes = markableToEdit.getXmlElementAttributes();
			markableInputDialog.loadMarkable(name, attributes, c);
			markableInputDialog.showMarkableEditDialog(true);
			Markable m = markableInputDialog.getValidatedMarkable();
			if (m != null) {
				markableToEdit.setXmlElementName(m.getXmlElementName());					
				markableToEdit.setXmlElementAttributes(m.getXmlElementAttributes());
				boolean b = markableInputDialog.isNoMarkableBoxSelected();
				localNonElementList.put(markableToEdit.toString(),
						new Boolean(b));
				nonElementStringToMarkable.put(markableToEdit.toString(),
						markableToEdit);
				caller.setSchemaChanged(true);
			}
		}
	}

	/**
	 * Enables / disables buttons in the cell editor, according to list
	 * selection
	 */
	public class CellElementsSelectionListener implements
			ListSelectionListener {

		public void valueChanged(ListSelectionEvent e) {
			if (e.getValueIsAdjusting() == false) {
				if (cellContentList.getSelectedIndex() == -1) {
					deleteElementButton.setEnabled(false);
					editElementButton.setEnabled(false);
				} else {
					deleteElementButton.setEnabled(true);
					editElementButton.setEnabled(true);
				}
			}
		}

	}

	protected SchemaEditor getSchemaEditor() {
		return caller;
	}

	protected Enumeration getCellContentElements(){
		return cellContentListModel.elements();
	}

}
