package org.sfb441.tradisc.schema;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.*;
import javax.swing.table.*;

import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.Markable;
import java.util.*;

public class SchemaEditor implements ActionListener {

	private JDialog dialog;

	// the calling GUI
	private GUI gui;

	// control
	private boolean dimensionsFixed;

	private JLabel instrLabel;

	// schema
	private SchemaModel schema;

	private boolean isSchemaChanged;

	private Hashtable<String, Boolean> nonElements;

	// table data
	private JTable dimensionTable;

	private DimensionInputDialog raibleDimensionInputDialog;

	// action constants
	static protected final String QUIT = "quit";

	static protected final String EDIT_COL_CAT = "editcolcat";

	static protected final String EDIT_ROW_CAT = "editrowcat";

	static protected final String ADD_ROW = "saddrow";

	static protected final String ADD_COL = "addcol";

	static protected final String INSERT_ROW = "insertrow";

	static protected final String INSERT_COL = "insertcol";

	static protected final String DEL_COL = "delcol";

	static protected final String DEL_ROW = "delrow";
	
	static protected final String OVERVIEW = "show schema overview";

	/**
	 * Konstruktor. Der SchemaEditor ist ein JDialog.
	 * 
	 * @param caller
	 */
	public SchemaEditor(GUI caller) {
		dialog = new JDialog(caller.getGuiFrame(), "schema editor", true);

		// set caller id and shortcut to schema
		this.gui = caller;

		// schema
		schema = new SchemaModel();
		schema.fillSchemaWithSampleData();
		nonElements = new Hashtable<String, Boolean>();

		// create dialog frame
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		// verhindern, dass jemand den JDialog einfach so zumacht
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				dispose();
			}
		});
		dialog.setLocationRelativeTo(null);
		dialog.setResizable(true);
		dialog.setContentPane(createContentPane());
		dialog.pack();

		// create dialog for table heads.
		// packen, aber noch nicht anzeigen. Er wird angezeigt,
		// wenn jemand den Namen eines Zeilen/Spaltenkopfs ndern will
		raibleDimensionInputDialog = new DimensionInputDialog(gui
				.getGuiFrame());
		raibleDimensionInputDialog.setLocationRelativeTo(gui.getGuiFrame());
		raibleDimensionInputDialog.pack();
	}

	/**
	 * gibt den JDialog mit der Tabelle drin zurck. Das braucht eigentlich nur
	 * die hier eingeschlossene Klasse "CellEditor" (wegen der Positionierung)
	 * 
	 * @return
	 */
	public JDialog getSchemaEditorDialog() {
		return dialog;
	}

	/*
	 * Auch ein Helfer, erstellt Bildchen fr die Knpfe
	 */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = SchemaEditor.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		}
		System.err.println("Couldn't find image resource: " + path);
		return null;
	}

	/*
	 * Helfer, macht die content pane voll
	 */
	protected JPanel createContentPane() {

		// Knpfe machen
		ImageIcon ii;

		ii = createImageIcon("images/insertRow.gif");
		JButton insertRowButton = new JButton(ii);
		insertRowButton.setActionCommand(INSERT_ROW);
		insertRowButton.addActionListener(this);
		insertRowButton.setToolTipText("insert row above current position");
		insertRowButton.setEnabled(true);

		ii = createImageIcon("images/delRow.gif");
		JButton delRowButton = new JButton(ii);
		delRowButton.setActionCommand(DEL_ROW);
		delRowButton.addActionListener(this);
		delRowButton.setToolTipText("delete current row");
		delRowButton.setEnabled(true);

		ii = createImageIcon("images/addRow.gif");
		JButton addRowButton = new JButton(ii);
		addRowButton.setActionCommand(ADD_ROW);
		addRowButton.addActionListener(this);
		addRowButton.setToolTipText("add row to the end of the table");
		addRowButton.setEnabled(true);

		ii = createImageIcon("images/text.gif");
		JButton editRowCatButton = new JButton(ii);
		editRowCatButton.setActionCommand(EDIT_ROW_CAT);
		editRowCatButton.addActionListener(this);
		editRowCatButton.setToolTipText("edit row category name");
		editRowCatButton.setEnabled(true);

		ii = createImageIcon("images/insertCol.gif");
		JButton insertColButton = new JButton(ii);
		insertColButton.setActionCommand(INSERT_COL);
		insertColButton.addActionListener(this);
		insertColButton.setToolTipText("insert column above current position");
		insertColButton.setEnabled(true);

		ii = createImageIcon("images/delCol.gif");
		JButton delColButton = new JButton(ii);
		delColButton.setActionCommand(DEL_COL);
		delColButton.addActionListener(this);
		delColButton.setToolTipText("delete current column");
		delColButton.setEnabled(true);

		ii = createImageIcon("images/addCol.gif");
		JButton addColButton = new JButton(ii);
		addColButton.setActionCommand(ADD_COL);
		addColButton.addActionListener(this);
		addColButton.setToolTipText("add column to the end of the table");
		addColButton.setEnabled(true);

		ii = createImageIcon("images/text.gif");
		JButton editColCatButton = new JButton(ii);
		editColCatButton.setActionCommand(EDIT_COL_CAT);
		editColCatButton.addActionListener(this);
		editColCatButton.setToolTipText("edit column category name");
		editColCatButton.setEnabled(true);

		// create panels
		// column features
		JPanel columnPane = new JPanel();
		columnPane.setLayout(new BoxLayout(columnPane, BoxLayout.LINE_AXIS));
		columnPane.setBorder(BorderFactory.createTitledBorder("columns"));
		columnPane.add(editColCatButton);
		columnPane.add(addColButton);
		columnPane.add(insertColButton);
		columnPane.add(delColButton);
		// row features
		JPanel rowPane = new JPanel();
		rowPane.setLayout(new BoxLayout(rowPane, BoxLayout.LINE_AXIS));
		rowPane.setBorder(BorderFactory.createTitledBorder("rows"));
		rowPane.add(editRowCatButton);
		rowPane.add(addRowButton);
		rowPane.add(insertRowButton);
		rowPane.add(delRowButton);

		// "editor quit" Knopf und das Panel dazu
		JButton quitButton = new JButton("quit editor");
		quitButton.setActionCommand(QUIT);
		quitButton.addActionListener(this);
		quitButton.setEnabled(true);
		JButton overviewButton = new JButton("overview");
		overviewButton.setActionCommand(OVERVIEW);
		overviewButton.addActionListener(this);
		overviewButton.setEnabled(true);
		JPanel controlButtonPane = new JPanel();
		controlButtonPane.setLayout(new BoxLayout(controlButtonPane,
				BoxLayout.LINE_AXIS));
		controlButtonPane.setBorder(BorderFactory
				.createTitledBorder("controls"));
		controlButtonPane.add(quitButton);
		controlButtonPane.add(overviewButton);
		// panel fr alle Knpfe
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.LINE_AXIS));
		buttonPane.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
		buttonPane.add(controlButtonPane);
		buttonPane.add(rowPane);
		buttonPane.add(columnPane);

		// Die Tabelle
		// Grundlagen
		dimensionTable = new JTable(this.schema);
		dimensionTable.getTableHeader().setReorderingAllowed(false);
		dimensionTable.setDragEnabled(false);
		// Auswahl von Zellen
		dimensionTable.changeSelection(0, 1, false, false);
		dimensionTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		dimensionTable.setCellSelectionEnabled(true);

		// renderers
		// data
		dimensionTable.setDefaultRenderer(Vector.class,
				new DefaultTableCellRenderer() {
					public void setValue(Object value) {
						Vector cellContent = (Vector) value;
						String text = "";
						// wir machen PUnkte dahinter, wenn mehr als
						// ein einziges Element in der Zelle steht.
						if (cellContent.size() > 0) {
							text = cellContent.elementAt(0).toString();
							if (cellContent.size() > 1) {
								text += "...";
							}

						}
						setText(text);
					}
				});
		// row heads
		dimensionTable.setDefaultRenderer(String.class,
				new DefaultTableCellRenderer() {
					public void setValue(Object value) {
						this.setOpaque(true);
						Color newColor = Color.LIGHT_GRAY;
						setBackground(newColor);
						setText((String) value);
					}
				});

		// content editor
		// wir brauchen nur einen Vektor-Editor. Ein String-Editor fr
		// die erste Spalte ist nicht ntig, dafr haben wir den anderen
		// Editor (RaibleDimension)
		dimensionTable.setDefaultEditor(Vector.class, new SchemaCellEditor(this));

		// wir machen noch ein Label dran
		instrLabel = new JLabel("click on a cell or press space to edit it.");
		instrLabel.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));

		// Tabelle erstellen
		JScrollPane dimensionTableSp = new JScrollPane(dimensionTable);
		dimensionTable.setPreferredScrollableViewportSize(new Dimension(500,
				120));
		JPanel tablePane = new JPanel(new BorderLayout());
		tablePane.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		tablePane.add(instrLabel, BorderLayout.PAGE_START);
		tablePane.add(dimensionTableSp, BorderLayout.CENTER);

		// focus-Eigenschaften: Tabelle kriegt primren Fokus
		dialog.addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent ce) {
				dimensionTable.requestFocusInWindow();
				dimensionTable.changeSelection(0, 1, false, false);
			}
		});

		// dialog schlieen per ESC
		// escape-Action auf Tabelle setzen, weil die Tabelle sonst
		// alle Tasten belegt.
		Action escapeAction2 = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				if (dimensionTable.isEditing()) {
					int row = dimensionTable.getEditingRow();
					int col = dimensionTable.getEditingColumn();
					dimensionTable.getCellEditor(row, col).cancelCellEditing();

				} else
					dispose();
			}
		};
		KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
		dimensionTable.getInputMap(JTable.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT)
				.put(escape, "ESCAPE");
		dimensionTable.getActionMap().put("ESCAPE", escapeAction2);

		// escape-Action auf die Tabelle
		Action escapeAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		};
		dialog.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
				.put(escape, "ESCAPE");
		dialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);

		// jetzt die contentPane erstellen und zurckgeben
		JPanel contentPane = new JPanel(new BorderLayout());
		contentPane.add(tablePane, BorderLayout.CENTER);
		contentPane.add(buttonPane, BorderLayout.PAGE_END);
		contentPane.setOpaque(true);

		return contentPane;
	}


	public void setFixDimensions(boolean b) {
		dimensionsFixed = b;
		dialog.setModal(true);
		if (b)
			instrLabel.setText("dimension changes are not allowed");
		else
			instrLabel.setText("");
	}

	// neues (d.h. leeres) Schema erstellen
	public void newSchema(boolean sampleData) {
		schema = new SchemaModel();
		// wir schreiben ein paar Beispieldaten rein
		if (sampleData)
			schema.fillSchemaWithSampleData();
		setSchema(schema);
		isSchemaChanged = false;
	}

	// schon bestehendes Schema oeffnen
	public void openSchema(File schemaFile) throws FileNotFoundException,
			SchemaException {
		SchemaReader sr = new SchemaReader(schemaFile);
		setSchema(sr.readSchema());
		isSchemaChanged = false;
	}

	public void mergeWithSchema(File schemaFile) throws FileNotFoundException,
	SchemaException {
		SchemaReader sr = new SchemaReader(schemaFile);
		SchemaModel mergeSchema = sr.readSchema();
		for (int i = 0; i < mergeSchema.getColumnCount() - 1; i++) {
			for (int j = 0; j < mergeSchema.getRowCount(); j++) {
				System.err.println(mergeSchema.getValueAt(j, i));
				
			}
			
		}
		
		schema.mergeWith(mergeSchema);
		setSchema(schema);
		isSchemaChanged = true;
	}
	
	
	// schema abspeichern
	public void saveSchema(File schemaFile) throws SchemaException, IOException {
		SchemaWriter out = new SchemaWriter(schemaFile);
		out.write(schema);
		out.close();
		isSchemaChanged = false;
	}

	public boolean isSchemaChanged() {
		return isSchemaChanged;
	}

	/**
	 * Mit dieser Methode kann man von auen (von GUI) ein neues Schema in die
	 * Tabelle laden. Wenn eine schon bestehende Annotation geffnet wird, ist
	 * auch immer ein Schema dabei: eine gespeicherte Annotation besteht aus
	 * Annotationsdaten + Schema, welches dann beim ffnen in den Schema-Editor
	 * geladen wird.
	 * 
	 * @param schema
	 */
	public void setSchema(SchemaModel schema) {
		this.schema = schema;
		dimensionTable.setModel(schema); // Tabellenmodell ist das neue
											// Schema
		schema.fireTableStructureChanged(); // zeichnet die Tabelle neu
	}

	/**
	 * gibt die aktuelle Tabelle zurck
	 * 
	 * @return
	 */
	public SchemaModel getSchema() {
		return schema;
	}

	/**
	 * zeigt den SchemaEditor-Dialog an.
	 */
	public void show() {
		dialog.setVisible(true);
	}

	private void dispose() {
		// store non elements
		Enumeration enumeration = nonElements.keys();
		while (enumeration.hasMoreElements()) {
			String mstr = (String) enumeration.nextElement();
			Markable m = schema.getMarkableForString(mstr);
			boolean b = nonElements.get(mstr).booleanValue();
			if (m != null) {
				System.err.println(m.toString() + ": " + b);
				schema.setNonElement(m, b);
			}
		}
		// if dimensionsFixed, then redo the currently opened annotation with
		// new schema
		if (dimensionsFixed && isSchemaChanged()) {
			gui.reloadAnnotation();
		}
		nonElements = new Hashtable<String, Boolean>();
		setFixDimensions(false);
		dialog.setVisible(false);
		gui.getComplexityTable().ensureCompatibility();
	}

	/* required by ActionListener */
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		System.err.println("[SchemaEditor] Command: " + command);

		dimensionTable.requestFocusInWindow();
	    if(OVERVIEW.equals(command)){
	    	new SchemaOverview(this.gui);

		// WEnn das Schema whrend einer laufenden Annotation gendert wird,
		// drfen die Dimensionen nicht gendert werden.
	    }else if (dimensionsFixed && !QUIT.equals(command)) {
			JOptionPane.showMessageDialog(dialog,
					"You may not edit the Dimensions.");
			return;
		}else if(OVERVIEW.equals(command)){
				new SchemaOverview(this.gui);
		}
		// Edit column category
		else if (EDIT_COL_CAT.equals(command)) {
			// richtige Tabellenzellen ausgewhlt?
			if (!isTableSelectionOK(4))
				return;

			// aktuelle Namen holen
			int selColNum = dimensionTable.getSelectedColumn();
			String selColName = dimensionTable.getColumnName(selColNum);
			if (selColName == null)
				selColName = "";

			int selRowNum = dimensionTable.getSelectedRow();

			// dialog setup
			raibleDimensionInputDialog.setTitle("edit column category");
			raibleDimensionInputDialog.setText(selColName);
			raibleDimensionInputDialog
					.setMessage("change the name of the column:");
			raibleDimensionInputDialog.setVisible(true);
			String r = raibleDimensionInputDialog.getValidatedData();
			if (r != null && schema.validateColumnName(r)) {
				System.err.println("Change col name from " + selColName
						+ " to " + r);
				schema.setColumnHead(r, selColNum);
				// this notifies the SchemaEditor window (repaints table)
				schema.fireTableStructureChanged();
				// this notifies the GUI
				isSchemaChanged = true;
				// re-set selection auf die vorherige Zelle, sonst geht der
				// Fokus glaube ich auf den Spaltenkopf
			} else {
				JOptionPane.showMessageDialog(null,
						"A column with this name already exists in the table");
			}
			dimensionTable.changeSelection(selRowNum, selColNum, false, false);

		} else if (EDIT_ROW_CAT.equals(command)) {
			// analog zu oben *************************************
			if (!isTableSelectionOK(0))
				return;
			// edit column category
			int selRowNum = dimensionTable.getSelectedRow();
			String selRowName = (String) dimensionTable
					.getValueAt(selRowNum, 0);
			if (selRowName == null)
				selRowName = "";

			int selColNum = dimensionTable.getSelectedColumn();

			raibleDimensionInputDialog.setTitle("edit row category");
			raibleDimensionInputDialog.setText(selRowName);
			raibleDimensionInputDialog
					.setMessage("change the name of the row:");
			raibleDimensionInputDialog.setVisible(true);
			String r = raibleDimensionInputDialog.getValidatedData();

			if (r != null && schema.validateRowName(r)) {
				System.err.println("Change row name from " + selRowName
						+ " to " + r);
				schema.setRowHead(r, selRowNum);
				schema.fireTableStructureChanged();
				isSchemaChanged = true;
				dimensionTable.changeSelection(selRowNum, selColNum, false,
						false);
			} else {
				JOptionPane.showMessageDialog(null,
						"A row with this name already exists in the table");
			}
		} else if (ADD_COL.equals(command)) {
			if (!isTableSelectionOK(0))
				return;
			// add a column to the end of the table

			int selRowNum = dimensionTable.getSelectedRow();
			int selColNum = dimensionTable.getSelectedColumn();

			raibleDimensionInputDialog.setTitle("add column");
			raibleDimensionInputDialog
					.setMessage("enter the name of the new column:");
			raibleDimensionInputDialog.setVisible(true);
			String r = raibleDimensionInputDialog.getValidatedData();
			if (r != null && schema.validateColumnName(r)) {
				System.err.println("Adding column named " + r);
				schema.addColumn(r);
				schema.fireTableStructureChanged();
				isSchemaChanged = true;
				dimensionTable.changeSelection(selRowNum, selColNum, false,
						false);
			} else {
				JOptionPane.showMessageDialog(null,
						"A column with this name already exists in the table");
			}
		} else if (ADD_ROW.equals(command)) {
			if (!isTableSelectionOK(0))
				return;
			// add row to the end of the table

			int selRowNum = dimensionTable.getSelectedRow();
			int selColNum = dimensionTable.getSelectedColumn();

			raibleDimensionInputDialog.setTitle("add row");
			raibleDimensionInputDialog
					.setMessage("enter the name of the new row:");
			raibleDimensionInputDialog.setVisible(true);
			String r = raibleDimensionInputDialog.getValidatedData();
			if (r != null && schema.validateRowName(r)) {
				System.err.println("Adding row named " + r);
				schema.addRow(r);
				schema.fireTableStructureChanged();
				isSchemaChanged = true;
				dimensionTable.changeSelection(selRowNum, selColNum, false,
						false);
			} else {
				JOptionPane.showMessageDialog(null,
						"A row with this name already exists in the table");
			}
		} else if (INSERT_COL.equals(command)) {
			if (!isTableSelectionOK(0))
				return;
			// insert a column on the LEFT of the current column

			int selRowNum = dimensionTable.getSelectedRow();
			int selColNum = dimensionTable.getSelectedColumn();

			raibleDimensionInputDialog.setTitle("insert column");
			raibleDimensionInputDialog
					.setMessage("enter the name of the new column:");
			raibleDimensionInputDialog.setVisible(true);
			String r = raibleDimensionInputDialog.getValidatedData();
			if (r != null && schema.validateColumnName(r)) {
				System.err.println("Inserting column named " + r);
				schema.insertColumn(dimensionTable.getSelectedColumn(), r);
				schema.fireTableStructureChanged();
				isSchemaChanged = true;
				dimensionTable.changeSelection(selRowNum, selColNum + 1, false,
						false);
			} else {
				JOptionPane.showMessageDialog(null,
						"A column with this name already exists in the table");
			}
		} else if (INSERT_ROW.equals(command)) {
			if (!isTableSelectionOK(0))
				return;
			// insert a row on top of the current row

			int selRowNum = dimensionTable.getSelectedRow();
			int selColNum = dimensionTable.getSelectedColumn();

			raibleDimensionInputDialog.setTitle("insert row");
			raibleDimensionInputDialog
					.setMessage("type in the name of the new row:");
			raibleDimensionInputDialog.setVisible(true);
			String r = raibleDimensionInputDialog.getValidatedData();
			if (r != null && schema.validateRowName(r)) {
				System.err.println("Inserting row named " + r);
				schema.insertRow(dimensionTable.getSelectedRow(), r);
				schema.fireTableStructureChanged();
				isSchemaChanged = true;
				dimensionTable.changeSelection(selRowNum + 1, selColNum, false,
						false);
			} else {
				JOptionPane.showMessageDialog(null,
						"A row with this name already exists in the table");
			}
		} else if (DEL_COL.equals(command)) {
			if (!isTableSelectionOK(2))
				return;
			// ask first
			Object[] options = { "yes", "no" };
			int n = JOptionPane.showOptionDialog(null,
					"This will delete the selected column forever. Proceed?",
					"Delete Column", JOptionPane.DEFAULT_OPTION,
					JOptionPane.WARNING_MESSAGE, null, options, options[0]);
			if (n == JOptionPane.OK_OPTION) {
				// delete a column. The table has to contain at least one
				// column.
				int selRowNum = dimensionTable.getSelectedRow();
				int selColNum = dimensionTable.getSelectedColumn();
				if (selColNum == dimensionTable.getColumnCount() - 1) {
					--selColNum;
				}

				int sc = dimensionTable.getSelectedColumn();
				if (sc > 0 && dimensionTable.getColumnCount() > 2) {
					schema.delCol(sc);
					schema.fireTableStructureChanged();
					isSchemaChanged = true;
					dimensionTable.changeSelection(selRowNum, selColNum, false,
							false);
				}
			}
		} else if (DEL_ROW.equals(command)) {
			if (!isTableSelectionOK(1))
				return;
			if (dimensionTable.getSelectedRow() < 0
					&& dimensionTable.getSelectedRow() >= dimensionTable
							.getRowCount())
				return;
			// ask first
			Object[] options = { "yes", "no" };
			int n = JOptionPane.showOptionDialog(null,
					"This will delete the selected row forever. Proceed?",
					"Delete Row", JOptionPane.DEFAULT_OPTION,
					JOptionPane.WARNING_MESSAGE, null, options, options[0]);
			if (n == JOptionPane.OK_OPTION) {
				// delete a row. The table has to contain at least one row.
				int selRowNum = dimensionTable.getSelectedRow();
				if (selRowNum == dimensionTable.getRowCount() - 1) {
					--selRowNum;
				}
				int selColNum = dimensionTable.getSelectedColumn();

				int sr = dimensionTable.getSelectedRow();
				if (sr > -1 && dimensionTable.getRowCount() > 1) {
					schema.delRow(sr);
					schema.fireTableStructureChanged();
					isSchemaChanged = true;
					dimensionTable.changeSelection(selRowNum, selColNum, false,
							false);
				}
			}
		} else if (QUIT.equals(command)) {
			dispose();
		}

	}

	// table selection: Helfer, checkt ob die aktuelle Zellauswahl fr
	// einen bestimmten Zweck passend ist.
	private boolean isTableSelectionOK(int forPurpose) {
		if (dimensionTable.getSelectedColumn() >= dimensionTable
				.getColumnCount()
				|| dimensionTable.getSelectedRow() >= dimensionTable
						.getRowCount()) {
			return false;
		}

		boolean ret = false;
		if (forPurpose == 0) { // rename/insert columns/rows
			ret = dimensionTable.getSelectedColumn() > -1
					&& dimensionTable.getSelectedRow() > -1;
		} else if (forPurpose == 1) { // delete rows
			ret = dimensionTable.getSelectedColumn() > -1
					&& dimensionTable.getSelectedRow() > -1
					&& (dimensionTable.getRowCount() - 1) > 0;
		} else if (forPurpose == 2) { // delete cols
			ret = dimensionTable.getSelectedColumn() > 0
					&& dimensionTable.getSelectedRow() > -1
					&& (dimensionTable.getColumnCount() - 1) > 1;
		} else if (forPurpose == 4){ //edit column name
			ret = dimensionTable.getSelectedColumn() > 0
			&& dimensionTable.getSelectedRow() > -1;			
		}
		return ret;
	}

	protected Hashtable<String, Boolean> getNonElements() {
		return nonElements;
	}

	protected void setSchemaChanged(boolean isSchemaChanged) {
		this.isSchemaChanged = isSchemaChanged;
	}

	protected GUI getGui() {
		return gui;
	}


}