package org.sfb441.tradisc.schema;

import java.util.*;

import javax.swing.table.*;

import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;

public class SchemaModel extends AbstractTableModel {

	private Hashtable<String, JunktorTableElement> junktorToPosition;

	private Hashtable<String, Vector<Markable>> positionToJunktor;

	Hashtable<String, Markable> stringToMarkable;

	Hashtable<String, Tag> stringToTag;

	// private Hashtable wildcards;

	private Vector<String> columnHeads;

	private Vector<String> rowHeads;

	public SchemaModel() {
		// rows, cols
		columnHeads = new Vector<String>();
		rowHeads = new Vector<String>();
		junktorToPosition = new Hashtable<String, JunktorTableElement>();
		positionToJunktor = new Hashtable<String, Vector<Markable>>();
		stringToMarkable = new Hashtable<String, Markable>();
		stringToTag = new Hashtable<String, Tag>();
		// wildcards = new Hashtable();
	}

	public SchemaModel(SchemaModel model) {
		// rows, cols
		columnHeads = (Vector<String>) model.columnHeads.clone();
		rowHeads = (Vector<String>) model.rowHeads.clone();
		junktorToPosition = (Hashtable<String, JunktorTableElement>) model.junktorToPosition.clone();
		positionToJunktor = (Hashtable<String, Vector<Markable>>) model.positionToJunktor.clone();
		stringToMarkable = (Hashtable<String, Markable>) model.stringToMarkable.clone();
		stringToTag = (Hashtable<String, Tag>) model.stringToTag.clone();
		// wildcards = (Hashtable) model.wildcards.clone();
	}

	public void fillSchemaWithSampleData() {
		addRow("row 1");
		addRow("row 2");
		addColumn("");
		addColumn("column 1");
		addColumn("column 2");
	}

	protected Vector<String> getNonElements() {
		Vector<String> ret = new Vector<String>();
		Enumeration enumeration = junktorToPosition.keys();
		while (enumeration.hasMoreElements()) {
			String name = (String) enumeration.nextElement();
			if (isNonElement(name)) {
				ret.add(name);
			}
		}
		return ret;
	}

	protected boolean isNonElement(String name) {
		JunktorTableElement jt = junktorToPosition.get(name);
		return jt.nonElement;
	}


	protected void setNonElement(Markable m, boolean b) {
		if (this.contains(m)) {
			JunktorTableElement jt = junktorToPosition.get(m.toString());
			jt.nonElement = b;
			junktorToPosition.put(m.toString(), jt);
		}
	}

	public boolean contains(Markable m) {
		Enumeration e = stringToMarkable.elements();
		while (e.hasMoreElements()) {
			Markable schemaM = (Markable) e.nextElement();
			if (m.isCompatibleWith(schemaM)) {
				return true;
			}
		}
		return false;
	}

	public boolean containsTagforMarkable(Markable m, Tag t) {
		Enumeration enumeration = junktorToPosition.keys();
		while (enumeration.hasMoreElements()) {
			String mtablestr = (String) enumeration.nextElement();
			Markable mtable = stringToMarkable.get(mtablestr);
			if (m.isCompatibleWith(mtable)) {
				JunktorTableElement jt = junktorToPosition.get(mtablestr);
				if (jt.tablePositions.contains(t))
					return true;
				else if (t.isNonMarkableTag() && (jt.nonElement))
					return true;
			}
		}
		return false;
	}

	/**
	 * return a Vector of all Markables that are currently in the Schema sorted
	 * alphabetically (with respect to Markable.toString())
	 */
	public Vector<Markable> getAllMarkables() {
		Enumeration e = stringToMarkable.keys();
		Vector<String> strings = new Vector<String>();
		while (e.hasMoreElements())
			strings.add((String)e.nextElement());
		Object arr[] = strings.toArray();
		Arrays.sort(arr);
		Vector<Markable> ret = new Vector<Markable>();
		for (int i = 0; i < arr.length; i++) {
			ret.add(stringToMarkable.get(arr[i]));
		}
		return ret;
	}

	public Markable getMarkableForString(String text) {
		if (stringToMarkable.containsKey(text))
			return stringToMarkable.get(text);
		else
			return null;
	}

	public Vector<Tag> getTagsFor(Markable m) {
		Enumeration enumeration = junktorToPosition.keys();
		Vector<Tag> tags = new Vector<Tag>();
		JunktorTableElement jt = null;
		boolean isnonel = false;
		// alle markables im schema durchchecken
		while (enumeration.hasMoreElements()) {
			String mtablestr = (String) enumeration.nextElement();
			Markable mtable = stringToMarkable.get(mtablestr);
			// wenn ein passendes gefunden wird, die Tags, die noch nicht
			// vorhanden sind, zum returnvector hinzufgen
			if (m.isCompatibleWith(mtable)) {
				jt = junktorToPosition.get(mtablestr);
				if (jt.nonElement)
					isnonel = true;
				Iterator it = jt.tablePositions.iterator();
				while (it.hasNext()) {
					Tag tag = (Tag) it.next();
					if (!tags.contains(tag))
						tags.add(tag);
				}
			}
		}
		Tag nontag = new Tag();
		nontag = Tag.nonTag();
		if (isnonel)
			tags.add(nontag);
		return tags;
	}

	public Tag getTagAtPosition(int row, int col) {
		if (row < 0 || row > this.getRowCount() - 1 || col < 1
				|| col > this.getColumnCount() - 1)
			return null;
		Tag ret = new Tag();
		String horiz = getRowName(row);
		String vert = getColumnName(col);
		ret.setHorizontalDim(horiz);
		ret.setVerticalDim(vert);
		return ret;
	}

	public boolean isEmpty() {
		return junktorToPosition.isEmpty();
	}

	/* my methods */

	public void delRow(int row) {
		Vector<Markable> delvec = new Vector<Markable>();
		Enumeration enumeration = positionToJunktor.keys();
		String r = getRowName(row);
		System.err.println("Deleting row " + row + " : " + r);
		while (enumeration.hasMoreElements()) {
			String keystr = (String) enumeration.nextElement();
			Tag t = stringToTag.get(keystr);
			if (t.getHorizontalDim().equals(r)) {
				Vector<Markable> v = positionToJunktor.get(t.toString());
				delvec.addAll(v);
			}
			positionToJunktor.remove(t.toString());
		}
		Iterator it = delvec.iterator();
		while (it.hasNext()) {
			Object o1 = it.next();
			System.err.println("[delvec Inhalt]: " + o1.getClass() + "\n");
			Markable m = (Markable) o1;
			JunktorTableElement jt = junktorToPosition
					.get(m.toString());
			Vector<Tag> tablePositions = jt.tablePositions;
			Iterator tit = tablePositions.iterator();
			while (tit.hasNext()) {
				Tag t = (Tag) tit.next();
				if (t.getHorizontalDim().equals(r)) {
					tablePositions.remove(t);
				}
			}
			jt.tablePositions = tablePositions;
		}
		System.err.println(rowHeads);
		rowHeads.removeElementAt(row);
		System.err.println(rowHeads);
	}

	public void delCol(int col) {
		Vector<Markable> delvec = new Vector<Markable>();
		Enumeration enumeration = positionToJunktor.keys();
		String r = getColumnName(col);
		System.err.println("Deleting column " + col + " : " + r);
		while (enumeration.hasMoreElements()) {
			String keystr = (String) enumeration.nextElement();
			Tag t = stringToTag.get(keystr);
			if (t.getVerticalDim().equals(r)) {
				Vector<Markable> v = positionToJunktor.get(t.toString());
				delvec.addAll(v);
			}
			positionToJunktor.remove(t.toString());
		}
		Iterator it = delvec.iterator();
		while (it.hasNext()) {
			Object o1 = it.next();
			System.err.println("[delvec Inhalt]: " + o1.getClass() + "\n");
			Markable m = (Markable) o1;
			JunktorTableElement jt = junktorToPosition.get(m.toString());
			Vector<Tag> tablePositions = jt.tablePositions;

			Vector<Tag> fix = new Vector<Tag>();

			Iterator tit = tablePositions.iterator();
			while (tit.hasNext()) {
				Tag t = (Tag) tit.next();
				if (t.getVerticalDim().equals(r)) {
					fix.add(t);
					// tablePositions.remove(t);
				}
			}

			Iterator fixit = fix.iterator();
			while (fixit.hasNext()) {
				tablePositions.remove(fixit.next());
			}

			jt.tablePositions = tablePositions;
		}
		columnHeads.removeElementAt(col);
	}

	public void addRow(String d) {
		rowHeads.addElement(d);
	}

	public void addColumn(String d) {
		columnHeads.addElement(d);
	}

	public void insertRow(int row, String d) {
		rowHeads.insertElementAt(d, row);
	}

	public void insertColumn(int col, String d) {
		columnHeads.insertElementAt(d, col);
	}

	public boolean validateColumnName(String d) {
		return !columnHeads.contains(d);
	}

	public boolean validateRowName(String d) {
		return !rowHeads.contains(d);
	}

	public void setColumnHead(String d, int col) {
		// change column name
		String olddim = columnHeads.get(col);
		columnHeads.removeElementAt(col);
		columnHeads.insertElementAt(d, col);
		// move data from old column to new one
		Enumeration e = this.positionToJunktor.keys();
		while (e.hasMoreElements()) {
			String tstr = (String) e.nextElement();
			Tag oldTag = stringToTag.get(tstr);
			if (oldTag.getVerticalDim().equals(olddim)) {
				Tag newTag = new Tag();
				newTag.setHorizontalDim(oldTag.getHorizontalDim());
				newTag.setVerticalDim(d);
				Vector<Markable> v = positionToJunktor.get(tstr);
				positionToJunktor.remove(tstr);
				positionToJunktor.put(newTag.toString(), v);
				stringToTag.remove(tstr);
				stringToTag.put(newTag.toString(), newTag);

				Iterator it = v.iterator();
				while (it.hasNext()) {
					Markable m = (Markable) it.next();
					JunktorTableElement jt = junktorToPosition
							.get(m.toString());
					Vector<Tag> posvec = jt.tablePositions;
					posvec.remove(oldTag);
					posvec.add(newTag);
					jt.tablePositions = posvec;
					junktorToPosition.put(m.toString(), jt);
				}
			}
		}
	}

	public void setRowHead(String d, int row) {
		// change row name
		String olddim = rowHeads.get(row);
		rowHeads.removeElementAt(row);
		rowHeads.insertElementAt(d, row);
		// move data from old row to new one
		Enumeration e = this.positionToJunktor.keys();
		while (e.hasMoreElements()) {
			String tstr = (String) e.nextElement();
			Tag oldTag = stringToTag.get(tstr);
			if (oldTag.getHorizontalDim().equals(olddim)) {
				Tag newTag = new Tag();
				newTag.setVerticalDim(oldTag.getVerticalDim());
				newTag.setHorizontalDim(d);
				Vector<Markable> v = positionToJunktor.get(tstr);
				positionToJunktor.remove(tstr);
				positionToJunktor.put(newTag.toString(), v);
				stringToTag.remove(tstr);
				stringToTag.put(newTag.toString(), newTag);

				Iterator it = v.iterator();
				while (it.hasNext()) {
					Markable m = (Markable) it.next();
					JunktorTableElement jt = junktorToPosition
							.get(m.toString());
					Vector<Tag> posvec = jt.tablePositions;
					posvec.remove(oldTag);
					posvec.add(newTag);
					jt.tablePositions = posvec;
					junktorToPosition.put(m.toString(), jt);
				}
			}
		}
	}

	/* from super */
	public String getColumnName(int col) {
		return columnHeads.elementAt(col);
	}

	/* extra */
	public String getRowName(int row) {
		return rowHeads.elementAt(row);
	}

	public Class getColumnClass(int col) {
		if (col == 0)
			return String.class;
		return Vector.class;
		// return getValueAt(0,col).getClass();
	}

	public int getRowCount() {
		return rowHeads.size();
	}

	public int getColumnCount() {
		return columnHeads.size();
	}

	public Object getValueAt(int row, int col) {
		if (col < 1) {
			return getRowName(row);
		} else {
			Tag t = new Tag();
			t.setHorizontalDim(getRowName(row));
			t.setVerticalDim(getColumnName(col));
			if (positionToJunktor.containsKey(t.toString())) {
				Vector<Markable> v = positionToJunktor.get(t.toString());
				Vector<String> strings = new Vector<String>();
				Iterator<Markable> it = v.iterator();
				while (it.hasNext())
					strings.add(it.next().toString());
				Object arr[] = strings.toArray();
				Arrays.sort(arr);
				Vector<Markable> ret = new Vector<Markable>();
				for (int i = 0; i < arr.length; i++) {
					ret.add(stringToMarkable.get(arr[i]));
				}
				return ret;
			} else {
				return new Vector();
			}
		}
	}

	public boolean isCellEmpty(Tag t) {
		if (positionToJunktor.containsKey(t.toString()))
			return false;
		return true;
	}

	public void setValueAt(Object content, int row, int col) {
		// spalte 0: Zeilennamen (row heads)
		if (col == 0 && content instanceof String) {
			rowHeads.set(row, (String) content);
			return;
		}
		Tag t = new Tag();
		t.setHorizontalDim(getRowName(row));
		t.setVerticalDim(getColumnName(col));
		// sicherstellen, das nur markables in content im schema an der position
		// row/col stehen
		Enumeration e = junktorToPosition.keys();
		while (e.hasMoreElements()) {
			String mstr = (String) e.nextElement();
			Markable m = stringToMarkable.get(mstr);
			if (!((Collection) content).contains(m)) {
				JunktorTableElement jt = junktorToPosition
						.get(mstr);
				Vector<Tag> tpos = jt.tablePositions;
				if (tpos.contains(t)) {
					tpos.remove(t);
				}
				// markable is at no position in the schema, so delete it
				if (tpos.size() == 0) {
					junktorToPosition.remove(mstr);
					stringToMarkable.remove(mstr);
				} else {
					jt.tablePositions = tpos;
					junktorToPosition.put(mstr, jt);
				}
			}
		}
		// junktor (Markable.toString()) -> position (JunktorTableElement, d.h.
		// Tags + boolean)
		Iterator contentIter = ((Collection) content).iterator();
		while (contentIter.hasNext()) {
			Markable m = (Markable) contentIter.next();
			JunktorTableElement j = new JunktorTableElement();
			if (junktorToPosition.containsKey(m.toString())) {
				j = junktorToPosition.get(m.toString());
				Vector<Tag> tablepos = j.tablePositions;
				if (!tablepos.contains(t))
					tablepos.add(t);
				j.tablePositions = tablepos;
			} else {
				Vector<Tag> v = new Vector<Tag>();
				v.add(t);
				j.tablePositions = v;
			}
			junktorToPosition.put(m.toString(), j);
			// markable.string() --> markable
			if (!stringToMarkable.contains(m.toString()))
				stringToMarkable.put(m.toString(), m);
		}
		// position (Tag.toString()) -> junktor vector von (Markables)
		positionToJunktor.put(t.toString(), (Vector<Markable>) content);
		if (!stringToTag.contains(t.toString()))
			stringToTag.put(t.toString(), t);
	}

	public boolean isCellEditable(int row, int col) {
		if (col < 1) {
			return false;
		} else {
			return true;
		}
	}

	
	
	/**
	 * merges this SchemaModel with the SchemaModel mergeSchema
	 * 
	 * @param mergeSchema
	 */
	public void mergeWith(SchemaModel mergeSchema) throws SchemaException {
		// sicherstellen, dass die dimensionen der beiden schemas die gleichen
		// sind
		if (!columnHeads.equals(mergeSchema.columnHeads)
				|| !rowHeads.equals(mergeSchema.rowHeads)) {
			throw (new SchemaException("Schema dimensions do not fit!"));
		}

		Enumeration e = mergeSchema.junktorToPosition.keys();
		while (e.hasMoreElements()) {
			String keyStr = (String) e.nextElement();
			JunktorTableElement mergejt = mergeSchema.junktorToPosition
					.get(keyStr);
			if (junktorToPosition.containsKey(keyStr)) {
				JunktorTableElement jt = junktorToPosition
						.get(keyStr);
				jt.nonElement = jt.nonElement || mergejt.nonElement;
				Iterator it = mergejt.tablePositions.iterator();
				while (it.hasNext()) {
					Tag mergeTag = (Tag) it.next();
					if (!jt.tablePositions.contains(mergeTag))
						jt.tablePositions.add(mergeTag);
				}
				junktorToPosition.put(keyStr, jt);
			} else {
				junktorToPosition.put(keyStr, mergeSchema.junktorToPosition
						.get(keyStr));
				stringToMarkable.put(keyStr, mergeSchema.stringToMarkable
						.get(keyStr));
			}
		}
		e = mergeSchema.positionToJunktor.keys();
		while (e.hasMoreElements()) {
			String keyStr = (String) e.nextElement();
			Vector<Markable> mergeMarkables = mergeSchema.positionToJunktor
					.get(keyStr);
			if (positionToJunktor.containsKey(keyStr)) {
				Vector<Markable> markables = positionToJunktor.get(keyStr);
				Iterator it = mergeMarkables.iterator();
				while (it.hasNext()) {
					Markable m = (Markable) it.next();
					if (!markables.contains(m))
						markables.add(m);
				}
				positionToJunktor.put(keyStr, markables);
			} else {
				positionToJunktor.put(keyStr, mergeSchema.positionToJunktor
						.get(keyStr));
				stringToTag.put(keyStr, mergeSchema.stringToTag.get(keyStr));
			}
		}
	}

}
