package org.sfb441.tradisc.schema;

/*
 * A 1.4 application that uses SpringLayout to create a compact grid.
 * Other files required: SpringUtilities.java.
 */

import javax.swing.*;
import javax.swing.filechooser.FileFilter;



import org.sfb441.tradisc.GUI;
import org.sfb441.tradisc.Markable;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import java.util.Vector;

public class SchemaOverview{
	
	
	private GUI gui;
	private JEditorPane textPane;
	private String tokenElements;
	private String displayAttributes;
	private JCheckBox displayBox;
	
	public SchemaOverview(GUI gui){
		this.gui = gui;
        //Create and set up the window.
        final JDialog dialog = new JDialog(gui.getGuiFrame(),true);
        dialog.setTitle( "schema overview");
        dialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
    	JPanel contentPane = new JPanel(new BorderLayout());        

    	//set up button panel
        JPanel buttonPane = new JPanel(); 
        displayBox = new JCheckBox();
        displayBox.setSelected(true);
        displayBox.setText("show only display attribute");
        displayBox.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {				
				updateDisplay(displayBox.isSelected());	
			}
        	
        });
        buttonPane.add(displayBox);
        JButton printButton = new JButton("HTML export");        
        printButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				export();		
			}        	
        });
        //TODO noch printbutton hinzufgen und drucken implementieren oder html-export
        buttonPane.add(printButton);
        JButton closeButton = new JButton("close");        
        closeButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {				
				dialog.dispose();				
			}        	
        });
        buttonPane.add(closeButton);
        
        createTexts();
        textPane = new JEditorPane("text/html", displayAttributes);
        textPane.setEditable(false);    
        JPanel p = new JPanel();
        p.add(textPane);
        //textPane.setPreferredSize(textPane.getSize());
        JScrollPane scroller = new JScrollPane(p);
        contentPane.add(scroller, BorderLayout.CENTER);
        contentPane.add(buttonPane, BorderLayout.SOUTH);
        dialog.setContentPane(contentPane);
		
        //Display the window.
        dialog.pack();
        Dimension dim = scroller.getSize();
        System.err.println(textPane.getScrollableTracksViewportWidth());
        
        scroller.setPreferredSize(new Dimension ((int)Math.min(dim.getWidth(), 950), (int)Math.min(dim.getHeight(), 700)));
        
        dialog.pack();
        dialog.setVisible(true);
	}
		
    
    private void updateDisplay(boolean showOnlyDisplayAttribute) {
    	if(showOnlyDisplayAttribute){
    		textPane.setText(displayAttributes);
    	}else{
    		textPane.setText(tokenElements);
    	}		
	}

    private void createTexts(){
    	SchemaModel schema = gui.getSchemaEditor().getSchema();
    	String displayAtt = gui.getCorpusFormatEditor().getCorpusFormat().getTokenDisplayAttr();    	 

        int rows = schema.getRowCount();
        int cols = schema.getColumnCount();
 
        displayAttributes = tokenElements = "<HTML>\n  <BODY>\n    <table border=\"1\">\n"        
        			+ "      <tr>\n        <td bgcolor=\"#C0C0C0\"></td>\n";
        
        for (int c = 1; c < cols; c++) {
        	displayAttributes += "        <td bgcolor=\"#C0C0C0\">"+ schema.getColumnName(c) + "</td>\n";
        	tokenElements += "        <td bgcolor=\"#C0C0C0\">"+ schema.getColumnName(c) + "</td>\n";
        }        
        displayAttributes += "      </tr>\n";
        tokenElements += "      </tr>\n";
        for (int r = 0; r < rows; r++) {
        	displayAttributes += "      <tr>\n";
        	tokenElements += "      <tr>\n";
            for (int c = 0; c < cols; c++) {            	
            	if(c == 0){
            		displayAttributes +="        <td bgcolor=\"#C0C0C0\">";
            		tokenElements +="         <td bgcolor=\"#C0C0C0\">";
            		displayAttributes += (String)schema.getValueAt(r,c);
            		tokenElements += (String)schema.getValueAt(r,c);
            	}else{
            		displayAttributes += "        <td>";
            		tokenElements += "        <td>";
	                Vector v = (Vector)schema.getValueAt(r,c);
	                
	                for (int i = 0; i<v.size(); ++i){
	                	String tempd = "";
	                	String tempt = "";
                		Markable m = (Markable)v.elementAt(i);
                		if(m.getAttributeValue(displayAtt) != null){
                			tempd = m.getAttributeValue(displayAtt);
	                	}else{
	                		tempd = m.toString();
	                	}
	                	tempt = v.elementAt(i).toString();
	                	tempd = tempd.replaceAll("<", "&lt;");
	                	tempd = tempd.replaceAll(">", "&gt;");
	                	tempt = tempt.replaceAll("<", "&lt;");
	                	tempt = tempt.replaceAll(">", "&gt;");
	                	
	                	displayAttributes += tempd;
	                	tokenElements += tempt;
	                	if(i<v.size()-1){
	                		displayAttributes += "<br>\n            ";
	                		tokenElements += "<br>\n            ";
	                	}
	                }
	                if(v.size() == 0){
	                	displayAttributes +=  "&nbsp;";
	                	tokenElements +=  "&nbsp;";
	                }
            	}
            	displayAttributes +=  "</td>\n";
            	tokenElements +=  "</td>\n";
            }
        displayAttributes += "      </tr>\n";
        tokenElements += "      </tr>\n";
        }
        displayAttributes += "    </table>\n  </BODY>\n</HTML>";
        tokenElements += "    </table>  \n</BODY>\n</HTML>";
    }

    private  String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
    
    private void export() {
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("select export location");
		fc.addChoosableFileFilter(new FileFilter(){			
		    //Accept all directories and html files
		    public boolean accept(File f) {
		        if (f.isDirectory()) {
		            return true;
		        }

		        String extension = getExtension(f);
		        if (extension != null) {
		            if (extension.equals("html") || extension.equals("htm")){
		            	return true;
		            } else {
		                return false;
		            }
		        }
		        return false;
		    }
		    //The description of this filter
		    public String getDescription() {
		        return "HTML Files (*.html, *.htm)";
		    }			
		});
		int n = fc.showSaveDialog(gui.getGuiFrame());
		if (n != JFileChooser.APPROVE_OPTION)
			return;
		File file = fc.getSelectedFile();
		
		if(getExtension(file) == null || !(getExtension(file).equals("html") || getExtension(file).equals("htm"))){
			file = new File(file.getPath()+".html");
		}
		boolean exists = file.exists();
		while (exists) {
			n = JOptionPane.showConfirmDialog(gui.getGuiFrame(), file.getName()
					+ " already exists. Overwrite?", "Please confirm",
					JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION)
				exists = false;
			else if (n == JOptionPane.NO_OPTION) {
				n = fc.showSaveDialog(gui.getGuiFrame());
				if (n == JFileChooser.APPROVE_OPTION){
					file = fc.getSelectedFile();
					if(getExtension(file) == null || !(getExtension(file).equals("html") || getExtension(file).equals("htm"))){
						file = new File(file.getPath()+".html");
					}
				}
				exists = file.exists();
				if (n == JFileChooser.CANCEL_OPTION) {
					JOptionPane.showMessageDialog(gui.getGuiFrame(),
							"Operation canceled!");
					return;
				}
			}
		}
		BufferedWriter writer;
		try {
			writer = new BufferedWriter(new FileWriter(file));
			if(displayBox.isSelected()){
				writer.write(displayAttributes);
			} else{
				writer.write(tokenElements);
			}					
			writer.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }


}


