package org.sfb441.tradisc.schema;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.XMLUtilities;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * SchemaReader
 * 
 * @author Wolfgang
 */
public class SchemaReader extends FileReader {

	private File schemaFile;

	public SchemaReader(File schemaFile) throws FileNotFoundException {
		super(schemaFile);
		this.schemaFile = schemaFile;
	}

	private static Markable element2Markable(Element markableel) {
		String name = markableel.getAttribute("name");
		Hashtable<String, String> attribHash = new Hashtable<String, String>();
		NodeList attribs = markableel
				.getElementsByTagName("attribute");
		for (int attr = 0; attr < attribs.getLength(); ++attr) {
			Element attrel = (Element) attribs.item(attr);
			String n = attrel.getAttribute("name");
			String v = attrel.getAttribute("value");
			attribHash.put(n, v);
		}
		// System.err.println(new Markable(name, value, attribHash).toString());
		return new Markable(name,  attribHash);
	}

	public static SchemaModel parseSchema(Element schemaDocEl)
			throws SchemaException {
		SchemaModel ret = new SchemaModel();
		// nonelements
		Vector<Markable> nonElementVec = new Vector<Markable>();
		NodeList nms = schemaDocEl.getElementsByTagName("nm");
		if (nms.getLength() == 0)
			throw new SchemaException("No non-elements in schema");
		Element nme = (Element) nms.item(0);
		NodeList nonElList = nme.getElementsByTagName("m");
		for (int i = 0; i < nonElList.getLength(); ++i) {
			Element nonel = (Element) nonElList.item(i);
			nonElementVec.add(element2Markable(nonel));
		}
		// spalten holen und fehler checken
		NodeList columnList = schemaDocEl.getElementsByTagName("vertical-dim");
		if (columnList.getLength() == 0)
			throw new SchemaException("No columns in schema.");
		// fehler checken fr zeilen
		NodeList rowTitlesList = ((Element) columnList.item(0))
				.getElementsByTagName("horizontal-dim");
		if (rowTitlesList.getLength() == 0)
			throw new SchemaException("No rows in schema.");

		// tabelle aufbauen
		ret.addColumn("");
		for (int col = 0; col < columnList.getLength(); ++col) {
			Element e = (Element) columnList.item(col);
			ret.addColumn(e.getAttribute("name"));
		}
		for (int row = 0; row < rowTitlesList.getLength(); ++row) {
			Element e = (Element) rowTitlesList.item(row);
			ret.addRow(e.getAttribute("name"));
		}

		// inhalte einfgen
		for (int col = 0; col < columnList.getLength(); ++col) {
			Element colel = (Element) columnList.item(col);
			NodeList rowList = colel.getElementsByTagName("horizontal-dim");
			for (int row = 0; row < rowList.getLength(); ++row) {
				Element rowel = (Element) rowList.item(row);
				NodeList markableList = rowel.getElementsByTagName("m");
				Vector<Markable> markables = new Vector<Markable>();
				for (int markint = 0; markint < markableList.getLength(); ++markint) {
					Element markableel = (Element) markableList.item(markint);
					Markable tm = element2Markable(markableel);
					markables.add(tm);
				}
				if (markables.size() > 0)
					ret.setValueAt(markables, row, col + 1);
				Iterator iter = markables.iterator();
				while (iter.hasNext()) {
					Markable ttm = (Markable) iter.next();
					if (nonElementVec.contains(ttm))
						ret.setNonElement(ttm, true);
				}
			}
		}

		return ret;
	}

	public SchemaModel readSchema() throws SchemaException {
		Document doc;
		SchemaModel ret = new SchemaModel();
		doc = XMLUtilities.parseXMLFile(schemaFile, false);
		if (doc == null)
			throw new SchemaException("Couldn't load XML file.");
		ret = parseSchema(doc.getDocumentElement());
		return ret;
	}

}
