package org.sfb441.tradisc.schema;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import org.sfb441.tradisc.Markable;

public class SchemaWriter extends FileWriter {

	public SchemaWriter(File schemaFile) throws IOException {
		super(schemaFile);
	}

	public void write(SchemaModel schema) throws SchemaException, IOException {
		super.write("<?xml version='1.0' encoding='ISO-8859-1'?>\n");
		schema2xml(schema, this);
	}

	public void write(String s) throws IOException {
		super.write(s);
	}

	public static void schema2xml(SchemaModel schema, Writer w)
			throws SchemaException, IOException {
		w.write("<raibleschema>\n");
		// vertical (rows)
		for (int cols = 1; cols < schema.getColumnCount(); ++cols) {
			String colName = schema.getColumnName(cols);
			w.write("  <vertical-dim name=\"" + colName + "\">");
			for (int rows = 0; rows < schema.getRowCount(); ++rows) {
				String rowName = schema.getRowName(rows);
				w.write("    <horizontal-dim name=\"" + rowName + "\"");
				Vector data = (Vector) schema.getValueAt(rows, cols);
				if (data.size() > 0) {
					w.write(">\n");
					Iterator it = data.iterator();
					while (it.hasNext()) {
						Markable m = (Markable) it.next();
						writeMarkable(m, w);
					}
					w.write("    </horizontal-dim>\n");
				} else {
					w.write(" />\n");
				}
			}
			w.write("  </vertical-dim>\n");
		}
		// jetzt schreiben wir noch die Junktoren auf, die manchmal keine
		// Junktoren sind
		w.write("  <nm>\n");
		Vector nms = schema.getNonElements();
		Iterator nmsIt = nms.iterator();
		while (nmsIt.hasNext()) {
			String mstr = (String) nmsIt.next();
			Markable m = schema.stringToMarkable.get(mstr);
			writeMarkable(m, w);
		}
		w.write("  </nm>\n");
		w.write("</raibleschema>\n");

	}

	private static void writeMarkable(Markable m, Writer w) throws IOException {
		w.write("      <m name=\"" + m.getXmlElementName() + "\"");
		Enumeration enumeration = m.getXmlElementAttributes().keys();
		if (!(enumeration.hasMoreElements())) {
			w.write(" />\n");
		} else {
			w.write(" >\n");
			while (enumeration.hasMoreElements()) {
				String an = (String) enumeration.nextElement();
				String av = m.getAttributeValue(an);
				if(av.equals("&")){
					av = "&amp;";
				}
				w.write("        <attribute name=\"" + an + "\" value=\"" + av
						+ "\" />\n");
			}
			w.write("      </m>\n");
		}
	}
}